//=======================================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : Psm3Sys (PSM3 TEST SYSTEM)
//  FUNCTION      : DDR-SDRAM I/F
//  VERSION       : Ver0.70
//
//                      Copyright (C) 2008 by Tomoyuki Inomoto
//=======================================================================
`timescale 1ns/1ns

//`define     SIM
//`define     SIM2
`define       SYN

`define     DDR_IF_PON     3'h0
`define     DDR_IF_INIT    3'h1
`define     DDR_IF_IDLE    3'h2
`define     DDR_IF_WAIT    3'h3
`define     DDR_IF_REF     3'h4
`define     DDR_IF_COM     3'h5
`define     DDR_IF_WRIT    3'h6
`define     DDR_IF_READ    3'h7

// {RAS,CAS,WE}
`define     CMD_NOP  3'b111
`define     CMD_DSEL 3'b111
`define     CMD_MSET 3'b000
`define     CMD_ACT  3'b011
`define     CMD_PRE  3'b010
`define     CMD_WRIT 3'b100
`define     CMD_READ 3'b101
`define     CMD_REF  3'b001
`define     CMD_BST  3'b110

module DdrIf(
    //======== SYSTEM ==============
    DdrClk          ,
    DdrClkP270      ,
    RstN            ,

    //======= Command Fifo & Read Data Fifo =============
    DdrComFifoEmpty ,
    DdrComFifoEmpty3,
    DdrComFifoRe    ,
    DdrComMstr      ,
    DdrBRST         ,
    DdrRW           ,
    DdrBE           ,
    DdrADR          ,
    DdrWD           ,

    DdrRdFifoWe     ,
    DdrRDMstr       ,
    DdrRD           ,

    //======== SYSTEM ==============
    DDRCK           ,
    DDRNCK          ,
    DDRCKE          ,
    DDRNCS          ,
    DDRRAS          ,
    DDRCAS          ,
    DDRNWE          ,
    DDRBA           ,
    DDRA            ,
    DDRDQS          ,
    DDRDM           ,
    DDRDQ
);
    //==================================================
    //======== port  ===================================
    //==================================================
    //======== SYSTEM ==============
    input           DdrClk          ;
    input           DdrClkP270      ;
    input           RstN            ;

    //======= Command Fifo & Read Data Fifo =============
    input           DdrComFifoEmpty ;
    input           DdrComFifoEmpty3;
    output          DdrComFifoRe    ;
    input   [5:0]   DdrComMstr      ;
    input           DdrBRST         ;
    input           DdrRW           ;
    input   [3:0]   DdrBE           ;
    input   [25:2]  DdrADR          ;
    input   [31:0]  DdrWD           ;

    output          DdrRdFifoWe     ;
    output  [5:0]   DdrRDMstr       ;
    output  [31:0]  DdrRD           ;

    //======== DDR-SDRAM ==============
    output          DDRCK           ;
    output          DDRNCK          ;
    output          DDRCKE          ;
    output          DDRNCS          ;
    output          DDRRAS          ;
    output          DDRCAS          ;
    output          DDRNWE          ;
    output  [1:0]   DDRBA           ;
    output  [12:0]  DDRA            ;
    output  [1:0]   DDRDQS          ;
    output  [1:0]   DDRDM           ;
    inout   [15:0]  DDRDQ           ;

    //==================================================
    //======== wire & reg ==============================
    //==================================================
    reg     [9:0]   RefreshCounter  ;
    reg             RefreshReq      ;
    reg     [14:0]  SubStateCounter ;
    reg     [3:0]   MainState       ;
    reg     [31:0]  DdrWD1          ;
    reg             DDRCKE          ;
    reg             DDRNCS          ;
    reg             DDRRAS          ;
    reg             DDRCAS          ;
    reg             DDRNWE          ;
    reg     [1:0]   DDRBA           ;
    reg     [12:0]  DDRA            ;

    reg             ComValid        ;
    wire            NewComReq0      ;
    wire            NewComReq1      ;
    wire            NewComReq       ;

    wire    [1:0]   Dqs             ;
    wire            DqsH            ;
    reg     [4:0]   DqsHSR          ;
    reg     [1:0]   DqsOE           ;
    reg     [4:0]   DqsOESR         ;
    reg     [15:0]  DqOE            ;
    reg     [3:0]   Dm              ;

    reg     [1:0]   DmHalfDly       ;
    wire    [15:0]  DqW             ;

    reg     [4:0]   DdrRdFifoWeSR   ;
    wire            DdrRdFifoWe     ;
    reg     [5:0]   DdrRDMstr       ;

    wire    [31:0]  DqR             ;
    reg     [31:16] DqRDelay        ;

    wire    [1:0]   BankAdr         ;
    wire    [12:0]  ColAdr          ;
    wire    [12:0]  RowAdr          ;

    reg     [1:0]   LastBankAdr     ;
    reg             LastRW          ; // 0:Read, 1:Write
    reg     [2:0]   IdleCounter     ;
    //====================================================================
    //======== BODY ======================================================
    //====================================================================
    //==================================================
    //======== Internal Signals  =======================
    //==================================================
    //======== BankAdr  =======================
    //assign BankAdr = {DdrADR[18], DdrADR[3]};
    assign BankAdr = DdrADR[5:4];

    //======== ColAdr  =======================
    //assign ColAdr = {3'h1, DdrADR[11:4], 2'b00};
    assign ColAdr = {3'h1, DdrADR[12:6], DdrADR[3:2], 1'b0};

    //======== RowAdr  =======================
    //assign RowAdr = {DdrADR[25:19], DdrADR[17:12]};
    assign RowAdr = DdrADR[25:13];

    //======== DdrComFifoRe  =======================
    always  @(posedge DdrClk or negedge RstN) begin
        if(~RstN)                            ComValid <= 1'b0;
        else if(DdrComFifoEmpty & NewComReq) ComValid <= 1'b0;
        else if(DdrComFifoRe)                ComValid <= 1'b1;
    end
    assign  DdrComFifoRe = ~DdrComFifoEmpty & (~ComValid | (ComValid & NewComReq));
    assign  NewComReq0 = (MainState == `DDR_IF_WRIT) & ((SubStateCounter[2:0] == 3'd5) | (DdrBRST & ((SubStateCounter[2:0] == 3'd2)
                                                                                                   | (SubStateCounter[2:0] == 3'd3)
                                                                                                   | (SubStateCounter[2:0] == 3'd4))));
    assign  NewComReq1 = (MainState == `DDR_IF_READ) & (SubStateCounter[2:0] == 3'd4);
    assign  NewComReq  = NewComReq0 | NewComReq1;

    //======== DdrRdFifoWe  =======================
    always  @(posedge DdrClk or negedge RstN) begin
        if(~RstN) begin
            DdrRdFifoWeSR <= 5'b00000;
        end
        else if((MainState == `DDR_IF_READ) & (SubStateCounter[2:0] == 3'd4)) begin
            if(DdrBRST) DdrRdFifoWeSR   <= 5'b01111;
            else        DdrRdFifoWeSR   <= 5'b01000;
        end
        else begin
            DdrRdFifoWeSR   <= {DdrRdFifoWeSR[3:0], 1'b0};
        end
    end
    assign DdrRdFifoWe = DdrRdFifoWeSR[4];
/*
    always  @(posedge DdrClk or negedge RstN) begin
        if(~RstN) DdrRdFifoWe <= 1'b0;
        else      DdrRdFifoWe <= DdrRdFifoWeSR[4];
    end
*/
    //==================================================
    //======== RefreshCounter    ======================
    //==================================================
    always  @(posedge DdrClk or negedge RstN) begin
        if(~RstN) begin
            RefreshCounter <= 10'h0;
        end
        else if((MainState == `DDR_IF_PON) | (MainState == `DDR_IF_INIT)) begin
            RefreshCounter <= 10'h0;
        end
        else if(RefreshCounter == 10'd700) begin
            RefreshCounter <= 10'h0;
        end
        else begin
            RefreshCounter <= RefreshCounter + 1'b1;
        end
    end
    //======== RefreshReq    ======================
    always  @(posedge DdrClk or negedge RstN) begin
        if(~RstN) begin
            RefreshReq <= 1'h0;
        end
        else if(RefreshCounter == 10'd700) begin
            RefreshReq <= 1'h1;
        end
        else if((MainState == `DDR_IF_REF) & (SubStateCounter[3:0] == 4'h0)) begin
            RefreshReq <= 1'h0;
        end
    end

    //======== IdleCounter    ======================
    always  @(posedge DdrClk or negedge RstN) begin
        if(~RstN)                           IdleCounter <= 3'h0;
        else if(MainState != `DDR_IF_IDLE)  IdleCounter <= 3'h0;
        else if(IdleCounter != 3'd4)        IdleCounter <= IdleCounter + 1'b1;
    end
    //==================================================
    //======== SubStateCounter    ======================
    //==================================================
    always  @(posedge DdrClk or negedge RstN) begin
        if(~RstN) begin
            SubStateCounter <= 15'h0;
        end
`ifdef SYN
        else if((MainState == `DDR_IF_IDLE) | (MainState == `DDR_IF_COM) | (SubStateCounter == 15'd25000)) begin
`endif
`ifdef SIM
        else if((MainState == `DDR_IF_IDLE) | (MainState == `DDR_IF_COM) | (SubStateCounter == 15'd30)) begin
`endif
`ifdef SIM2
        else if((MainState == `DDR_IF_IDLE) | (MainState == `DDR_IF_COM) | (SubStateCounter == 15'd25000)) begin
`endif
            SubStateCounter <= 15'h0;
        end
        else begin
            SubStateCounter <= SubStateCounter + 1'b1;
        end
    end

    //======== DqsH ======================
    always  @(posedge DdrClk or negedge RstN) begin
        if(~RstN) begin
            DqsHSR  <= 5'b00000;
        end
        else if((MainState == `DDR_IF_WRIT) & (SubStateCounter[2:0] == 3'd1)) begin
            DqsHSR  <= 5'b01111;
        end
        else begin
            DqsHSR  <= {DqsHSR[3:0], 1'b0};
        end
    end
    assign  DqsH = DqsHSR[4];
    //======== DqsOE ======================
    always  @(posedge DdrClk or negedge RstN) begin
        if(~RstN) begin
            DqsOESR <= 5'b00000;
        end
        else if((MainState == `DDR_IF_WRIT) & (SubStateCounter[2:0] == 3'd1)) begin
            DqsOESR <= 5'b11111;
        end
        else begin
            DqsOESR <= {DqsOESR[3:0], 1'b0};
        end
    end
    always  @(posedge DdrClk or negedge RstN) begin
        if(~RstN)   DqsOE   <= 2'b00;
        else        DqsOE   <= {2{DqsOESR[4]}};
    end
    //======== DqOE ======================
    always  @(posedge DdrClk or negedge RstN) begin
        if(~RstN)   DqOE    <= 16'h0;
        else        DqOE    <= {16{DqsOESR[4]}};
    end

    //======== Dm ======================
    always  @(posedge DdrClk or negedge RstN) begin
        if(~RstN) begin
            Dm <= 4'b1111;
        end
        else if(DdrBRST) begin
            if(MainState == `DDR_IF_WRIT) begin
                if((SubStateCounter[2:0] == 3'd2) |
                   (SubStateCounter[2:0] == 3'd3) |
                   (SubStateCounter[2:0] == 3'd4) |
                   (SubStateCounter[2:0] == 3'd5)) Dm <= ~DdrBE;
                else                               Dm <= 4'b1111;
            end
            else begin
                Dm <= 4'b1111;
            end
        end
        else begin
            if(MainState == `DDR_IF_WRIT) begin
                if(SubStateCounter[2:0] == 3'd2) Dm <= ~DdrBE;
                else                             Dm <= 4'b1111;
            end
            else begin
                Dm <= 4'b1111;
            end
        end
    end

    //==================================================
    //======== Main State Machine ======================
    //==================================================
    always  @(posedge DdrClk or negedge RstN) begin
        if(~RstN) begin
            MainState   <= `DDR_IF_PON;
            DDRCKE      <= 1'b0;
            DDRNCS      <= 1'b1;
            {DDRRAS, DDRCAS, DDRNWE}    <= `CMD_NOP;
            DDRBA       <= 2'b00;
            DDRA        <= 13'h0;
            DdrWD1      <= 32'h0;
            LastBankAdr <= 2'b00;
            LastRW      <= 1'b0;
            DdrRDMstr   <= 6'h0;
        end
        else begin
            if(MainState == `DDR_IF_PON) begin         //=== Wait 200uS ===============
`ifdef SYN
                if(SubStateCounter == 15'd25000) MainState <= `DDR_IF_INIT;  // Wait 250uS
`endif
`ifdef SIM
                if(SubStateCounter == 15'd30) MainState <= `DDR_IF_INIT;  // FOR SIMULATION
`endif
`ifdef SIM2
                if(SubStateCounter == 15'd25000) MainState <= `DDR_IF_INIT;  // Wait 250uS
`endif
            end
            else if(MainState == `DDR_IF_INIT) begin    //=== Mode Reg Set ===============
                DDRCKE    <= 1'b1;
                if((SubStateCounter[7:0] == 8'd1) | (SubStateCounter[7:0] == 8'd7))begin
                    DDRNCS      <= 1'b0;
                    {DDRRAS, DDRCAS, DDRNWE}    <= `CMD_PRE;
                    DDRBA       <= 2'b00;
                    DDRA        <= 13'h400;
                end
                else if(SubStateCounter[7:0] == 8'd3) begin
                    DDRNCS      <= 1'b0;
                    {DDRRAS, DDRCAS, DDRNWE}    <= `CMD_MSET; // Extend Mode Reg
                    DDRBA       <= 2'b01;
                    DDRA        <= 13'h000;
                    //DDRA      <= 13'h002; //=== Half Drive =========
                end
                else if(SubStateCounter[7:0] == 8'd5) begin
                    DDRNCS      <= 1'b0;
                    {DDRRAS, DDRCAS, DDRNWE}    <= `CMD_MSET; // Mode Reg
                    DDRBA       <= 2'b00;
                    //DDRA        <= 13'h122; // 4BURST 2CAS-LATENCY
                    DDRA        <= 13'h123; // 8BURST 2CAS-LATENCY
                end
                else if((SubStateCounter[7:0] == 8'd9) | (SubStateCounter[7:0] == 8'd17))begin
                    DDRNCS      <= 1'b0;
                    {DDRRAS, DDRCAS, DDRNWE}    <= `CMD_REF;
                end
                else begin
                    DDRNCS      <= 1'b1;
                    {DDRRAS, DDRCAS, DDRNWE}    <= `CMD_NOP;
`ifdef SYN
                    if(SubStateCounter[7:0] == 8'd251) MainState <= `DDR_IF_IDLE;
`endif
`ifdef SIM
                    if(SubStateCounter[7:0] == 8'd10) MainState <= `DDR_IF_IDLE;
`endif
`ifdef SIM2
                    if(SubStateCounter[7:0] == 8'd251) MainState <= `DDR_IF_IDLE;
`endif
                end
            end
            else if(MainState == `DDR_IF_IDLE) begin    //=== IDLE ===============
                DDRNCS      <= 1'b1;
                {DDRRAS, DDRCAS, DDRNWE}    <= `CMD_NOP;
                DDRBA       <= 2'b00;
                DDRA        <= 13'h0;
                if(RefreshReq) begin
                    if(IdleCounter == 3'd4) MainState <= `DDR_IF_REF;
                end
                else if(ComValid) begin
                    if((LastBankAdr == BankAdr) | ({LastRW, DdrRW} == 2'b01)) begin
                        if(IdleCounter == 3'd4)  begin
                            if(DdrBRST & DdrRW) begin
                                if(~DdrComFifoEmpty3) MainState <= `DDR_IF_COM;
                            end
                            else begin
                                MainState <= `DDR_IF_COM;
                            end
                        end
                    end
                    else begin
                        if(DdrBRST & DdrRW) begin
                            if(~DdrComFifoEmpty3) MainState <= `DDR_IF_COM;
                        end
                        else begin
                            MainState <= `DDR_IF_COM;
                        end
                    end
                end
            end
            else if(MainState == `DDR_IF_COM) begin     //=== ACT ===============
                DDRNCS      <= 1'b0;
                {DDRRAS, DDRCAS, DDRNWE}    <= `CMD_ACT;
                DDRBA       <= BankAdr;
                LastBankAdr <= BankAdr;
                DDRA        <= RowAdr;
                LastRW      <= DdrRW;
                if(DdrRW) MainState <= `DDR_IF_WRIT;
                else      MainState <= `DDR_IF_READ;
            end
            else if(MainState == `DDR_IF_WRIT) begin    //=== WRITE ===============
                if((SubStateCounter[2:0] == 3'd1))begin
                    DDRNCS      <= 1'b0;
                    {DDRRAS, DDRCAS, DDRNWE}    <= `CMD_WRIT;
                    DDRBA       <= BankAdr;
                    DDRA        <= ColAdr;
                end
                else begin
                    DDRNCS      <= 1'b1;
                    {DDRRAS, DDRCAS, DDRNWE}    <= `CMD_NOP;
                    if(SubStateCounter[2:0] == 3'd5) MainState <= `DDR_IF_IDLE;
                    if(SubStateCounter[2:0] == 3'd2) DdrWD1    <= DdrWD;
                    if(DdrBRST & ((SubStateCounter[2:0] == 3'd3)
                                | (SubStateCounter[2:0] == 3'd4)
                                | (SubStateCounter[2:0] == 3'd5))) DdrWD1    <= DdrWD;
                end
            end
            else if(MainState == `DDR_IF_READ) begin    //=== READ  ===============
                if((SubStateCounter[2:0] == 3'd1))begin
                    DDRNCS      <= 1'b0;
                    {DDRRAS, DDRCAS, DDRNWE}    <= `CMD_READ;
                    DDRBA       <= BankAdr;
                    DDRA        <= ColAdr;
                end
                else begin
                    DDRNCS      <= 1'b1;
                    {DDRRAS, DDRCAS, DDRNWE}    <= `CMD_NOP;
                    if(SubStateCounter[2:0] == 3'd4) begin
                        DdrRDMstr <= DdrComMstr;
                        MainState <= `DDR_IF_IDLE;
                    end
                end
            end
            else if(MainState == `DDR_IF_REF) begin     //=== REFRESH ===============
                if(SubStateCounter[3:0] == 4'd1)begin
                    DDRNCS      <= 1'b0;
                    {DDRRAS, DDRCAS, DDRNWE}    <= `CMD_PRE;
                    DDRBA       <= 2'b00;
                    DDRA        <= 13'h400;
                end
                else if(SubStateCounter[3:0] == 4'd3) begin
                    DDRNCS      <= 1'b0;
                    {DDRRAS, DDRCAS, DDRNWE}    <= `CMD_REF;
                end
                else begin
                    DDRNCS      <= 1'b1;
                    {DDRRAS, DDRCAS, DDRNWE}    <= `CMD_NOP;
                    if(SubStateCounter[3:0] == 4'd11) MainState <= `DDR_IF_IDLE;
                end
            end
            else begin
                MainState <= `DDR_IF_IDLE;
            end
        end
    end
    //==================================================
    //======== DDR Signals  ============================
    //==================================================
    //======== DDRCK, DDRNCK  ============================
    ODDR2  DDRCK_FF  (.Q(DDRCK ), .C0(DdrClkP270), .C1(~DdrClkP270), .CE(1'b1), .D0(1'b1), .D1(1'b0), .R(~RstN), .S(1'b0));
    ODDR2  DDRNCK_FF (.Q(DDRNCK), .C0(DdrClkP270), .C1(~DdrClkP270), .CE(1'b1), .D0(1'b0), .D1(1'b1), .R(~RstN), .S(1'b0));
    //======== DDRDQS  ============================
    ODDR2  DDRDQS0_FF(.Q(Dqs[0]), .C0(DdrClkP270), .C1(~DdrClkP270), .CE(1'b1), .D0(DqsH), .D1(1'b0), .R(~RstN), .S(1'b0));
    ODDR2  DDRDQS1_FF(.Q(Dqs[1]), .C0(DdrClkP270), .C1(~DdrClkP270), .CE(1'b1), .D0(DqsH), .D1(1'b0), .R(~RstN), .S(1'b0));
    assign DDRDQS[0] = (DqsOE[0])? Dqs[0] : 1'bz;
    assign DDRDQS[1] = (DqsOE[1])? Dqs[1] : 1'bz;

    //======== DDRDM  ============================
    always @(negedge DdrClk or negedge RstN)
      if(~RstN) DmHalfDly <= 2'b11;
      else      DmHalfDly <= Dm[1:0];
    ODDR2  DDRDM0_FF (.Q(DDRDM[0]), .C0(~DdrClk), .C1(DdrClk), .CE(1'b1), .D0(Dm[2]), .D1(DmHalfDly[0]), .R(~RstN), .S(1'b0));
    ODDR2  DDRDM1_FF (.Q(DDRDM[1]), .C0(~DdrClk), .C1(DdrClk), .CE(1'b1), .D0(Dm[3]), .D1(DmHalfDly[1]), .R(~RstN), .S(1'b0));

    //======== DDRDQ  ============================
    ODDR2  DDRDQ00W_FF (.Q(DqW[ 0]), .C0(~DdrClk), .C1(DdrClk), .CE(1'b1), .D0(DdrWD1[16]), .D1(DdrWD1[ 0]), .R(~RstN), .S(1'b0));
    ODDR2  DDRDQ01W_FF (.Q(DqW[ 1]), .C0(~DdrClk), .C1(DdrClk), .CE(1'b1), .D0(DdrWD1[17]), .D1(DdrWD1[ 1]), .R(~RstN), .S(1'b0));
    ODDR2  DDRDQ02W_FF (.Q(DqW[ 2]), .C0(~DdrClk), .C1(DdrClk), .CE(1'b1), .D0(DdrWD1[18]), .D1(DdrWD1[ 2]), .R(~RstN), .S(1'b0));
    ODDR2  DDRDQ03W_FF (.Q(DqW[ 3]), .C0(~DdrClk), .C1(DdrClk), .CE(1'b1), .D0(DdrWD1[19]), .D1(DdrWD1[ 3]), .R(~RstN), .S(1'b0));
    ODDR2  DDRDQ04W_FF (.Q(DqW[ 4]), .C0(~DdrClk), .C1(DdrClk), .CE(1'b1), .D0(DdrWD1[20]), .D1(DdrWD1[ 4]), .R(~RstN), .S(1'b0));
    ODDR2  DDRDQ05W_FF (.Q(DqW[ 5]), .C0(~DdrClk), .C1(DdrClk), .CE(1'b1), .D0(DdrWD1[21]), .D1(DdrWD1[ 5]), .R(~RstN), .S(1'b0));
    ODDR2  DDRDQ06W_FF (.Q(DqW[ 6]), .C0(~DdrClk), .C1(DdrClk), .CE(1'b1), .D0(DdrWD1[22]), .D1(DdrWD1[ 6]), .R(~RstN), .S(1'b0));
    ODDR2  DDRDQ07W_FF (.Q(DqW[ 7]), .C0(~DdrClk), .C1(DdrClk), .CE(1'b1), .D0(DdrWD1[23]), .D1(DdrWD1[ 7]), .R(~RstN), .S(1'b0));
    ODDR2  DDRDQ08W_FF (.Q(DqW[ 8]), .C0(~DdrClk), .C1(DdrClk), .CE(1'b1), .D0(DdrWD1[24]), .D1(DdrWD1[ 8]), .R(~RstN), .S(1'b0));
    ODDR2  DDRDQ09W_FF (.Q(DqW[ 9]), .C0(~DdrClk), .C1(DdrClk), .CE(1'b1), .D0(DdrWD1[25]), .D1(DdrWD1[ 9]), .R(~RstN), .S(1'b0));
    ODDR2  DDRDQ10W_FF (.Q(DqW[10]), .C0(~DdrClk), .C1(DdrClk), .CE(1'b1), .D0(DdrWD1[26]), .D1(DdrWD1[10]), .R(~RstN), .S(1'b0));
    ODDR2  DDRDQ11W_FF (.Q(DqW[11]), .C0(~DdrClk), .C1(DdrClk), .CE(1'b1), .D0(DdrWD1[27]), .D1(DdrWD1[11]), .R(~RstN), .S(1'b0));
    ODDR2  DDRDQ12W_FF (.Q(DqW[12]), .C0(~DdrClk), .C1(DdrClk), .CE(1'b1), .D0(DdrWD1[28]), .D1(DdrWD1[12]), .R(~RstN), .S(1'b0));
    ODDR2  DDRDQ13W_FF (.Q(DqW[13]), .C0(~DdrClk), .C1(DdrClk), .CE(1'b1), .D0(DdrWD1[29]), .D1(DdrWD1[13]), .R(~RstN), .S(1'b0));
    ODDR2  DDRDQ14W_FF (.Q(DqW[14]), .C0(~DdrClk), .C1(DdrClk), .CE(1'b1), .D0(DdrWD1[30]), .D1(DdrWD1[14]), .R(~RstN), .S(1'b0));
    ODDR2  DDRDQ15W_FF (.Q(DqW[15]), .C0(~DdrClk), .C1(DdrClk), .CE(1'b1), .D0(DdrWD1[31]), .D1(DdrWD1[15]), .R(~RstN), .S(1'b0));
    assign DDRDQ[ 0] = (DqOE[ 0])? DqW[ 0] : 1'bz;
    assign DDRDQ[ 1] = (DqOE[ 1])? DqW[ 1] : 1'bz;
    assign DDRDQ[ 2] = (DqOE[ 2])? DqW[ 2] : 1'bz;
    assign DDRDQ[ 3] = (DqOE[ 3])? DqW[ 3] : 1'bz;
    assign DDRDQ[ 4] = (DqOE[ 4])? DqW[ 4] : 1'bz;
    assign DDRDQ[ 5] = (DqOE[ 5])? DqW[ 5] : 1'bz;
    assign DDRDQ[ 6] = (DqOE[ 6])? DqW[ 6] : 1'bz;
    assign DDRDQ[ 7] = (DqOE[ 7])? DqW[ 7] : 1'bz;
    assign DDRDQ[ 8] = (DqOE[ 8])? DqW[ 8] : 1'bz;
    assign DDRDQ[ 9] = (DqOE[ 9])? DqW[ 9] : 1'bz;
    assign DDRDQ[10] = (DqOE[10])? DqW[10] : 1'bz;
    assign DDRDQ[11] = (DqOE[11])? DqW[11] : 1'bz;
    assign DDRDQ[12] = (DqOE[12])? DqW[12] : 1'bz;
    assign DDRDQ[13] = (DqOE[13])? DqW[13] : 1'bz;
    assign DDRDQ[14] = (DqOE[14])? DqW[14] : 1'bz;
    assign DDRDQ[15] = (DqOE[15])? DqW[15] : 1'bz;
//========================= for SYN begin ====================================================================================================
`ifdef SYN
    IDDR2  DDRDQ00R_FF (.Q0(DqR[16]), .Q1(DqR[ 0]), .C0(DdrClkP270), .C1(~DdrClkP270), .CE(1'b1), .D(DDRDQ[ 0]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ01R_FF (.Q0(DqR[17]), .Q1(DqR[ 1]), .C0(DdrClkP270), .C1(~DdrClkP270), .CE(1'b1), .D(DDRDQ[ 1]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ02R_FF (.Q0(DqR[18]), .Q1(DqR[ 2]), .C0(DdrClkP270), .C1(~DdrClkP270), .CE(1'b1), .D(DDRDQ[ 2]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ03R_FF (.Q0(DqR[19]), .Q1(DqR[ 3]), .C0(DdrClkP270), .C1(~DdrClkP270), .CE(1'b1), .D(DDRDQ[ 3]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ04R_FF (.Q0(DqR[20]), .Q1(DqR[ 4]), .C0(DdrClkP270), .C1(~DdrClkP270), .CE(1'b1), .D(DDRDQ[ 4]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ05R_FF (.Q0(DqR[21]), .Q1(DqR[ 5]), .C0(DdrClkP270), .C1(~DdrClkP270), .CE(1'b1), .D(DDRDQ[ 5]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ06R_FF (.Q0(DqR[22]), .Q1(DqR[ 6]), .C0(DdrClkP270), .C1(~DdrClkP270), .CE(1'b1), .D(DDRDQ[ 6]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ07R_FF (.Q0(DqR[23]), .Q1(DqR[ 7]), .C0(DdrClkP270), .C1(~DdrClkP270), .CE(1'b1), .D(DDRDQ[ 7]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ08R_FF (.Q0(DqR[24]), .Q1(DqR[ 8]), .C0(DdrClkP270), .C1(~DdrClkP270), .CE(1'b1), .D(DDRDQ[ 8]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ09R_FF (.Q0(DqR[25]), .Q1(DqR[ 9]), .C0(DdrClkP270), .C1(~DdrClkP270), .CE(1'b1), .D(DDRDQ[ 9]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ10R_FF (.Q0(DqR[26]), .Q1(DqR[10]), .C0(DdrClkP270), .C1(~DdrClkP270), .CE(1'b1), .D(DDRDQ[10]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ11R_FF (.Q0(DqR[27]), .Q1(DqR[11]), .C0(DdrClkP270), .C1(~DdrClkP270), .CE(1'b1), .D(DDRDQ[11]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ12R_FF (.Q0(DqR[28]), .Q1(DqR[12]), .C0(DdrClkP270), .C1(~DdrClkP270), .CE(1'b1), .D(DDRDQ[12]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ13R_FF (.Q0(DqR[29]), .Q1(DqR[13]), .C0(DdrClkP270), .C1(~DdrClkP270), .CE(1'b1), .D(DDRDQ[13]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ14R_FF (.Q0(DqR[30]), .Q1(DqR[14]), .C0(DdrClkP270), .C1(~DdrClkP270), .CE(1'b1), .D(DDRDQ[14]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ15R_FF (.Q0(DqR[31]), .Q1(DqR[15]), .C0(DdrClkP270), .C1(~DdrClkP270), .CE(1'b1), .D(DDRDQ[15]), .R(~RstN), .S(1'b0));

    always  @(posedge DdrClk or negedge RstN) begin
        if(~RstN)   DqRDelay <= 16'h0;
        else        DqRDelay <= DqR[31:16];
    end
    assign  DdrRD[31:0] = {DqRDelay, DqR[15:0]};
`endif
//========================= for SYN end ====================================================================================================

//========================= for SIM begin ====================================================================================================
`ifdef SIM
    reg [15:0] DDRDQR;
    always @(DDRDQ) #0 DDRDQR = DDRDQ;

    IDDR2  DDRDQ00R_FF (.Q0(DqR[16]), .Q1(DqR[ 0]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[ 0]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ01R_FF (.Q0(DqR[17]), .Q1(DqR[ 1]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[ 1]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ02R_FF (.Q0(DqR[18]), .Q1(DqR[ 2]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[ 2]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ03R_FF (.Q0(DqR[19]), .Q1(DqR[ 3]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[ 3]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ04R_FF (.Q0(DqR[20]), .Q1(DqR[ 4]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[ 4]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ05R_FF (.Q0(DqR[21]), .Q1(DqR[ 5]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[ 5]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ06R_FF (.Q0(DqR[22]), .Q1(DqR[ 6]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[ 6]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ07R_FF (.Q0(DqR[23]), .Q1(DqR[ 7]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[ 7]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ08R_FF (.Q0(DqR[24]), .Q1(DqR[ 8]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[ 8]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ09R_FF (.Q0(DqR[25]), .Q1(DqR[ 9]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[ 9]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ10R_FF (.Q0(DqR[26]), .Q1(DqR[10]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[10]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ11R_FF (.Q0(DqR[27]), .Q1(DqR[11]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[11]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ12R_FF (.Q0(DqR[28]), .Q1(DqR[12]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[12]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ13R_FF (.Q0(DqR[29]), .Q1(DqR[13]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[13]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ14R_FF (.Q0(DqR[30]), .Q1(DqR[14]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[14]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ15R_FF (.Q0(DqR[31]), .Q1(DqR[15]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[15]), .R(~RstN), .S(1'b0));

    reg [31:0] DdrRD0d;
    always  @(posedge DdrClk) DdrRD0d <= DqR;
    assign  DdrRD[31:0] = DdrRD0d;
`endif
`ifdef SIM2
    reg [15:0] DDRDQR;
    always @(DDRDQ) #0 DDRDQR = DDRDQ;

    IDDR2  DDRDQ00R_FF (.Q0(DqR[16]), .Q1(DqR[ 0]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[ 0]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ01R_FF (.Q0(DqR[17]), .Q1(DqR[ 1]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[ 1]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ02R_FF (.Q0(DqR[18]), .Q1(DqR[ 2]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[ 2]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ03R_FF (.Q0(DqR[19]), .Q1(DqR[ 3]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[ 3]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ04R_FF (.Q0(DqR[20]), .Q1(DqR[ 4]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[ 4]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ05R_FF (.Q0(DqR[21]), .Q1(DqR[ 5]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[ 5]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ06R_FF (.Q0(DqR[22]), .Q1(DqR[ 6]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[ 6]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ07R_FF (.Q0(DqR[23]), .Q1(DqR[ 7]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[ 7]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ08R_FF (.Q0(DqR[24]), .Q1(DqR[ 8]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[ 8]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ09R_FF (.Q0(DqR[25]), .Q1(DqR[ 9]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[ 9]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ10R_FF (.Q0(DqR[26]), .Q1(DqR[10]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[10]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ11R_FF (.Q0(DqR[27]), .Q1(DqR[11]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[11]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ12R_FF (.Q0(DqR[28]), .Q1(DqR[12]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[12]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ13R_FF (.Q0(DqR[29]), .Q1(DqR[13]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[13]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ14R_FF (.Q0(DqR[30]), .Q1(DqR[14]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[14]), .R(~RstN), .S(1'b0));
    IDDR2  DDRDQ15R_FF (.Q0(DqR[31]), .Q1(DqR[15]), .C0(DdrClk), .C1(~DdrClk), .CE(1'b1), .D(DDRDQR[15]), .R(~RstN), .S(1'b0));

    reg [31:0] DdrRD0d;
    always  @(posedge DdrClk) DdrRD0d <= DqR;
    assign  DdrRD[31:0] = DdrRD0d;
`endif
//========================= for SIM end ====================================================================================================
endmodule
