//============================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : Psm3Sys (PSM3 TEST SYSTEM)
//  FUNCTION      : OCM(2KB) Only 4Byte Access
//
//                      Copyright (C) 2008 by Tomoyuki Inomoto
//============================================================
module Ocm(
    //======== SYSTEM ==============
    Clk         ,
    RstN        ,

    //======== Peripheral Bus ======
    HostCS      ,
    HostRQ      ,
    HostRW      ,
    HostBE      ,
    HostADR     ,
    HostWD      ,
    HostRD
);
    //======== SYSTEM ==============
    input           Clk         ;
    input           RstN        ;

    //======== Peripheral Bus ======
    input           HostCS      ;
    input           HostRQ      ;
    input           HostRW      ;
    input   [3:0]   HostBE      ;
    input   [11:0]  HostADR     ;
    input   [31:0]  HostWD      ;
    output  [31:0]  HostRD      ;

    //==================================================
    //======== wire & reg ==============================
    //==================================================
    wire            OcmWe       ;
    wire    [8:0]   OcmAdr      ;
    wire    [31:0]  MemWD       ;

    //====================================================================
    //======== BODY ======================================================
    //====================================================================
    assign  OcmWe = HostRW & HostRQ & HostCS;
    assign  OcmAdr = HostADR[10:2];

    assign  MemWD[31:24] = (HostBE[3])? HostWD[31:24] : HostRD[31:24];
    assign  MemWD[23:16] = (HostBE[2])? HostWD[23:16] : HostRD[23:16];
    assign  MemWD[15: 8] = (HostBE[1])? HostWD[15: 8] : HostRD[15: 8];
    assign  MemWD[ 7: 0] = (HostBE[0])? HostWD[ 7: 0] : HostRD[ 7: 0];

    OcmDpram OcmDpram(
        .addra   ( OcmAdr   ),
        .addrb   ( OcmAdr   ),
        .clka    (~Clk      ),
        .clkb    ( Clk      ),
        .dinb    ( MemWD    ),
        .douta   ( HostRD   ),
        .web     ( OcmWe    )
    );

endmodule
/*=======================================================================*
 *  TARGET PROJECT: PSM3(Prosessor Study Model3)                         *
 *  TARGET SYSTEM : Psm3Sys (PSM3 TEST SYSTEM)                           *
 *  FUNCTION      : OCM DPRAM                                            *
 *=======================================================================*/
module OcmDpram(
    addra   ,
    addrb   ,
    clka    ,
    clkb    ,
    dinb    ,
    douta   ,
    web
);

    input   [8:0]   addra;
    input   [8:0]   addrb;
    input           clka;
    input           clkb;
    input   [31:0]  dinb;
    output  [31:0]  douta;
    input           web;

    reg     [31:0]  mem[0:511];
    reg     [31:0]  douta;

    always @(posedge clka) douta <= mem[addra];
    always @(posedge clkb) if(web) mem[addrb] <= dinb;

endmodule
