//============================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : Psm3Sys (PSM3 TEST SYSTEM)
//  FUNCTION      : Bus Arbiter for Peripheral
//
//                      Copyright (C) 2008 by Tomoyuki Inomoto
//============================================================
`define     PERI_ABT_IDLE    2'h0
`define     PERI_ABT_WRITE   2'h1
`define     PERI_ABT_READ0   2'h2
`define     PERI_ABT_READ1   2'h3


module P3dPeriAbt(
    //======== SYSTEM ==============
    Clk             ,
    RstN            ,

    //======= Peri Bus (Master0) ========
    PeriMstr0RQ     ,
    PeriMstr0AK     ,
    PeriMstr0RW     ,
    PeriMstr0BE     ,
    PeriMstr0ADR    ,
    PeriMstr0WD     ,
    PeriMstr0RD     ,

    //======= Peri Bus (Master1) ========
    PeriMstr1RQ     ,
    PeriMstr1AK     ,
    PeriMstr1RW     ,
    PeriMstr1BE     ,
    PeriMstr1ADR    ,
    PeriMstr1WD     ,
    PeriMstr1RD     ,

    //======= Peri Bus (Master2) ========
    PeriMstr2RQ     ,
    PeriMstr2AK     ,
    PeriMstr2RW     ,
    PeriMstr2BE     ,
    PeriMstr2ADR    ,
    PeriMstr2WD     ,
    PeriMstr2RD     ,

    //======= Peri Bus (Slave) =======
    Irq0CS          ,
    Irq1CS          ,
    Timer0CS        ,
    Timer1CS        ,
    UartCS          ,
    GpioCS          ,
    VgaCS           ,
    PMon0CS         ,
    PMon1CS         ,
    OcmCS           ,

    PeriSlaveRQ     ,
    PeriSlaveRW     ,
    PeriSlaveBE     ,
    PeriSlaveADR    ,
    PeriSlaveWD     ,

    Irq0RD          ,
    Irq1RD          ,
    Timer0RD        ,
    Timer1RD        ,
    UartRD          ,
    GpioRD          ,
    VgaRD           ,
    PMon0RD         ,
    PMon1RD         ,
    OcmRD

);
    //==================================================
    //======== port  ===================================
    //==================================================
    //======== SYSTEM ==============
    input           Clk             ;
    input           RstN            ;

    //======= Peri Bus (Host0) ========
    input           PeriMstr0RQ     ;
    output          PeriMstr0AK     ;
    input           PeriMstr0RW     ;
    input   [3:0]   PeriMstr0BE     ;
    input   [15:0]  PeriMstr0ADR    ;
    input   [31:0]  PeriMstr0WD     ;
    output  [31:0]  PeriMstr0RD     ;

    //======= Peri Bus (Host1) ========
    input           PeriMstr1RQ     ;
    output          PeriMstr1AK     ;
    input           PeriMstr1RW     ;
    input   [3:0]   PeriMstr1BE     ;
    input   [15:0]  PeriMstr1ADR    ;
    input   [31:0]  PeriMstr1WD     ;
    output  [31:0]  PeriMstr1RD     ;

    //======= Peri Bus (Host2) ========
    input           PeriMstr2RQ     ;
    output          PeriMstr2AK     ;
    input           PeriMstr2RW     ;
    input   [3:0]   PeriMstr2BE     ;
    input   [15:0]  PeriMstr2ADR    ;
    input   [31:0]  PeriMstr2WD     ;
    output  [31:0]  PeriMstr2RD     ;

    //======= Peri Bus (Slave) =======
    output          Irq0CS          ;
    output          Irq1CS          ;
    output          Timer0CS        ;
    output          Timer1CS        ;
    output          UartCS          ;
    output          GpioCS          ;
    output          VgaCS           ;
    output          PMon0CS         ;
    output          PMon1CS         ;
    output          OcmCS           ;

    output          PeriSlaveRQ     ;
    output          PeriSlaveRW     ;
    output  [3:0]   PeriSlaveBE     ;
    output  [11:0]  PeriSlaveADR    ;
    output  [31:0]  PeriSlaveWD     ;

    input   [31:0]  Irq0RD          ;
    input   [31:0]  Irq1RD          ;
    input   [31:0]  Timer0RD        ;
    input   [31:0]  Timer1RD        ;
    input   [31:0]  UartRD          ;
    input   [31:0]  GpioRD          ;
    input   [31:0]  VgaRD           ;
    input   [31:0]  PMon0RD         ;
    input   [31:0]  PMon1RD         ;
    input   [31:0]  OcmRD           ;

    //==================================================
    //======== wire & reg ==============================
    //==================================================
    reg     [1:0]   AbtState        ;

    wire    [2:0]   SelectedMstr    ;
    reg     [2:0]   SelectedMstrHold;
    wire            PeriMstrRQ      ;
    wire            PeriMstrRW      ;
    wire    [3:0]   PeriMstrBE      ;
    wire    [15:0]  PeriMstrADR     ;
    wire    [31:0]  PeriMstrWD      ;
    reg     [31:0]  PeriMstrRD      ;

    reg             Irq0CS          ;
    reg             Irq1CS          ;
    reg             Timer0CS        ;
    reg             Timer1CS        ;
    reg             UartCS          ;
    reg             GpioCS          ;
    reg             VgaCS           ;
    reg             PMon0CS         ;
    reg             PMon1CS         ;
    reg             OcmCS           ;

    reg             PeriSlaveRW     ;
    reg     [3:0]   PeriSlaveBE     ;
    reg             PeriSlaveRQ     ;
    reg     [11:0]  PeriSlaveADR    ;
    reg     [31:0]  PeriSlaveWD     ;

    reg             PeriMstr0AK     ;
    reg             PeriMstr1AK     ;
    reg             PeriMstr2AK     ;

    wire            PeriSelCom      ;

    //====================================================================
    //======== BODY ======================================================
    //====================================================================
    //==================================================
    //======== Internal Signals  =======================
    //==================================================
    assign  SelectedMstr = (PeriMstr0RQ)? 3'b001 :
                           (PeriMstr1RQ)? 3'b010 :
                           (PeriMstr2RQ)? 3'b100 : 3'b000;

    assign  PeriMstrRQ    = PeriMstr0RQ | PeriMstr1RQ | PeriMstr2RQ;

    assign  PeriMstrRW   = (PeriMstr0RQ)? PeriMstr0RW:
                           (PeriMstr1RQ)? PeriMstr1RW:
                                          PeriMstr2RW;

    assign  PeriMstrBE   = (PeriMstr0RQ)? PeriMstr0BE:
                           (PeriMstr1RQ)? PeriMstr1BE:
                                          PeriMstr2BE;

    assign  PeriMstrADR  = (PeriMstr0RQ)? PeriMstr0ADR:
                           (PeriMstr1RQ)? PeriMstr1ADR:
                                          PeriMstr2ADR;

    assign  PeriMstrWD   = (PeriMstr0RQ)? PeriMstr0WD:
                           (PeriMstr1RQ)? PeriMstr1WD:
                                          PeriMstr2WD;

    assign  PeriSelCom   = (PeriMstrADR[15:8] == 8'h0);

    //==================================================
    //======== State Machine     =======================
    //==================================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            Irq0CS          <= 1'h0;
            Irq1CS          <= 1'h0;
            Timer0CS        <= 1'h0;
            Timer1CS        <= 1'h0;
            UartCS          <= 1'h0;
            GpioCS          <= 1'h0;
            VgaCS           <= 1'h0;
            PMon0CS         <= 1'h0;
            PMon1CS         <= 1'h0;
            OcmCS           <= 1'h0;

            PeriSlaveRW     <= 1'h0;
            PeriSlaveRQ     <= 1'h0;
            PeriSlaveBE     <= 4'h0;
            PeriSlaveADR    <= 12'h0;
            PeriSlaveWD     <= 32'h0;

            PeriMstr0AK     <= 1'h0;
            PeriMstr1AK     <= 1'h0;
            PeriMstr2AK     <= 1'h0;

            SelectedMstrHold <= 3'b000;

            AbtState        <= `PERI_ABT_IDLE;
        end
        else begin
            if(AbtState == `PERI_ABT_IDLE) begin
                if(PeriMstrRQ) begin
                    Irq0CS      <= PeriSelCom & (PeriMstrADR[7: 4] == 4'h0);
                    Irq1CS      <= PeriSelCom & (PeriMstrADR[7: 4] == 4'h1);
                    Timer0CS    <= PeriSelCom & (PeriMstrADR[7: 4] == 4'h2);
                    Timer1CS    <= PeriSelCom & (PeriMstrADR[7: 4] == 4'h3);
                    GpioCS      <= PeriSelCom & (PeriMstrADR[7: 4] == 4'h4);
                    UartCS      <= PeriSelCom & (PeriMstrADR[7: 4] == 4'h5);
                    VgaCS       <= PeriSelCom & (PeriMstrADR[7: 4] == 4'h6);
                    PMon0CS     <= PeriSelCom & (PeriMstrADR[7: 4] == 4'hE);
                    PMon1CS     <= PeriSelCom & (PeriMstrADR[7: 4] == 4'hF);
                    OcmCS       <= (PeriMstrADR[15:12] == 4'h8);

                    PeriSlaveRW     <= PeriMstrRW;
                    PeriSlaveRQ     <= 1'h1;
                    PeriSlaveBE     <= PeriMstrBE;
                    PeriSlaveADR    <= PeriMstrADR[11:0];
                    PeriSlaveWD     <= PeriMstrWD;

                    SelectedMstrHold <= SelectedMstr;

                    if(PeriMstrRW) begin
                        AbtState    <= `PERI_ABT_WRITE;
                        PeriMstr0AK <= SelectedMstr[0];
                        PeriMstr1AK <= SelectedMstr[1];
                        PeriMstr2AK <= SelectedMstr[2];
                    end
                    else begin
                        AbtState    <= `PERI_ABT_READ0;
                        PeriMstr0AK <= 1'h0;
                        PeriMstr1AK <= 1'h0;
                        PeriMstr2AK <= 1'h0;
                    end
                end
            end
            else if(AbtState == `PERI_ABT_WRITE) begin
                Irq0CS          <= 1'h0;
                Irq1CS          <= 1'h0;
                Timer0CS        <= 1'h0;
                Timer1CS        <= 1'h0;
                UartCS          <= 1'h0;
                GpioCS          <= 1'h0;
                VgaCS           <= 1'h0;
                PMon0CS         <= 1'h0;
                PMon1CS         <= 1'h0;
                OcmCS           <= 1'h0;

                PeriSlaveRW     <= 1'h0;
                PeriSlaveRQ     <= 1'h0;

                PeriMstr0AK     <= 1'h0;
                PeriMstr1AK     <= 1'h0;
                PeriMstr2AK     <= 1'h0;
                AbtState        <= `PERI_ABT_IDLE;
            end
            else if(AbtState == `PERI_ABT_READ0) begin
                AbtState        <= `PERI_ABT_READ1;
                PeriMstr0AK     <= SelectedMstrHold[0];
                PeriMstr1AK     <= SelectedMstrHold[1];
                PeriMstr2AK     <= SelectedMstrHold[2];
            end
            else if(AbtState == `PERI_ABT_READ1) begin
                Irq0CS          <= 1'h0;
                Irq1CS          <= 1'h0;
                Timer0CS        <= 1'h0;
                Timer1CS        <= 1'h0;
                UartCS          <= 1'h0;
                GpioCS          <= 1'h0;
                VgaCS           <= 1'h0;
                PMon0CS         <= 1'h0;
                PMon1CS         <= 1'h0;
                OcmCS           <= 1'h0;

                PeriSlaveRW     <= 1'h0;
                PeriSlaveRQ     <= 1'h0;

                PeriMstr0AK     <= 1'h0;
                PeriMstr1AK     <= 1'h0;
                PeriMstr2AK     <= 1'h0;
                AbtState        <= `PERI_ABT_IDLE;
            end
            else begin
                AbtState <= `PERI_ABT_IDLE;
            end
        end
    end
    //==================================================
    //======== Read Data Select  =======================
    //==================================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            PeriMstrRD  <= 32'h0;
        end
        else begin
            if     (OcmCS       ) PeriMstrRD <= OcmRD    ;
            else if(Irq0CS      ) PeriMstrRD <= Irq0RD   ;
            else if(Irq1CS      ) PeriMstrRD <= Irq1RD   ;
            else if(Timer0CS    ) PeriMstrRD <= Timer0RD ;
            else if(Timer1CS    ) PeriMstrRD <= Timer1RD ;
            else if(UartCS      ) PeriMstrRD <= UartRD   ;
            else if(GpioCS      ) PeriMstrRD <= GpioRD   ;
            else if(VgaCS       ) PeriMstrRD <= VgaRD    ;
            else if(PMon0CS     ) PeriMstrRD <= PMon0RD  ;
            else if(PMon1CS     ) PeriMstrRD <= PMon1RD  ;
            else                  PeriMstrRD <= 32'h0    ;
        end
    end
    assign  PeriMstr0RD = PeriMstrRD;
    assign  PeriMstr1RD = PeriMstrRD;
    assign  PeriMstr2RD = PeriMstrRD;
endmodule

