//=======================================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : *(PSM3 TARGET SYSTEM)
//  FUNCTION      : Psm3 ALU
//
//                      Copyright (C) 2008-2010 by Tomoyuki Inomoto
//=======================================================================
// 20100327:Some signals about forwarding are added. (FwdData)
//         :The Instruction which makes a flag change is changed.
//=======================================================================
`define     FUNC_MOV    4'h0
`define     FUNC_MOVL   4'h1
`define     FUNC_MOVH   4'h2
`define     FUNC_ADD    4'h3
`define     FUNC_SUB    4'h4
`define     FUNC_AND    4'h5
`define     FUNC_OR     4'h6
`define     FUNC_XOR    4'h7
`define     FUNC_LSR    4'h8
`define     FUNC_LSL    4'h9
`define     FUNC_ASR    4'hA
`define     FUNC_MUL    4'hB
`define     FUNC_DIVU   4'hC
`define     FUNC_DIV    4'hD
`define     FUNC_MODU   4'hE
`define     FUNC_MOD    4'hF

//`define     MUL1CYC
`define     MUL2CYC

module P3Alu(
    //======== SYSTEM ==================
    Clk             ,
    RstN            ,

    //======== from DECU ===============
    GoAhead         ,
    AluAct0         ,
    RfWe0           , // 20100327 add
    RfRdDataB       ,
    RfRdDataC       ,
    RfRdDC          ,
    Imm16           ,
    ImmSel          ,
    Func0           ,

    //======= to/from Common Unit ======
    ComInB          ,
    ComInC          ,
    ComFunc         ,

    ComBSftRQ       ,
    ComBSftAK       ,
    ComBSftCmpl     ,
    ComBSftRes      ,

    ComDivRQ        ,
    ComDivAK        ,
    ComDivCmpl      ,
    ComDivRes       ,

    //======== Output =================
    DataADR0        ,
    ResALU          ,
    Over            ,
    Carry           ,
    Sign            ,
    Zero            ,

    AluAct          ,
    UpDateCV        ,
    UpDateSZ        ,
    AluWait         ,
    FwdData         ,// 20100327
    FwdDataValid     // 20100327
);
    //==========================================================
    //======== port  ===========================================
    //==========================================================
    //======== SYSTEM ========================
    input           Clk             ;
    input           RstN            ;

    //======== from DECU =================
    input           GoAhead         ;
    input           AluAct0         ; // ALU ACTIVE
    input           RfWe0           ; // Register file Write Enable  // 20100327 add
    input   [31:0]  RfRdDataB       ; // PortB Read Data
    input   [31:0]  RfRdDataC       ; // PortC Read Data
    input   [31:0]  RfRdDC          ; // GPR PortC Read Data
    input   [15:0]  Imm16           ; // 16bit Immdiate
    input           ImmSel          ; // Immdiate Select
    input   [3:0]   Func0           ; // ALU FUNCTION

    //======= to/from Common Unit =================
    output  [31:0]  ComInB          ; // PortB Data
    output  [31:0]  ComInC          ; // PortC Data
    output  [3:0]   ComFunc         ; // FUNCTION

    output          ComBSftRQ       ; // Common Barrel Shifter REQ
    input           ComBSftAK       ; // Common Barrel Shifter ACK
    input           ComBSftCmpl     ; // Common Barrel Shifter Complete
    input   [31:0]  ComBSftRes      ; // Result of Common Barrel Shifter

    output          ComDivRQ        ; // Common Divisor REQ
    input           ComDivAK        ; // Common Divisor ACK
    input           ComDivCmpl      ; // Common Divisor Complete
    input   [31:0]  ComDivRes       ; // Result of Common Divisor

    //======== Output =================
    output  [27:0]  DataADR0        ; // Address Of data bus
    output  [31:0]  ResALU          ; // Result of ALU
    output          Over            ; // Flag
    output          Carry           ; // Flag
    output          Sign            ; // Flag
    output          Zero            ; // Flag

    output          AluAct          ; // ALU Active
    output          UpDateCV        ; // UpDate Carry & Over
    output          UpDateSZ        ; // UpDate Sign & Zero
    output          AluWait         ; // ALU Wait
    output  [31:0]  FwdData         ; // Data for forwarding // 20100327
    output          FwdDataValid    ; // Data for forwarding valid // 20100327
    
    //==========================================================
    //======== wire & reg ======================================
    //==========================================================
    reg             ComBSftRQ       ; // Common Barrel Shifter REQ
    reg             ComDivRQ        ; // Common Divisor REQ
    reg     [3:0]   Func            ; // ALU FUNCTION
    reg             AluAct          ; // ALU Active
    reg     [31:0]  ResLogic0       ; // Result of Logic
    reg     [31:0]  ResLogic        ; // Result of Logic
    wire    [32:0]  ResAddSub0      ; // Result of Add/Sub
    reg     [32:0]  ResAddSub       ; // Result of Add/Sub
    reg     [31:0]  ResSftDiv       ; // Result of Div/Mod & Shift
    reg             MulRQ           ; // Multiplier Reqest
    wire            MulAK           ; // Multiplier Ack
    wire            MulCmpl         ; // Multiplier Complete
    wire    [31:0]  ResMul0         ; // Result of Multiplier
    reg     [31:0]  ResMul          ; // Result of Mul
    reg     [31:0]  AluInB          ; // Input of Mul/Div/Sft
    reg     [31:0]  AluInC          ; // Input of Mul/Div/Sft
    wire    [31:0]  AluInC0         ; // Input of Mul/Div/Sft/AddSub
    wire    [31:0]  LogicAluInC     ; // Input of Logic
    wire    [31:0]  MovResult       ; // Input of Logic
    reg     [31:0]  ResALU          ; // Result of ALU
    reg             AluWait         ; // ALU Wait
    reg             AddSubSel       ; // 20100327 add
    reg             FwdDataValid    ; // Data for forwarding valid // 20100327 add
    reg             UpDateCV        ; // UpDate Carry & Over // 20100327 add
    reg             UpDateSZ        ; // UpDate Sign & Zero  // 20100327 add
    reg             AddSubIsZero    ; // 20100327 add
    reg             LogicIsZero     ; // 20100327 add
    //==========================================================
    //======== BODY       ======================================
    //==========================================================

    //==========================================
    //======== CONTROL    ======================
    //==========================================
    //======== AluInB & AluInC ======================
    //assign AluInC0 = (ImmSel)? {{16{Imm16[15]}}, Imm16} : RfRdDataC;
    assign AluInC0 = (ImmSel)? {{16{Imm16[15]}}, Imm16} : RfRdDC;
    always @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            AluInB <= 32'h0;
            AluInC <= 32'h0;
        end
        else if(GoAhead) begin
            AluInB <= RfRdDataB;
            AluInC <= AluInC0;
        end
    end
    assign ComInB   = AluInB;
    assign ComInC   = AluInC;

    //======== Func ======================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)        Func <= 4'h0;
        else if(GoAhead) Func <= Func0;
    end
    assign ComFunc  = Func;

    //======== AluAct ======================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)        AluAct <= 1'h0;
        else if(GoAhead) AluAct <= AluAct0;
    end

    //======== UpDateCV =================
    //assign UpDateCV = AluAct & ((Func == `FUNC_ADD) | (Func == `FUNC_SUB));  // 20100327 remove
    always @(posedge Clk or negedge RstN) begin  // 20100327 add
        if(~RstN)        UpDateCV <= 1'h0;
        else if(GoAhead) UpDateCV <= AluAct0 & ((Func0 == `FUNC_ADD) | (Func0 == `FUNC_SUB));
    end

    //======== UpDateSZ =================
    //assign UpDateSZ = AluAct & ~((Func == `FUNC_MOV) | (Func == `FUNC_MOVH) | (Func == `FUNC_MOVL)); // 20100327 remove
    always @(posedge Clk or negedge RstN) begin  // 20100327 add
        if(~RstN)        UpDateSZ <= 1'h0;
        else if(GoAhead) UpDateSZ <= AluAct0 & ((Func0 == `FUNC_ADD) | (Func0 == `FUNC_SUB) | (Func0 == `FUNC_AND) | (Func0 == `FUNC_OR) | (Func0 == `FUNC_XOR));
    end

    //======== AluWait  =================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            AluWait <= 1'h0;
        end
        else begin
            if(GoAhead) begin
                if((Func0 == `FUNC_LSR)  | (Func0 == `FUNC_LSL) | (Func0 == `FUNC_ASR ) | (Func0 == `FUNC_MUL) |
                   (Func0 == `FUNC_DIVU) | (Func0 == `FUNC_DIV) | (Func0 == `FUNC_MODU) | (Func0 == `FUNC_MOD)) begin
                    AluWait <= AluAct0;
                end
                else begin
                    AluWait <= 1'b0;
                end
            end
            else if(ComDivCmpl | ComBSftCmpl | MulCmpl) begin
                AluWait <= 1'b0;
            end
        end
    end
    //==========================================
    //========  Logic       ====================
    //==========================================
    //======== ResLogic ======================
    assign LogicAluInC = (ImmSel)? {16'h0, Imm16} : RfRdDC;
    //always @(posedge Clk or negedge RstN) begin // 20100327 remove
    //    if(~RstN) begin
    //        ResLogic <= 32'h0;
    //    end
    //    else if(GoAhead) begin
    //        case(Func0)
    //            `FUNC_MOVH  : ResLogic <= {LogicAluInC[15:0], 16'h0};
    //            `FUNC_MOVL  : ResLogic <= {16'h0, LogicAluInC[15:0]};
    //            `FUNC_AND   : ResLogic <= RfRdDataB & LogicAluInC;
    //            `FUNC_OR    : ResLogic <= RfRdDataB | LogicAluInC;
    //            default     : ResLogic <= RfRdDataB ^ LogicAluInC;
    //        endcase
    //    end
    //end
    always @(Func0 or LogicAluInC or RfRdDataB) begin // 20100327 add
        case(Func0)
            `FUNC_MOVH  : ResLogic0 = {LogicAluInC[15:0], 16'h0};
            `FUNC_MOVL  : ResLogic0 = {16'h0, LogicAluInC[15:0]};
            `FUNC_AND   : ResLogic0 = RfRdDataB & LogicAluInC;
            `FUNC_OR    : ResLogic0 = RfRdDataB | LogicAluInC;
            default     : ResLogic0 = RfRdDataB ^ LogicAluInC;
        endcase
    end
    always @(posedge Clk or negedge RstN) begin // 20100327 add
        if(~RstN) begin
            LogicIsZero <= 1'b0;
        end
        else if(GoAhead) begin
            LogicIsZero <= (ResLogic0 == 32'h0);
        end
    end
    always @(posedge Clk or negedge RstN) begin // 20100327 add
        if(~RstN) begin
            ResLogic <= 32'h0;
        end
        else if(GoAhead) begin
            ResLogic <= ResLogic0;
        end
    end
    //==========================================
    //========  Add/Sub     ====================
    //==========================================
    //======== MovResult ======================
    assign  MovResult = (ImmSel)? {{16{Imm16[15]}}, Imm16} : RfRdDataC;

    //======== ResAddSub ======================
    assign  ResAddSub0 = (Func0 == `FUNC_ADD)? (RfRdDataB + AluInC0) :
                         (Func0 == `FUNC_SUB)? (RfRdDataB - AluInC0) : {1'b0, MovResult};
    assign  DataADR0   = ResAddSub0[27:0];
    always @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            ResAddSub <= 33'h0;
        end
        else if(GoAhead) begin
            ResAddSub <= ResAddSub0;
        end
    end
    always @(posedge Clk or negedge RstN) begin // 20100327 add
        if(~RstN) begin
            AddSubIsZero <= 1'b0;
        end
        else if(GoAhead) begin
            AddSubIsZero <= (ResAddSub0[31:0] == 32'h0);
        end
    end

    //==========================================
    //========  Shift       ====================
    //==========================================
    //======== ComBSftRQ =======================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            ComBSftRQ <= 1'h0;
        end
        else if(GoAhead) begin
            if((Func0 == `FUNC_LSR) | (Func0 == `FUNC_LSL)  | (Func0 == `FUNC_ASR)) begin
                ComBSftRQ <= AluAct0;
            end
            else begin
                ComBSftRQ <= 1'b0;
            end
        end
        else if(ComBSftAK) begin
            ComBSftRQ <= 1'b0;
        end
    end
    //==========================================
    //========  Divisor     ====================
    //==========================================
    //======== ComDivRQ =======================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            ComDivRQ <= 1'h0;
        end
        else if(GoAhead) begin
            if((Func0 == `FUNC_DIVU) | (Func0 == `FUNC_DIV) | (Func0 == `FUNC_MODU) | (Func0 == `FUNC_MOD)) begin
                ComDivRQ <= AluAct0;
            end
            else begin
                ComDivRQ <= 1'b0;
            end
        end
        else if(ComDivAK) begin
            ComDivRQ <= 1'b0;
        end
    end
    //==========================================
    //========  ResSftDiv   ====================
    //==========================================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)            ResSftDiv <= 32'h0;
        else if(ComBSftCmpl) ResSftDiv <= ComBSftRes;
        else if(ComDivCmpl)  ResSftDiv <= ComDivRes;
    end

    //==========================================
    //========  Multiplier  ====================
    //==========================================
    //======== MulRQ =======================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            MulRQ <= 1'h0;
        end
        else if(GoAhead) begin
            if(Func0 == `FUNC_MUL) begin
                MulRQ <= AluAct0;
            end
            else begin
                MulRQ <= 1'b0;
            end
        end
        else if(MulAK) begin
            MulRQ <= 1'b0;
        end
    end
    //======== Mul(32bit x 32bit = 64bit) ========
    Mul32x32 Mul32x32(
        .Clk    ( Clk       ),
        .RstN   ( RstN      ),
        .MulRQ  ( MulRQ     ),
        .MulAK  ( MulAK     ),
        .MulCmpl( MulCmpl   ),
        .MulInB ( AluInB    ),
        .MulInC ( AluInC    ),
        .MulOut ( ResMul0   )
    );
    //======== ResMul ======================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            ResMul <= 32'h0;
        end
        else if(MulCmpl) begin
            ResMul <= ResMul0;
        end
    end

    //==========================================
    //========  ResALU & Flags  ================
    //==========================================
    //======== FwdData     =================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            AddSubSel <= 1'h0;
        end
        else if(GoAhead) begin
            AddSubSel <= ((Func0 == `FUNC_ADD ) | (Func0 == `FUNC_SUB) | (Func0 == `FUNC_MOV));
        end
    end
    assign  FwdData = (AddSubSel)? ResAddSub[31:0] : ResLogic; // 20100327 add

    //======== FwdDataValid ======================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)        FwdDataValid <= 1'h0;
        else if(GoAhead) FwdDataValid <= AluAct0 & RfWe0 & ((Func0 == `FUNC_ADD ) | (Func0 == `FUNC_SUB ) 
                                                          | (Func0 == `FUNC_MOV ) | (Func0 == `FUNC_MOVL)
                                                          | (Func0 == `FUNC_MOVH) | (Func0 == `FUNC_AND )
                                                          | (Func0 == `FUNC_OR)   | (Func0 == `FUNC_XOR ));
    end
    //======== ResALU =================
    //always @(Func or ResAddSub or ResSftDiv or ResMul or ResLogic) begin // 20100327 remove
    //    if     ((Func == `FUNC_DIVU) | (Func == `FUNC_DIV) | (Func == `FUNC_MODU)
    //          | (Func == `FUNC_MOD ) | (Func == `FUNC_LSR) | (Func == `FUNC_LSL) | (Func == `FUNC_ASR )) ResALU = ResSftDiv;
    //    else if((Func == `FUNC_ADD ) | (Func == `FUNC_SUB) | (Func == `FUNC_MOV))                        ResALU = ResAddSub[31:0];
    //    else if(Func == `FUNC_MUL)                                                                       ResALU = ResMul;
    //    else                                                                                             ResALU = ResLogic;
    //end

    always @(Func or ResSftDiv or ResMul or FwdData) begin
        if     ((Func == `FUNC_DIVU) | (Func == `FUNC_DIV) | (Func == `FUNC_MODU)
              | (Func == `FUNC_MOD ) | (Func == `FUNC_LSR) | (Func == `FUNC_LSL) | (Func == `FUNC_ASR )) ResALU = ResSftDiv;
        else if(Func == `FUNC_MUL)                                                                       ResALU = ResMul;
        else                                                                                             ResALU = FwdData;
    end
    //======== Flags     =================
    //assign  Zero  = (ResALU[31:0] == 32'h0); // 20100227 remove
    assign  Zero  = (AddSubSel)? AddSubIsZero : LogicIsZero; // 20100227 add
    //assign  Sign  =  ResALU[31]; // 20100227 remove
    assign  Sign  =  FwdData[31]; // 20100227 add
    assign  Carry =  ResAddSub[32];
    assign  Over  = ({AluInB[31], AluInC[31], ResAddSub[32:31]} == 4'b0001)
                  | ({AluInB[31], AluInC[31], ResAddSub[32:31]} == 4'b1110)
                  | ({AluInB[31], AluInC[31], ResAddSub[32:31]} == 4'b0111)
                  | ({AluInB[31], AluInC[31], ResAddSub[32:31]} == 4'b1000);
                  

endmodule

/*=======================================================================*
 *  TARGET PROJECT: PSM3(Prosessor Study Model3)                         *
 *  TARGET SYSTEM : *(PSM3 TARGET SYSTEM)                                *
 *  FUNCTION      : 32bit x 32bit = 32bit                                *
 *=======================================================================*/
module Mul32x32(
    Clk     ,
    RstN    ,
    MulRQ   ,
    MulAK   ,
    MulCmpl ,
    MulInB  ,
    MulInC  ,
    MulOut
);
    //==========================================================
    //======== port  ===========================================
    //==========================================================
    //======== SYSTEM ====================
    input           Clk     ;
    input           RstN    ;
    //======== FUNCTION ====================
    input           MulRQ   ; // Reqest
    output          MulAK   ; // Acknowledge
    output          MulCmpl ; // Complete
    input   [31:0]  MulInB  ; // Input Port B
    input   [31:0]  MulInC  ; // Input Port C
    output  [31:0]  MulOut  ; // Result

    //==========================================================
    //======== wire & reg ======================================
    //==========================================================
    reg             MulCmpl ; // Complete

    //==========================================================
    //======== BODY       ======================================
    //==========================================================
    assign MulAK  = MulRQ;
    assign MulOut = MulInB * MulInC;
`ifdef MUL1CYC
    assign MulCmpl = MulRQ;
`endif
`ifdef MUL2CYC
    always @(posedge Clk or negedge RstN) begin
        if(~RstN) MulCmpl <= 1'h0;
        else      MulCmpl <= MulAK;
    end
`endif

endmodule
