//=======================================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : *(PSM3 TARGET SYSTEM)
//  FUNCTION      : Psm3 Common Barrel Shifter
//
//                      Copyright (C) 2008 by Tomoyuki Inomoto
//=======================================================================
`define     FUNC_MOV    4'h0
`define     FUNC_MOVL   4'h1
`define     FUNC_MOVH   4'h2
`define     FUNC_ADD    4'h3
`define     FUNC_SUB    4'h4
`define     FUNC_AND    4'h5
`define     FUNC_OR     4'h6
`define     FUNC_XOR    4'h7
`define     FUNC_LSR    4'h8
`define     FUNC_LSL    4'h9
`define     FUNC_ASR    4'hA
`define     FUNC_MUL    4'hB
`define     FUNC_DIVU   4'hC
`define     FUNC_DIV    4'hD
`define     FUNC_MODU   4'hE
`define     FUNC_MOD    4'hF

module P3ComBSft(
    //======== SYSTEM ===============
    Clk                 ,
    RstN                ,
    //======== P0 =================
    P0ComBSftRQ         ,
    P0ComBSftAK         ,
    P0ComBSftCmpl       ,
    P0ComBSftInB        ,
    P0ComBSftInC        ,
    P0ComFunc           ,

    //======== P1 =================
    P1ComBSftRQ         ,
    P1ComBSftAK         ,
    P1ComBSftCmpl       ,
    P1ComBSftInB        ,
    P1ComBSftInC        ,
    P1ComFunc           ,

    //======== Result ===============
    ComBSftRes
);
    //==========================================================
    //======== port  ===========================================
    //==========================================================
    //======== SYSTEM ========================
    input           Clk                 ;
    input           RstN                ;

    //======== P0 =================
    input           P0ComBSftRQ         ; // ComBSft REQ
    output          P0ComBSftAK         ; // ComBSft ACK
    output          P0ComBSftCmpl       ; // ComBSft Complete
    input   [31:0]  P0ComBSftInB        ; // PortB Data
    input   [4:0]   P0ComBSftInC        ; // PortC Data
    input   [3:0]   P0ComFunc           ; // FUNCTION

    //======== P1 =================
    input           P1ComBSftRQ         ; // ComBSft REQ
    output          P1ComBSftAK         ; // ComBSft ACK
    output          P1ComBSftCmpl       ; // ComBSft Complete
    input   [31:0]  P1ComBSftInB        ; // PortB Data
    input   [4:0]   P1ComBSftInC        ; // PortC Data
    input   [3:0]   P1ComFunc           ; // FUNCTION

    //======== Result =================
    output  [31:0]  ComBSftRes          ; // Result of ComBSft

    //==========================================================
    //======== wire & reg ======================================
    //==========================================================
    reg     [31:0]  ComBSftRes0         ; // Inter Result
    wire    [31:0]  ComBSftInB          ; // PortB Data
    wire    [4:0]   ComBSftInC          ; // PortC Data
    reg     [2:0]   ComBSftInC1d        ; // PortC Data One Delay
    //reg     [1:0]   ComBSftInC1d        ; // PortC Data One Delay
    wire    [3:0]   ComFunc             ; // FUNCTION
    reg     [3:0]   ComFunc1d           ; // FUNCTION
    reg             P0ComBSftAK         ; // ComBSft ACK
    reg             P0ComBSftCmpl       ; // ComBSft Complete
    reg             P1ComBSftAK         ; // ComBSft ACK
    reg             P1ComBSftCmpl       ; // ComBSft Complete
    reg     [31:0]  ComBSftRes          ; // Result of ComBSft

    //==========================================================
    //======== BODY       ======================================
    //==========================================================
    //======== P0ComBSftAK and P1ComBSftAK ========================
    always @(P0ComBSftRQ or P1ComBSftRQ) begin
        if(P0ComBSftRQ) begin
            P0ComBSftAK = 1'b1;
            P1ComBSftAK = 1'b0;
        end
        else if(P1ComBSftRQ) begin
            P0ComBSftAK = 1'b0;
            P1ComBSftAK = 1'b1;
        end
        else begin
            P0ComBSftAK = 1'b0;
            P1ComBSftAK = 1'b0;
        end
    end
    //======== P0ComBSftCmpl ========================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN) P0ComBSftCmpl <= 1'h0;
        else      P0ComBSftCmpl <= P0ComBSftAK;
    end

    //======== P1ComBSftCmpl ========================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN) P1ComBSftCmpl <= 1'h0;
        else      P1ComBSftCmpl <= P1ComBSftAK;
    end

    //======== ComBSftRes0 =======================
    assign ComBSftInB = (P0ComBSftRQ)? P0ComBSftInB : P1ComBSftInB;
    assign ComBSftInC = (P0ComBSftRQ)? P0ComBSftInC : P1ComBSftInC;
    assign ComFunc    = (P0ComBSftRQ)? P0ComFunc : P1ComFunc;
    always @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            ComBSftRes0 <= 32'h0;
        end
        else begin
            case(ComFunc)
                `FUNC_LSR   : ComBSftRes0 <= ComBSftInB >> {ComBSftInC[4:3], 3'h0};
                `FUNC_LSL   : ComBSftRes0 <= ComBSftInB << {ComBSftInC[4:3], 3'h0};
                default     : ComBSftRes0 <= $unsigned($signed(ComBSftInB) >>> {ComBSftInC[4:3], 3'h0});
                //`FUNC_LSR   : ComBSftRes0 <= ComBSftInB >> {ComBSftInC[4:2], 2'h0};
                //`FUNC_LSL   : ComBSftRes0 <= ComBSftInB << {ComBSftInC[4:2], 2'h0};
                //default     : ComBSftRes0 <= $unsigned($signed(ComBSftInB) >>> {ComBSftInC[4:2], 2'h0});
            endcase
        end
    end

    //======== ComBSftRes =======================
    always @(posedge Clk) ComBSftInC1d <= ComBSftInC[2:0];
    //always @(posedge Clk) ComBSftInC1d <= ComBSftInC[1:0];
    always @(posedge Clk) ComFunc1d <= ComFunc;
    always @(ComFunc1d or ComBSftRes0 or ComBSftInC1d) begin
        case(ComFunc1d)
            `FUNC_LSR   : ComBSftRes = ComBSftRes0 >> ComBSftInC1d;
            `FUNC_LSL   : ComBSftRes = ComBSftRes0 << ComBSftInC1d;
            default     : ComBSftRes = $unsigned($signed(ComBSftRes0) >>> ComBSftInC1d);
        endcase
    end
endmodule
