//=======================================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : Psm3Sys (PSM3 TEST SYSTEM)
//  FUNCTION      : Data Cache For PSM3
//
//                      Copyright (C) 2008-2010 by Tomoyuki Inomoto
//=======================================================================
// 20100327:Lead data select logic renewal.
// 20100529:Cash->Cache (spell miss)
//=======================================================================

`define     DC_CLEAR0       4'h0
`define     DC_CLEAR1       4'h1
`define     DC_CLEAR2       4'h2
`define     DC_CLEAR3       4'h3

`define     DC_IDLE         4'h0
`define     DC_SNGL_RDRQ    4'h1
`define     DC_SNGL_RDWAIT  4'h2
`define     DC_SNGL_WRRQ    4'h3
`define     DC_BRST_RDRQ    4'h4
`define     DC_BRST_RDWAIT0 4'h5
`define     DC_BRST_RDWAIT1 4'h6
`define     DC_BRST_RDWAIT2 4'h7
`define     DC_BRST_RDWAIT3 4'h8
`define     DC_BRST_WRINI0  4'h9
`define     DC_BRST_WRINI1  4'hA
`define     DC_BRST_WRRQ    4'hB
`define     DC_BRST_WRWAIT1 4'hC
`define     DC_BRST_WRWAIT2 4'hD
`define     DC_BRST_WRWAIT3 4'hE
`define     DC_MSTR_AK      4'hF


module P3DCache4K32bWB(
    //======== SYSTEM ==============
    Clk             ,
    RstN            ,
    DCE             ,

    //======= Master Bus ================
    MstrRQ          ,
    MstrAK          ,
    MstrRW          ,
    MstrBE          ,
    MstrADR         ,
    MstrWD          ,
    MstrRD          ,

    MstrDCDrainRQ   ,
    MstrDCRe        ,
    MstrDCArea      ,
    MstrDCIndex     ,

    MstrLock        ,

    //======= External Bus ================
    ExCRQ           ,
    ExCAK           ,
    ExBRST          ,
    ExRW            ,
    ExBE            ,
    ExADR           ,
    ExWD            ,
    ExWDAK          ,
    ExRD            ,
    ExRDAK          ,

    //======== MONITOR ================
    CoreDataHit

);
    //==================================================
    //======== port  ===================================
    //==================================================
    //======== SYSTEM ==============
    input           Clk             ;
    input           RstN            ;
    input           DCE             ;

    //======= Master Bus ================
    input           MstrRQ          ;
    output          MstrAK          ;
    input           MstrRW          ;
    input   [3:0]   MstrBE          ;
    input   [27:0]  MstrADR         ;
    input   [31:0]  MstrWD          ;
    output  [31:0]  MstrRD          ;

    input           MstrDCDrainRQ   ;
    input           MstrDCRe        ; // Data Cache Read Enable 
    input   [27:23] MstrDCArea      ; // Data Cache Accsess Area
    input   [11:2]  MstrDCIndex     ; // Data Cache Index

    output          MstrLock        ; // InterLock Signal to Core

    //======= External Bus ================
    output          ExCRQ           ; // Command RQ
    input           ExCAK           ; // Command AK
    output          ExBRST          ; // BURST RQ
    output          ExRW            ; // Read/Write
    output  [3:0]   ExBE            ; // Byte Enable
    output  [27:0]  ExADR           ; // Address
    output  [31:0]  ExWD            ; // Write Data
    input           ExWDAK          ; // Write Data Ack
    input   [31:0]  ExRD            ; // Read Data
    input           ExRDAK          ; // Read Data Ack

    //======== MONITOR ================
    output          CoreDataHit     ;

    //==================================================
    //======== wire & reg ==============================
    //==================================================
    reg             ClearMode   ;
    reg     [3:0]   MainState   ;
    reg     [7:0]   ClrAdr      ;

    reg             ExCRQ       ;
    reg             ExBRST      ;
    reg             ExRW        ;
    reg  [3:0]      ExBE        ;
    reg  [27:0]     ExADR       ;

    wire            TagWe0      ;
    wire            TagWe1      ;
    wire            TagWe2      ;
    wire            TagWe3      ;
    wire            TagWe       ;
    wire    [7:0]   TagWrIndex  ;
    wire    [7:0]   TagRdIndex  ;
    wire            TagRe       ;
    wire    [14:0]  TagAdr      ;
    wire            EnaArea     ;
    wire            CacheEna    ;

    wire            ValidIn     ;
    wire            ValidOut    ;
    wire            DirtyIn     ;
    wire            DirtyOut    ;

    reg             DCE1d       ;
    wire            DceNegEdge  ;

    wire            SingleRead  ;
    wire            SingleWrite ;
    wire            Hit         ;
    wire            BurstRead   ;
    wire            BurstWrite  ;
    wire            CacheHit    ;
    wire            OverWrite   ;
    wire            ValidClear  ;

    reg     [1:0]   BAdr        ;
    reg     [2:0]   WBBAdr      ;
    reg     [2:0]   WBBAdr1d    ;
    wire    [31:0]  WriteBackWD ;
    reg     [31:0]  ExRDBuf     ;

    wire            CacheRe     ;
    wire    [9:0]   CacheRdIndex;

    wire            CacheWe     ;
    wire    [9:0]   CacheWrIndex;

    wire    [31:0]  CacheRD     ;

    wire    [31:0]  CacheWD     ;
    reg     [31:0]  CacheWD1d   ;
    reg     [11:2]  MstrADR1d   ;
    reg             CacheWe1d   ;
    wire            WriteBufHit ;

    //====================================================================
    //======== BODY ======================================================
    //====================================================================
    //======== MONITOR  ================================
    assign  CoreDataHit = CacheHit;

    //====== DceNegEdge ===========================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)   DCE1d <= 1'h0;
        else        DCE1d <= DCE;
    end
    assign DceNegEdge = ({DCE1d, DCE} == 2'b10);

    //==================================================
    //======== Main State    ===========================
    //==================================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            ClearMode   <= 1'b1;
            MainState   <= `DC_CLEAR0;
            ClrAdr      <= 8'h0;
            ExCRQ       <= 1'b0;
            ExBRST      <= 1'b0;
            ExRW        <= 1'b0;
            ExBE        <= 4'h0;
            ExADR       <= 28'h0;
            BAdr        <= 2'h0;
            WBBAdr      <= 3'h4;
            ExRDBuf     <= 32'h0;
        end
        //===== COMMON MODE (WRITE BACK)=======================
        else if(MainState == `DC_BRST_WRINI0) begin
                MainState   <= `DC_BRST_WRINI1;
                if(~WBBAdr[2])  WBBAdr  <= WBBAdr + 1'h1;
        end
        else if(MainState == `DC_BRST_WRINI1) begin
                MainState   <= `DC_BRST_WRRQ;
                ExCRQ       <= 1'b1;
                ExBRST      <= 1'b1;
                ExRW        <= 1'b1;
                ExBE        <= 4'hF;
                if(ClearMode)   ExADR   <= {TagAdr, ClrAdr[7:0],   4'h0};
                else            ExADR   <= {TagAdr, MstrADR[11:4], 4'h0};
                BAdr        <= 2'h0;
                if(~WBBAdr[2])  WBBAdr  <= WBBAdr + 1'h1;
        end
        else if(MainState == `DC_BRST_WRRQ) begin
            if(~WBBAdr[2])  WBBAdr  <= WBBAdr + 1'h1;
            if(ExCAK) begin
                MainState   <= `DC_BRST_WRWAIT1;
                ExCRQ       <= 1'b0;
                BAdr        <= 2'h1;
            end
        end
        else if(MainState == `DC_BRST_WRWAIT1) begin
            if(~WBBAdr[2])  WBBAdr  <= WBBAdr + 1'h1;
            if(ExWDAK) begin
                MainState   <= `DC_BRST_WRWAIT2;
                BAdr        <= 2'h2;
            end
        end
        else if(MainState == `DC_BRST_WRWAIT2) begin
            if(~WBBAdr[2])  WBBAdr  <= WBBAdr + 1'h1;
            if(ExWDAK) begin
                MainState   <= `DC_BRST_WRWAIT3;
                BAdr        <= 2'h3;
            end
        end
        else if(MainState == `DC_BRST_WRWAIT3) begin
            if(~WBBAdr[2])  WBBAdr  <= WBBAdr + 1'h1;
            if(ExWDAK) begin
                BAdr        <= 2'h0;
                if(ClearMode) begin
                    MainState   <= `DC_CLEAR2;
                end
                else if(MstrDCDrainRQ) begin
                    MainState   <= `DC_MSTR_AK;
                end
                else begin
                    MainState   <= `DC_BRST_RDRQ;
                    ExCRQ       <= 1'b1;
                    ExBRST      <= 1'b1;
                    ExRW        <= 1'b0;
                    ExBE        <= 4'hF;
                    ExADR       <= {MstrADR[27:4],4'h0};
                end
            end
        end
        //===== CLEAR MODE ====================================
        else if(ClearMode) begin
            if(MainState == `DC_CLEAR0) begin
                MainState <= `DC_CLEAR1;
            end
            else if(MainState == `DC_CLEAR1) begin
                if(DirtyOut) begin
                    MainState   <= `DC_BRST_WRINI0;
                    WBBAdr      <= 3'h0;
                end
                else begin
                    MainState <= `DC_CLEAR2;
                end
            end
            else if(MainState == `DC_CLEAR2) begin
                MainState <= `DC_CLEAR3;
                ClrAdr <= ClrAdr + 1'b1;
            end
            else if(MainState == `DC_CLEAR3) begin
                if(ClrAdr == 8'h0) begin
                    MainState <= `DC_IDLE;
                    ClearMode <= 1'b0;
                end
                else begin
                    MainState <= `DC_CLEAR1;
                end
            end
        end
        //===== NOMAL MODE ====================================
        else begin
            if(MainState == `DC_IDLE) begin //========= IDLE ==============
                if(DceNegEdge) begin
                    MainState   <= `DC_CLEAR0;
                    ClearMode   <= 1'b1;
                    ClrAdr      <= 8'h0;
                end
                else if(SingleRead) begin
                    MainState   <= `DC_SNGL_RDRQ;
                    ExCRQ       <= 1'b1;
                    ExBRST      <= 1'b0;
                    ExRW        <= 1'b0;
                    ExBE        <= MstrBE;
                    ExADR       <= MstrADR;
                end
                else if(SingleWrite) begin
                    MainState   <= `DC_SNGL_WRRQ;
                    ExCRQ       <= 1'b1;
                    ExBRST      <= 1'b0;
                    ExRW        <= 1'b1;
                    ExBE        <= MstrBE;
                    ExADR       <= MstrADR;
                end
                else if(BurstRead) begin
                    MainState   <= `DC_BRST_RDRQ;
                    ExCRQ       <= 1'b1;
                    ExBRST      <= 1'b1;
                    ExRW        <= 1'b0;
                    ExBE        <= 4'hF;
                    ExADR       <= {MstrADR[27:4],4'h0};
                    BAdr        <= 2'h0;
                end
                else if(BurstWrite) begin
                    MainState   <= `DC_BRST_WRINI0;
                    WBBAdr      <= 3'h0;
                end
                else if(ValidClear) begin
                    MainState   <= `DC_MSTR_AK;
                end
            end
            else if(MainState == `DC_SNGL_RDRQ) begin //========= Single Read ==============
                if(ExCAK & ExRDAK) begin
                    MainState   <= `DC_MSTR_AK;
                    ExCRQ       <= 1'b0;
                    ExRDBuf     <= ExRD;
                end
                else if(ExCAK) begin
                    MainState   <= `DC_SNGL_RDWAIT;
                    ExCRQ       <= 1'b0;
                end
            end
            else if(MainState == `DC_SNGL_RDWAIT) begin
                if(ExRDAK) begin
                    MainState   <= `DC_MSTR_AK;
                    ExRDBuf     <= ExRD;
                end
            end
            else if(MainState == `DC_SNGL_WRRQ) begin //========= Single Write ==============
                if(ExCAK) begin
                    //MainState   <= `DC_MSTR_AK;
                    MainState   <= `DC_IDLE;
                    ExCRQ       <= 1'b0;
                end
            end
            else if(MainState == `DC_BRST_RDRQ) begin //========= Burst Read ==============
                if(ExCAK & ExRDAK) begin
                    MainState   <= `DC_BRST_RDWAIT1;
                    ExCRQ       <= 1'b0;
                    if(MstrADR[3:2] == 2'h0) ExRDBuf <= ExRD;
                    BAdr      <= 2'h1;
                end
                else if(ExCAK) begin
                    MainState   <= `DC_BRST_RDWAIT0;
                    ExCRQ       <= 1'b0;
                end
            end
            else if(MainState == `DC_BRST_RDWAIT0) begin
                if(ExRDAK) begin
                    MainState   <= `DC_BRST_RDWAIT1;
                    if(MstrADR[3:2] == 2'h0) ExRDBuf <= ExRD;
                    BAdr      <= 2'h1;
                end
            end
            else if(MainState == `DC_BRST_RDWAIT1) begin
                if(ExRDAK) begin
                    MainState   <= `DC_BRST_RDWAIT2;
                    if(MstrADR[3:2] == 2'h1) ExRDBuf <= ExRD;
                    BAdr      <= 2'h2;
                end
            end
            else if(MainState == `DC_BRST_RDWAIT2) begin
                if(ExRDAK) begin
                    MainState   <= `DC_BRST_RDWAIT3;
                    if(MstrADR[3:2] == 2'h2) ExRDBuf <= ExRD;
                    BAdr      <= 2'h3;
                end
            end
            else if(MainState == `DC_BRST_RDWAIT3) begin
                if(ExRDAK) begin
                    MainState   <= `DC_MSTR_AK;
                    ExBRST      <= 1'b0;
                    if(MstrADR[3:2] == 2'h3) ExRDBuf <= ExRD;
                    BAdr      <= 2'h0;
                end
            end
            else begin //========= MstrAK ==============
                MainState   <= `DC_IDLE;
            end
        end
    end

    //==================================================
    //======== WriteBackDataBuf ========================
    //==================================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)   WBBAdr1d <= 3'h4;
        else        WBBAdr1d <= WBBAdr;
    end
    WriteBackDataBuf WBDBuf(
        .Clk        (  Clk           ),
        .WriteAdr   (  WBBAdr1d[1:0] ),
        .WraiteEna  ( ~WBBAdr1d[2]   ),
        .WriteData  (  CacheRD       ),
        .ReadAdr    (  BAdr          ),
        .ReadData   (  WriteBackWD   )
    );

    //==================================================
    //======== External Bus  ===========================
    //==================================================
    assign  ExWD = (MainState == `DC_SNGL_WRRQ)? MstrWD : WriteBackWD;

    //==================================================
    //======== Master Bus     ==========================
    //==================================================
    assign  MstrAK = CacheHit | (MainState == `DC_MSTR_AK) | (ExCAK & (MainState == `DC_SNGL_WRRQ));
    assign  WriteBufHit = CacheWe1d & (MstrADR1d == MstrADR[11:2]);
    assign  MstrRD = (MainState == `DC_IDLE)? (WriteBufHit)? CacheWD1d : CacheRD : ExRDBuf;
    assign  MstrLock = ClearMode;

    //====== CacheEna ===========================
    P3DCacheEnaAreaTbl EnaAreaTbl(
        .Clk            ( Clk           ),
        .RstN           ( RstN          ),
        .MstrDCRe       ( MstrDCRe      ),
        .MstrDCArea     ( MstrDCArea    ),
        .EnaArea        ( EnaArea       )
    );
    assign  CacheEna = ~ClearMode & DCE & EnaArea;

    //====== CacheHit ===========================
    assign  Hit         =  ValidOut & (MstrADR[26:12] == TagAdr);
    assign  SingleRead  =  MstrRQ & ~MstrRW &  ~CacheEna;
    assign  SingleWrite =  MstrRQ &  MstrRW & (~CacheEna | (CacheEna & ~Hit));
    assign  BurstRead   =  MstrRQ & ~MstrRW &   CacheEna & ~Hit & ~DirtyOut;
    assign  BurstWrite  =((MstrRQ & ~MstrRW &   CacheEna & ~Hit) | MstrDCDrainRQ) & DirtyOut;
    assign  ValidClear  =  MstrDCDrainRQ & ~DirtyOut;
    assign  CacheHit    = MstrRQ & CacheEna & Hit;
    assign  OverWrite = CacheHit &  MstrRW;

    //====== Cache Memory Control ===========================
    assign  TagWe0   = ExRDAK & (MainState == `DC_BRST_RDWAIT3);        // End of ExMem Read
    assign  TagWe1   = ClearMode & (MainState == `DC_CLEAR2);           // Clear
    assign  TagWe2   = MstrDCDrainRQ & (MainState == `DC_BRST_WRINI1);  // Drain && Dirty
    assign  TagWe3   = ValidClear;                                      // Drain && ~Dirty
    assign  TagWe    = TagWe0 | TagWe1 | TagWe2 | TagWe3 | OverWrite;
    assign  TagWrIndex = (ClearMode)? ClrAdr[7:0] : MstrADR[11:4];
    assign  TagRdIndex = (ClearMode)? ClrAdr[7:0] : MstrDCIndex[11:4];
    assign  TagRe    = MstrDCRe | ClearMode;
    assign  ValidIn  = (ClearMode | MstrDCDrainRQ)? 1'b0 : 1'b1;
    assign  DirtyIn  = (ClearMode | MstrDCDrainRQ)? 1'b0 : OverWrite;

    assign  CacheRe   = MstrDCRe | ~WBBAdr[2];
    assign  CacheRdIndex = (MstrDCRe)? MstrDCIndex[11:2] : (ClearMode)? {ClrAdr, WBBAdr[1:0]} : {MstrADR[11:4], WBBAdr[1:0]};

    assign  CacheWe   = OverWrite | (ExRDAK & ((MainState == `DC_BRST_RDRQ   ) |
                                              (MainState == `DC_BRST_RDWAIT0) |
                                              (MainState == `DC_BRST_RDWAIT1) |
                                              (MainState == `DC_BRST_RDWAIT2) |
                                              (MainState == `DC_BRST_RDWAIT3)));
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)   CacheWe1d <= 1'h0;
        else        CacheWe1d <= CacheWe;
    end
    assign  CacheWrIndex = (OverWrite)? MstrADR[11:2] : {MstrADR[11:4], BAdr};
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)   MstrADR1d <= 10'h0;
        else        MstrADR1d <= CacheWrIndex;
    end
    assign  CacheWD[31:24] = (OverWrite)? ((MstrBE[3])? MstrWD[31:24] : CacheRD[31:24]) : ExRD[31:24];
    assign  CacheWD[23:16] = (OverWrite)? ((MstrBE[2])? MstrWD[23:16] : CacheRD[23:16]) : ExRD[23:16];
    assign  CacheWD[15: 8] = (OverWrite)? ((MstrBE[1])? MstrWD[15: 8] : CacheRD[15: 8]) : ExRD[15: 8];
    assign  CacheWD[ 7: 0] = (OverWrite)? ((MstrBE[0])? MstrWD[ 7: 0] : CacheRD[ 7: 0]) : ExRD[ 7: 0];
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)   CacheWD1d <= 32'h0;
        else        CacheWD1d <= CacheWD;
    end

    //====== TAG RAM ===========================
    P3DtagRam DTag(
        .addra  ( TagRdIndex                         ),
        .addrb  ( TagWrIndex                         ),
        .clka   ( Clk                                ),
        .clkb   ( Clk                                ),
        .dinb   ( {DirtyIn, ValidIn, MstrADR[26:12]} ),
        .douta  ( {DirtyOut, ValidOut, TagAdr}       ),
        .ena    ( TagRe                              ),
        .web    ( TagWe                              )
    );

    //====== CASH RAM ===========================
    P3DCacheRam DCache(
        .addra  ( CacheRdIndex              ),
        .addrb  ( CacheWrIndex              ),
        .clka   ( Clk                       ),
        .clkb   ( Clk                       ),
        .dinb   ( CacheWD                   ),
        .ena    ( CacheRe                   ),
        .douta  ( CacheRD                   ),
        .web    ( CacheWe                   )
    );

endmodule
/*=======================================================================*
 *  TARGET PROJECT: PSM3(Prosessor Study Model3)                         *
 *  TARGET SYSTEM : Psm3Sys (PSM3 TEST SYSTEM)                           *
 *  FUNCTION      : CacheEnableAreaTable for Data Cache of PSM3          *
 *=======================================================================*/
`define     DCASH_ENABLE    1'b1
`define     DCASH_DISABLE   1'b0
module P3DCacheEnaAreaTbl(
    Clk             ,
    RstN            ,
    MstrDCRe        ,
    MstrDCArea      ,
    EnaArea
);
    //==================================================
    //======== port  ===================================
    //==================================================
    input           Clk             ;
    input           RstN            ;
    input           MstrDCRe        ; // Data Cache Read Enable
    input   [27:23] MstrDCArea      ;
    output          EnaArea         ;

    //==================================================
    //======== wire & reg ==============================
    //==================================================
    reg             EnaArea     ;

    //====================================================================
    //======== BODY ======================================================
    //====================================================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            EnaArea <= 1'b0;
        end
        else if(MstrDCRe) begin
            if(MstrDCArea[27]) begin
                // AREA:0x0800_0000 - 0x0BFF_FFFF (128-192MB) AREA2(Flash)
                // AREA:0x0C00_0000 - 0x0FFF_FFFF (192-256MB) AREA3(Peripheral)
                EnaArea <= `DCASH_DISABLE;
            end
            else begin
                case(MstrDCArea[26:23])
                    //========== AREA0(Mirror of AREA1) ===========================================
                    4'h0    : EnaArea <= `DCASH_ENABLE  ; // AREA:0x0000_0000-0x007F_FFFF (  0-  8MB)
                    4'h1    : EnaArea <= `DCASH_ENABLE  ; // AREA:0x0080_0000-0x00FF_FFFF (  8- 16MB)
                    4'h2    : EnaArea <= `DCASH_ENABLE  ; // AREA:0x0100_0000-0x017F_FFFF ( 16- 24MB)
                    4'h3    : EnaArea <= `DCASH_ENABLE  ; // AREA:0x0180_0000-0x01FF_FFFF ( 24- 32MB)
                    4'h4    : EnaArea <= `DCASH_ENABLE  ; // AREA:0x0200_0000-0x027F_FFFF ( 32- 40MB)
                    4'h5    : EnaArea <= `DCASH_ENABLE  ; // AREA:0x0280_0000-0x02FF_FFFF ( 40- 48MB)
                    4'h6    : EnaArea <= `DCASH_ENABLE  ; // AREA:0x0300_0000-0x037F_FFFF ( 48- 56MB)
                    4'h7    : EnaArea <= `DCASH_DISABLE ; // AREA:0x0380_0000-0x03FF_FFFF ( 56- 64MB)
                    //========== AREA1(SDRAM) =====================================================
                    4'h8    : EnaArea <= `DCASH_DISABLE ; // AREA:0x0400_0000-0x047F_FFFF ( 64- 72MB)
                    4'h9    : EnaArea <= `DCASH_DISABLE ; // AREA:0x0480_0000-0x04FF_FFFF ( 72- 80MB)
                    4'hA    : EnaArea <= `DCASH_DISABLE ; // AREA:0x0500_0000-0x057F_FFFF ( 80- 88MB)
                    4'hB    : EnaArea <= `DCASH_DISABLE ; // AREA:0x0580_0000-0x05FF_FFFF ( 88- 96MB)
                    4'hC    : EnaArea <= `DCASH_DISABLE ; // AREA:0x0600_0000-0x067F_FFFF ( 96-104MB)
                    4'hD    : EnaArea <= `DCASH_DISABLE ; // AREA:0x0680_0000-0x06FF_FFFF (104-112MB)
                    4'hE    : EnaArea <= `DCASH_DISABLE ; // AREA:0x0700_0000-0x077F_FFFF (112-120MB)
                    default : EnaArea <= `DCASH_DISABLE ; // AREA:0x0780_0000-0x07FF_FFFF (120-128MB)
                endcase
            end
        end
    end

endmodule

/*=======================================================================*
 *  TARGET PROJECT: PSM3(Prosessor Study Model3)                         *
 *  TARGET SYSTEM : Psm3Sys (PSM3 TEST SYSTEM)                           *
 *  FUNCTION      : TagRam for Data Cache of PSM3                        *
 *=======================================================================*/
module P3DtagRam(
    addra   ,
    addrb   ,
    clka    ,
    clkb    ,
    dinb    ,
    douta   ,
    ena     ,
    web
);
    //==================================================
    //======== port  ===================================
    //==================================================
    input   [7:0]   addra;
    input   [7:0]   addrb;
    input           clka;
    input           clkb;
    input   [16:0]  dinb;
    output  [16:0]  douta;
    input           ena;
    input           web;
    //==================================================
    //======== wire & reg ==============================
    //==================================================
    reg     [16:0]  mem[0:255];
    reg     [16:0]  douta;

    //====================================================================
    //======== BODY ======================================================
    //====================================================================
    always @(posedge clka) if(ena) douta <= mem[addra];
    always @(posedge clkb) if(web) mem[addrb] <= dinb;

endmodule

/*=======================================================================*
 *  TARGET PROJECT: PSM3(Prosessor Study Model3)                         *
 *  TARGET SYSTEM : Psm3Sys (PSM3 TEST SYSTEM)                           *
 *  FUNCTION      : CacheRam for Data Cache of PSM3                      *
 *=======================================================================*/
module P3DCacheRam(
    addra   ,
    addrb   ,
    clka    ,
    clkb    ,
    dinb    ,
    douta   ,
    ena     ,
    web
);
    //==================================================
    //======== port  ===================================
    //==================================================
    input   [9:0]   addra;
    input   [9:0]   addrb;
    input           clka;
    input           clkb;
    input   [31:0]  dinb;
    output  [31:0]  douta;
    input           ena;
    input           web;

    //==================================================
    //======== wire & reg ==============================
    //==================================================
    reg     [31:0]  mem[0:1023];
    reg     [31:0]  douta;

    //====================================================================
    //======== BODY ======================================================
    //====================================================================
    always @(posedge clka) if(ena) douta <= mem[addra];
    always @(posedge clkb) if(web) mem[addrb] <= dinb;

endmodule

/*=======================================================================*
 *  TARGET PROJECT: PSM3(Prosessor Study Model3)                         *
 *  TARGET SYSTEM : Psm3Sys (PSM3 TEST SYSTEM)                           *
 *  FUNCTION      : BufferRam for Write Back Data                        *
 *=======================================================================*/
module WriteBackDataBuf(
    Clk         ,
    WriteAdr    ,
    WraiteEna   ,
    WriteData   ,
    ReadAdr     ,
    ReadData
);
    //==================================================
    //======== port  ===================================
    //==================================================
    input           Clk ;
    input   [1:0]   WriteAdr    ;
    input           WraiteEna   ;
    input   [31:0]  WriteData   ;
    input   [1:0]   ReadAdr     ;
    output  [31:0]  ReadData    ;

    //==================================================
    //======== wire & reg ==============================
    //==================================================
    reg     [31:0]  mem[0:3];

    //====================================================================
    //======== BODY ======================================================
    //====================================================================
    always  @(posedge Clk) if(WraiteEna) mem[WriteAdr] <= WriteData;
    assign  ReadData = mem[ReadAdr];

endmodule
