//=======================================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : *(PSM3 TARGET SYSTEM)
//  FUNCTION      : Psm3 DECoder Unit
//
//                      Copyright (C) 2008-2010 by Tomoyuki Inomoto
//=======================================================================
// 20100327:UpDateAfter, PtrUpDate0, RfRdAdrB are removed
//         :Some signals about forwarding are added. (RegAFwdEna, RegBFwdEna, RegCFwdEna, FwdData)
// 20100529:Cash->Cache (spell miss)
//=======================================================================
module P3Decu(
    //======== SYSTEM ========================
    Clk             ,
    RstN            ,
    PID             ,
    ICE             ,
    DCE             ,
    IRQ             ,
    BusLockRQ       ,
    BusLockAK       ,
    LockByDCache    ,

    //======== from/to IFU =================
    DecInstRQ       ,
    DecInstAK       ,
    PC0             ,
    Inst            ,
    Break0          ,
    NextPcSel       ,
    PcWrited        ,
    JmpAdr          ,
    BrPredict0      , // 20100403 add

    //======== from/to EXU =================
    GoAhead         ,
    PC1             ,
    RfWrAdr0        ,
    //RfRdAdrB        , // 20100327
    RfWe0           ,
    RfRdDataA       ,
    RfRdDataB       ,
    RfRdDataC       ,
    RfRdDC          ,
    Imm16           ,
    ImmSel          ,

    RfWrAdrEx       ,
    RfWeEx          ,
    RfWrDEx         ,

    Func0           ,
    AluAct0         ,
    UpDateCV        ,
    UpDateSZ        ,
    Over            ,
    Carry           ,
    Sign            ,
    Zero            ,
    AluWait         ,
    FwdData         , // 20100327 add
    FwdDataValid    , // 20100327 add

    DataRQ0         ,
    DataRW0         ,
    DataSIZE0       ,
    //DrainRQ0        , // 20100402 remove
    //UpDateAfter     , // 20100327 remove
    //PtrUpDate0      , // 20100327 remove
    LsuWait         ,

    //======== DEBUG================
    Break           ,
    DbgRq           ,
    DbgRegRq        ,
    DbgEna          ,
    DbgRegWe        ,
    DbgRegAdr       ,
    DbgWD           ,
    DbgRD           ,
    Bubble
);
    //============================================================
    //======== port  =============================================
    //============================================================
    //======== SYSTEM ========================
    input           Clk             ; // Clock
    input           RstN            ; // Reset (L-Active)
    input           PID             ; // Processor ID
    output          ICE             ; // Inst Cache Enable
    output          DCE             ; // Data Cache Enable
    input           IRQ             ; // Interrupt Reqest
    output          BusLockRQ       ; // Bus Lock Reqest
    input           BusLockAK       ; // Bus Lock ACK
    input           LockByDCache    ; // InterLock Signal from DCache

    //======== from/to IFU =================
    output          DecInstRQ       ; // Program Counter on IFU
    input           DecInstAK       ; // Instruction Reqest From DECU
    input   [25:0]  PC0             ; // Ack of Instruction Reqest to DECU
    input   [31:0]  Inst            ; // Instruction Data
    input           Break0          ; // Break0
    output  [1:0]   NextPcSel       ; // 00:IncPC, 01:JmpAdr, 1*:Vector
    input           PcWrited        ; // PC is OverWrited by Debuger
    output  [25:0]  JmpAdr          ; // Jump Address
    input           BrPredict0   ; // Branch Prediction act // 20100403

    //======== from/to EXU =================
    output          GoAhead         ; // Go Ahead Pipeline
    output  [25:0]  PC1             ; // Program Counter at DECU
    output  [4:0]   RfWrAdr0        ; // Register file Write Address
    //output  [3:0]   RfRdAdrB        ; // Register file Read Address B // 20100327
    output          RfWe0           ; // Register file Write Enable
    output  [31:0]  RfRdDataA       ; // PortA Read Data
    output  [31:0]  RfRdDataB       ; // PortB Read Data
    output  [31:0]  RfRdDataC       ; // PortC Read Data
    output  [31:0]  RfRdDC          ; // GPR PortC Read Data
    output  [15:0]  Imm16           ; // 16bit Immdiate
    output          ImmSel          ; // Immdiate Select

    input   [4:0]   RfWrAdrEx       ; // Register file Write Address
    input           RfWeEx          ; // Register file Write Enable
    input   [31:0]  RfWrDEx         ; // Register file Write Data

    output  [3:0]   Func0           ; // ALU FUNCTION
    output          AluAct0         ; // ALU ACTIVE
    input           UpDateCV        ; // UpDate Carry & Over
    input           UpDateSZ        ; // UpDate Sign & Zero
    input           Over            ; // Flag
    input           Carry           ; // Flag
    input           Sign            ; // Flag
    input           Zero            ; // Flag
    input           AluWait         ; // ALU Wait
    input   [31:0]  FwdData         ; // Data for forwarding // 20100327
    input           FwdDataValid    ; // Data for forwarding valid // 20100327

    output          DataRQ0         ; // DATA BUS REQEST (1:REQ)
    output          DataRW0         ; // DATA BUS READ/WRITE (1:WRITE)
    output  [2:0]   DataSIZE0       ; // DATA BUS DATA SIZE (000:WORD, 010:HALF, 011:Signed HALF, 100:BYTE, 101:Signed Half)
    //output          DrainRQ0        ; // Data Cache Drain RQ (for Write Back Cache) // 20100402 remove
    //output          UpDateAfter     ; // Pointer UpDate After Access  // 20100327 remove
    //output          PtrUpDate0      ; // Pointer UpDate Flag  // 20100327 remove
    input           LsuWait         ; // LSU Wait

    //======== DEBUG================
    output          Break           ; // BREAK Instruction Detect Signal to Debugger
    input           DbgRq           ; // Debug Reqest from Debugger
    input           DbgRegRq        ; // Debug Reg Access Request from Debugger
    output          DbgEna          ; // Debug Enable to Debugger
    input           DbgRegWe        ; // Register Write Enable from Debugger
    input   [4:0]   DbgRegAdr       ; // Register Address from Debugger
    input   [31:0]  DbgWD           ; // Register Write Data from Debugger
    output  [31:0]  DbgRD           ; // Register Read  Data to   Debugger
    output          Bubble          ; // Bubble

    //============================================================
    //======== wire & reg ========================================
    //============================================================
    reg     [31:0]  IR              ;
    reg             BrPredict       ; // Branch Prediction act // 20100403
    reg             Break           ;
    reg             IrqIn           ;
    reg     [25:0]  PC1             ; // Current Program Counter Value
    reg     [25:0]  JmpAdr          ;
    wire            RegWeMask       ; // 1:Mask, 0:NotMask
    wire    [1:0]   JmpAdrMode      ; // 00:PC + Imm, 01:PC+REG, 10:REG
    wire    [3:0]   RfRdAdrA        ; // Register file Read Address A
    wire    [3:0]   RfRdAdrB        ; // Register file Read Address B // 20100327
    wire    [4:0]   RfRdAdrC        ; // Register file Read Address C
    wire            EscapeSR        ; // EscapeSR
    wire            ReturnSR        ; // ReturnSR
    wire            SetIE           ; // SET   IE@SR
    wire            ClrIE           ; // CLEAR IE@SR
    wire            SetICE          ; // SET   ICE@SR
    wire            ClrICE          ; // CLEAR ICE@SR
    wire            SetDCE          ; // SET   DCE@SR
    wire            ClrDCE          ; // CLEAR DCE@SR
    wire            SetBL           ; // SET   BL@SR
    wire            ClrBL           ; // CLEAR BL@SR
    wire    [7:0]   SR              ; //0:Z, 1:S, 2:C, 3:V, 4:IE, 5:ICE, 6:DCE
    wire            IEforID         ; // IrqEnable for Instruction Decorder
    wire            RegAFwdEna      ; // Reg A Forwarding Enable // 20100327 add
    wire            RegBFwdEna      ; // Reg B Forwarding Enable // 20100327 add
    wire            RegCFwdEna      ; // Reg C Forwarding Enable // 20100327 add
    wire    [31:0]  RfRdDA          ; // GPR PortA Read Data

    //============================================================
    //======== BODY       ========================================
    //============================================================
    //======== ICE ======================
    assign ICE = SR[5];

    //======== DCE ======================
    assign DCE = SR[6];

    //======== BusLockRQ ================
    assign BusLockRQ = SR[7];

    //======== IR & Break ======================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            IR      <= 32'hFFFF_FFFF; // NOP
            Break   <= 1'b0;
            BrPredict <= 1'b0; // 20100403
        end
        else if(DecInstAK) begin
            IR      <= Inst;
            Break   <= (Break0 & ~Bubble);
            BrPredict <= BrPredict0; // 20100403
        end
    end

    //======== Imm16 ================
    assign Imm16 = IR[15:0];

    //======== PC1 ======================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)          PC1 <= 26'h0;
        else if(DecInstAK) PC1 <= PC0;
    end

    //======== IrqIn =======================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)          IrqIn <= 1'h0;
        else if(DecInstAK) IrqIn <= IRQ;
        //else if(GoAhead) IrqIn <= IRQ;
    end

    //======== JmpAdr ======================
    //always  @(JmpAdrMode or PC1 or IR or RfRdDataA) begin // 20100327 remove
    //    case(JmpAdrMode)
    //        2'b00   : JmpAdr = PC1 + {IR[23:0],        2'h0};
    //        2'b01   : JmpAdr = PC1 + {RfRdDataA[25:2], 2'h0};
    //        default : JmpAdr =       {RfRdDataA[25:2], 2'h0};
    //    endcase
    //end
    always  @(JmpAdrMode or PC1 or IR or RfRdDA) begin // 20100327 add
        case(JmpAdrMode)
            2'b00   : JmpAdr = PC1 + {IR[23:0],     2'h0};
            2'b01   : JmpAdr = PC1 + {RfRdDA[25:2], 2'h0};
            default : JmpAdr =       {RfRdDA[25:2], 2'h0};
        endcase
    end

    //============================================================
    //==== Psm3 PipeLine Controler  ==============================
    //============================================================
    P3PlCtrl P3PlCtrl(
        .Clk             ( Clk             ),
        .RstN            ( RstN            ),
        .LockByDCache    ( LockByDCache    ),
        .DecInstRQ       ( DecInstRQ       ),
        .DecInstAK       ( DecInstAK       ),
        .IR              ( IR[31:28]       ),
        .RfWrAdrEx       ( RfWrAdrEx[3:0]  ),
        .RfWeEx          ( RfWeEx          ),
        .RfRdAdrA        ( RfRdAdrA        ),
        .RfRdAdrB        ( RfRdAdrB        ),
        .RfRdAdrC        ( RfRdAdrC[3:0]   ),
        //.UpDateCV        ( UpDateCV        ), // 20100327 remove
        //.UpDateSZ        ( UpDateSZ        ), // 20100327 remove
        .AluWait         ( AluWait         ),
        .LsuWait         ( LsuWait         ),
        .DbgRq           ( DbgRq           ),
        .DbgEna          ( DbgEna          ),
        .GoAhead         ( GoAhead         ),
        .RegWeMask       ( RegWeMask       ),
        .FwdDataValid    ( FwdDataValid    ),  // 20100327 add
        .RegAFwdEna      ( RegAFwdEna      ),  // 20100327 add
        .RegBFwdEna      ( RegBFwdEna      ),  // 20100327 add
        .RegCFwdEna      ( RegCFwdEna      )   // 20100327 add
    );

    //============================================================
    //==== Psm3 Instruction Decoder ==============================
    //============================================================
    P3Id P3Id(
        .Clk             ( Clk             ),
        .RstN            ( RstN            ),
        .GoAhead         ( GoAhead         ),
        .DecInstAK       ( DecInstAK       ),
        //.Inst            ( Inst            ), // 20100327
        .IR              ( IR              ),
        .BrPredict       ( BrPredict       ), // 20100403 add
        .SR              ( SR[3:0]         ),
        .UpDateCV        ( UpDateCV        ), // 20100327 add
        .UpDateSZ        ( UpDateSZ        ), // 20100327 add
        .FwdOver         ( Over            ), // 20100327 add
        .FwdCarry        ( Carry           ), // 20100327 add
        .FwdSign         ( Sign            ), // 20100327 add
        .FwdZero         ( Zero            ), // 20100327 add
        .IEforID         ( IEforID         ),
        .IrqIn           ( IrqIn           ),
        .NextPcSel       ( NextPcSel       ),
        .PcWrited        ( PcWrited        ),
        .RfWrAdr0        ( RfWrAdr0        ),
        .RfWe0           ( RfWe0           ),
        .JmpAdrMode      ( JmpAdrMode      ),
        .RfRdAdrA        ( RfRdAdrA        ),
        .RfRdAdrB        ( RfRdAdrB        ),
        .RfRdAdrC        ( RfRdAdrC        ),
        .ImmSel          ( ImmSel          ),
        .Func0           ( Func0           ),
        .AluAct0         ( AluAct0         ),
        .DataRQ0         ( DataRQ0         ),
        .DataRW0         ( DataRW0         ),
        .DataSIZE0       ( DataSIZE0       ),
        //.DrainRQ0        ( DrainRQ0        ), // 20100402 remove
        //.UpDateAfter     ( UpDateAfter     ), // 20100327 remove
        //.PtrUpDate0      ( PtrUpDate0      ), // 20100327 remove
        .EscapeSR        ( EscapeSR        ),
        .ReturnSR        ( ReturnSR        ),
        .SetIE           ( SetIE           ),
        .ClrIE           ( ClrIE           ),
        .SetICE          ( SetICE          ),
        .ClrICE          ( ClrICE          ),
        .SetDCE          ( SetDCE          ),
        .ClrDCE          ( ClrDCE          ),
        .SetBL           ( SetBL           ),
        .ClrBL           ( ClrBL           ),
        .Bubble          ( Bubble          )
        //.RegAFwdEna      ( RegAFwdEna      ),  // 20100327
        //.RegBFwdEna      ( RegBFwdEna      ),  // 20100327
        //.RegCFwdEna      ( RegCFwdEna      )   // 20100327
    );

    //============================================================
    //==== Psm3 Register File       ==============================
    //============================================================
    P3Rf P3Rf(
        .Clk             ( Clk             ),
        .RstN            ( RstN            ),
        .BusLockAK       ( BusLockAK       ),
        .PID             ( PID             ),
        .GoAhead         ( GoAhead         ),
        .DecInstAK       ( DecInstAK       ),
        .RegWeMask       ( RegWeMask       ),
        .PC0             ( PC0             ),
        .PC1             ( PC1             ),
        .RfWrAdrEx       ( RfWrAdrEx       ),
        .RfWeEx          ( RfWeEx          ),
        .RfWrDEx         ( RfWrDEx         ),
        .RfRdAdrA        ( RfRdAdrA        ),
        .RfRdAdrB        ( RfRdAdrB        ),
        .RfRdAdrC        ( RfRdAdrC        ),
        .EscapeSR        ( EscapeSR        ),
        .ReturnSR        ( ReturnSR        ),
        .SetIE           ( SetIE           ),
        .ClrIE           ( ClrIE           ),
        .SetICE          ( SetICE          ),
        .ClrICE          ( ClrICE          ),
        .SetDCE          ( SetDCE          ),
        .ClrDCE          ( ClrDCE          ),
        .SetBL           ( SetBL           ),
        .ClrBL           ( ClrBL           ),
        .SR              ( SR              ),
        .IEforID         ( IEforID         ),
        .UpDateCV        ( UpDateCV        ),
        .UpDateSZ        ( UpDateSZ        ),
        .Over            ( Over            ),
        .Carry           ( Carry           ),
        .Sign            ( Sign            ),
        .Zero            ( Zero            ),
        .RfRdDataA       ( RfRdDataA       ),
        .RfRdDataB       ( RfRdDataB       ),
        .RfRdDataC       ( RfRdDataC       ),
        .RfRdDA          ( RfRdDA          ),  // 20100327
        .RfRdDC          ( RfRdDC          ),
        .RegAFwdEna      ( RegAFwdEna      ),  // 20100327
        .RegBFwdEna      ( RegBFwdEna      ),  // 20100327
        .RegCFwdEna      ( RegCFwdEna      ),  // 20100327
        .FwdData         ( FwdData         ),  // 20100327
        .DbgRegRq        ( DbgRegRq        ),
        .DbgRegWe        ( DbgRegWe        ),
        .DbgRegAdr       ( DbgRegAdr       ),
        .DbgWD           ( DbgWD           ),
        .DbgRD           ( DbgRD           )
    );

endmodule
