//=======================================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : *(PSM3 TARGET SYSTEM)
//  FUNCTION      : Psm3 Exe Unit
//
//                      Copyright (C) 2008-2010 by Tomoyuki Inomoto
//=======================================================================
// 20100327:UpDateAfter, PtrUpDate0, PtrUpDate, RfRdAdrB1, RfRdAdrB are removed
//         :Some signals about forwarding are added. (FwdData)
// 20100529:Cash->Cache (spell miss)
//=======================================================================
module P3Exu(
    //======== SYSTEM ========================
    Clk             ,
    RstN            ,

    //======== from/to EXU =================
    GoAhead         ,
    PC1             ,
    RfWrAdr0        ,
    //RfRdAdrB        , // 20100327
    RfWe0           ,
    RfRdDataA       ,
    RfRdDataB       ,
    RfRdDataC       ,
    RfRdDC          ,
    Imm16           ,
    ImmSel          ,

    RfWrAdrEx       ,
    RfWeEx          ,
    RfWrDEx         ,

    Func0           ,
    AluAct0         ,
    UpDateCV        ,
    UpDateSZ        ,
    Over            ,
    Carry           ,
    Sign            ,
    Zero            ,
    AluWait         ,
    FwdData         , // 20100327 add
    FwdDataValid    , // 20100327 add
    
    DataRQ0         ,
    DataRW0         ,
    DataSIZE0       ,
    //DrainRQ0        , // 20100403 remove
    //UpDateAfter     , // 20100327 remove
    //PtrUpDate0      , // 20100327 remove
    LsuWait         ,

    //======= to/from Common Unit ======
    ComInB          ,
    ComInC          ,
    ComFunc         ,

    ComBSftRQ       ,
    ComBSftAK       ,
    ComBSftCmpl     ,
    ComBSftRes      ,

    ComDivRQ        ,
    ComDivAK        ,
    ComDivCmpl      ,
    ComDivRes       ,

    //======== DATA BUS ==================
    DataRQ          ,
    DataAK          ,
    DataRW          ,
    DataBE          ,
    DataADR         ,
    DataWD          ,
    DataRD          ,

    //======== DATA Cache =================
    //DCacheDrainRQ    , // 20100403
    DCacheRe        ,
    DCacheArea      ,
    DCacheIndex     ,

    //======== DEBUG================
    DbgRegRq        ,
    DbgWD
);
    //============================================================
    //======== port  =============================================
    //============================================================
    //======== SYSTEM ========================
    input           Clk             ; // Clock
    input           RstN            ; // Reset (L-Active)

    //======== from/to EXU =================
    input           GoAhead         ; // Go Ahead Pipeline
    input   [25:0]  PC1             ; // Program Counter at DECU
    input   [4:0]   RfWrAdr0        ; // Register file Write Address
    //input   [3:0]   RfRdAdrB        ; // Register file Read Address B // 20100327
    input           RfWe0           ; // Register file Write Enable
    input   [31:0]  RfRdDataA       ; // PortA Read Data
    input   [31:0]  RfRdDataB       ; // PortB Read Data
    input   [31:0]  RfRdDataC       ; // PortC Read Data
    input   [31:0]  RfRdDC          ; // GPR PortC Read Data
    input   [15:0]  Imm16           ; // 16bit Immdiate
    input           ImmSel          ; // Immdiate Select

    output  [4:0]   RfWrAdrEx       ; // Register file Write Address
    output          RfWeEx          ; // Register file Write Enable
    output  [31:0]  RfWrDEx         ; // Register file Write Data

    input   [3:0]   Func0           ; // ALU FUNCTION
    input           AluAct0         ; // ALU ACTIVE
    output          UpDateCV        ; // UpDate Carry & Over
    output          UpDateSZ        ; // UpDate Sign & Zero
    output          Over            ; // Flag
    output          Carry           ; // Flag
    output          Sign            ; // Flag
    output          Zero            ; // Flag
    output          AluWait         ; // ALU Wait
    output  [31:0]  FwdData         ; // Data for forwarding // 20100327
    output          FwdDataValid    ; // Data for forwarding valid // 20100327

    input           DataRQ0         ; // DATA BUS REQEST (1:REQ)
    input           DataRW0         ; // DATA BUS READ/WRITE (1:WRITE)
    input   [2:0]   DataSIZE0       ; // DATA BUS DATA SIZE (000:WORD, 010:HALF, 011:Signed HALF, 100:BYTE, 101:Signed Half)
    //input           DrainRQ0        ; // Data Cache Drain RQ (for Write Back Cache)  // 20100403 remove
    //input           UpDateAfter     ; // Pointer UpDate After Access // 20100327 remove
    //input           PtrUpDate0      ; // Pointer UpDate Flag // 20100327 remove
    output          LsuWait         ; // LSU Wait

    //======= to/from Common Unit =================
    output  [31:0]  ComInB          ; // PortB Data
    output  [31:0]  ComInC          ; // PortC Data
    output  [3:0]   ComFunc         ; // FUNCTION

    output          ComBSftRQ       ; // Common Barrel Shifter REQ
    input           ComBSftAK       ; // Common Barrel Shifter ACK
    input           ComBSftCmpl     ; // Common Barrel Shifter Complete
    input   [31:0]  ComBSftRes      ; // Result of Common Barrel Shifter

    output          ComDivRQ        ; // Common Divisor REQ
    input           ComDivAK        ; // Common Divisor ACK
    input           ComDivCmpl      ; // Common Divisor Complete
    input   [31:0]  ComDivRes       ; // Result of Common Divisor

    //======== DATA BUS ==================
    output          DataRQ          ; // DATA BUS REQ
    input           DataAK          ; // DATA BUS ACK
    output          DataRW          ; // DATA BUS READ/WRITE (1:WRITE)
    output  [3:0]   DataBE          ; // DATA BUS BYTE ENABLE
    output  [27:0]  DataADR         ; // DATA BUS ADDRESS
    output  [31:0]  DataWD          ; // DATA BUS WRITE DATA
    input   [31:0]  DataRD          ; // DATA BUS READ DATA

    //======== DATA Cache =================
    //output          DCacheDrainRQ    ; // Data Cache Drain RQ (for Write Back Cache) // 20100403
    output          DCacheRe        ; // Data Cache Read Enable
    output  [27:23] DCacheArea      ; // Data Cache Accsess Area
    output  [11:2]  DCacheIndex     ; // Data Cache Index (4KByte)

    //======== DEBUG=====================
    input           DbgRegRq        ; // Debug Reg Access Request from Debugger
    input   [31:0]  DbgWD           ; // Register Write Data from Debugger

    //============================================================
    //======== wire & reg ========================================
    //============================================================
    wire    [31:0]  ResALU          ; // Result of ALU
    wire            AluAct          ; // ALU Active
    wire    [31:0]  LsuRD           ; // LSU DataBus/DadaCache Read Data
    wire            LsuAct          ; // LSU Active
    reg     [25:0]  PC2             ; // PC2
    reg     [4:0]   RfWrAdr1        ; // Register file Write Address
    //reg     [3:0]   RfRdAdrB1       ; // Register file Read Address B // 20100327
    reg             RfWe1           ; // Register file Write Enable
    reg     [31:0]  RfWrDEx         ; // Register file Write Data
    //wire            PtrUpDate       ; // Pointer UpDate Flag // 20100327
    wire    [27:0]  DataADR0        ; // Address Of data bus

    //============================================================
    //======== BODY       ========================================
    //============================================================
    //======== PC2 ====================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)           PC2 <= 26'h0;
        else if(GoAhead)    PC2 <= PC1;
    end

    //======== RfRdAdrB1 ====================
    //always @(posedge Clk or negedge RstN) begin  // 20100327
    //    if(~RstN)           RfRdAdrB1 <= 4'h0;
    //    else if(GoAhead)    RfRdAdrB1 <= RfRdAdrB;
    //end

    //======== RfWrAdr1 ====================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)           RfWrAdr1 <= 5'h0;
        else if(GoAhead)    RfWrAdr1 <= RfWrAdr0;
    end

    //======== RfWrAdrEx ====================
    //assign  RfWrAdrEx = (PtrUpDate)? {1'b0, RfRdAdrB1} : RfWrAdr1;
    assign  RfWrAdrEx = RfWrAdr1; // 20100327

    //======== RfWe1 ====================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)           RfWe1 <= 4'h0;
        else if(GoAhead)    RfWe1 <= RfWe0;
    end
    //======== RfWeEx ====================
    //assign  RfWeEx = RfWe1 | PtrUpDate; // 20100327
    assign  RfWeEx = RfWe1; // 20100327

    //======== RfWrDEx ====================
    //always @(PtrUpDate or AluAct or LsuAct or PC2 or ResALU or LsuRD or DbgRegRq or DbgWD) begin
    //    if(PtrUpDate)     RfWrDEx = ResALU;
    //    else if(LsuAct)   RfWrDEx = LsuRD;
    //    else if(AluAct)   RfWrDEx = ResALU;
    //    else if(DbgRegRq) RfWrDEx = DbgWD;
    //    else              RfWrDEx = {6'h0, PC2};
    //end
    always @(AluAct or LsuAct or PC2 or ResALU or LsuRD or DbgRegRq or DbgWD) begin // 20100327
        if     (LsuAct)   RfWrDEx = LsuRD;
        else if(AluAct)   RfWrDEx = ResALU;
        else if(DbgRegRq) RfWrDEx = DbgWD;
        else              RfWrDEx = {6'h0, PC2};
    end

    //==================================================
    //======== ALU       ===============================
    //==================================================
    P3Alu P3Alu(
        .Clk             ( Clk          ),
        .RstN            ( RstN         ),
        .GoAhead         ( GoAhead      ),
        .AluAct0         ( AluAct0      ),
        .RfWe0           ( RfWe0        ),  // 20100327 add
        .RfRdDataB       ( RfRdDataB    ),
        .RfRdDataC       ( RfRdDataC    ),
        .RfRdDC          ( RfRdDC       ),
        .Imm16           ( Imm16        ),
        .ImmSel          ( ImmSel       ),
        .Func0           ( Func0        ),

        .ComInB          ( ComInB       ),
        .ComInC          ( ComInC       ),
        .ComFunc         ( ComFunc      ),

        .ComBSftRQ       ( ComBSftRQ    ),
        .ComBSftAK       ( ComBSftAK    ),
        .ComBSftCmpl     ( ComBSftCmpl  ),
        .ComBSftRes      ( ComBSftRes   ),

        .ComDivRQ        ( ComDivRQ     ),
        .ComDivAK        ( ComDivAK     ),
        .ComDivCmpl      ( ComDivCmpl   ),
        .ComDivRes       ( ComDivRes    ),

        .DataADR0        ( DataADR0     ),
        .ResALU          ( ResALU       ),
        .Over            ( Over         ),
        .Carry           ( Carry        ),
        .Sign            ( Sign         ),
        .Zero            ( Zero         ),
        .AluAct          ( AluAct       ),
        .UpDateCV        ( UpDateCV     ),
        .UpDateSZ        ( UpDateSZ     ),
        .AluWait         ( AluWait      ),
        .FwdData         ( FwdData      ), // 20100327 add
        .FwdDataValid    ( FwdDataValid )  // 20100327 add
    );

    //==================================================
    //======== LSU       ===============================
    //==================================================
    P3Lsu P3Lsu(
        .Clk             ( Clk          ),
        .RstN            ( RstN         ),
        .GoAhead         ( GoAhead      ),
        .DataRQ0         ( DataRQ0      ),
        .DataRW0         ( DataRW0      ),
        .DataSIZE0       ( DataSIZE0    ),
        //.DrainRQ0        ( DrainRQ0     ), // 20100403 remove
        //.UpDateAfter     ( UpDateAfter  ), // 20100327
        //.PtrUpDate0      ( PtrUpDate0   ), // 20100327
        //.PtrUpDate       ( PtrUpDate    ), // 20100327
        .RfRdDataA       ( RfRdDataA    ),
        //.RfRdDataB       ( RfRdDataB    ), // 20100327
        .LsuRD           ( LsuRD        ),
        .LsuWait         ( LsuWait      ),
        .LsuAct          ( LsuAct       ),
        .DataADR0        ( DataADR0     ),
        .DataRQ          ( DataRQ       ),
        .DataAK          ( DataAK       ),
        .DataRW          ( DataRW       ),
        .DataBE          ( DataBE       ),
        .DataADR         ( DataADR      ),
        .DataWD          ( DataWD       ),
        .DataRD          ( DataRD       ),
        //.DCacheDrainRQ    ( DCacheDrainRQ ),  // 20100403
        .DCacheRe        ( DCacheRe     ),
        .DCacheArea      ( DCacheArea   ),
        .DCacheIndex     ( DCacheIndex  )
    );

endmodule
