//=======================================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : Psm3Sys (PSM3 TEST SYSTEM)
//  FUNCTION      : Instruction Cache For PSM3
//
//                      Copyright (C) 2008-2010 by Tomoyuki Inomoto
//=======================================================================
// 20100403:add Branch Prediction
// 20100529:Cash->Cache (spell miss)
//=======================================================================

`define     IC_IDLE         4'h0
`define     IC_SNGL_RQ      4'h1
`define     IC_SNGL_WAIT    4'h2
`define     IC_BRST_RQ      4'h3
`define     IC_BRST_WAIT0   4'h4
`define     IC_BRST_WAIT1   4'h5
`define     IC_BRST_WAIT2   4'h6
`define     IC_BRST_WAIT3   4'h7
`define     IC_MSTR_AK      4'h8

module P3ICache4K32b4Brst(
    //======== SYSTEM ==============
    Clk         ,
    RstN        ,
    ICE         ,

    //======= Master Bus ================
    MstrRQ      ,
    MstrAK      ,
    MstrADR     ,
    MstrRD      ,
    MstrBrInst  , // 20100403
    MstrBrCountOut,// 20100403
    MstrBrCountIn, // 20100403
    MstrBrCountWe, // 20100403

    MstrICRe    ,
    MstrICIndex ,

    //======= External Bus ================
    ExCRQ      ,
    ExCAK      ,
    ExBRST     ,
    ExADR      ,
    ExRD       ,
    ExRDAK     ,

    //======== MONITOR ================
    CoreInstHit
);
    //==================================================
    //======== port  ===================================
    //==================================================
    //======== SYSTEM ==============
    input           Clk         ;
    input           RstN        ;
    input           ICE         ;

    //======= Master Bus ================
    input           MstrRQ      ;
    output          MstrAK      ;
    input   [25:0]  MstrADR     ;
    output  [31:0]  MstrRD      ;
    output          MstrBrInst  ; // 20100403 indicate Inst is branch
    output  [1:0]   MstrBrCountOut;// 20100403
    input   [1:0]   MstrBrCountIn; // 20100403
    input           MstrBrCountWe; // 20100403

    input           MstrICRe    ; // Instruction Cache Read Enable
    input   [11:2]  MstrICIndex ; // Instruction Cache Index

    //======= External Bus ================
    output          ExCRQ       ; // Command RQ
    input           ExCAK       ; // Command AK
    output          ExBRST      ; // BURST RQ
    output  [25:0]  ExADR       ; // ADDRESS
    input   [31:0]  ExRD        ; // READ DATA
    input           ExRDAK      ; // READ DATA AK

    //======== MONITOR ================
    output          CoreInstHit;

    //==================================================
    //======== wire & reg ==============================
    //==================================================
    wire            Hit         ;
    wire            CacheHit    ;
    reg     [3:0]   MainState   ;
    reg             ExCRQ       ;
    reg             ExBRST      ;
    reg     [31:0]  ExRDBuf     ;
    wire    [31:0]  CacheRD     ;
    reg     [8:0]   ClrAdr      ;
    reg             ICE1d       ;
    wire            IceNegEdge  ;
    reg             Clear       ;
    wire            CacheEna    ;
    wire            ValidIn     ;
    wire            ValidOut    ;
    wire    [13:0]  TagAdr      ;
    wire            TagWe0      ;
    wire            TagWe       ;
    wire            CacheWe     ;
    wire    [7:0]   TagWrIndex  ;
    reg     [1:0]   BAdr        ;
    wire            BrInst0     ; // 20100403
    wire            BrInst1     ; // 20100403
    wire    [1:0]   BrCount0    ; // 20100403
    wire    [1:0]   BrCount1    ; // 20100403
    reg     [11:2]  MstrADR1d   ; // 20100403
    reg     [31:0]  CacheRD1d   ; // 20100403
    reg             BrInst1_1d  ; // 20100403
    wire    [11:2]  CacheRamWrAdr; // 20100403
    wire    [34:0]  CacheRamWrData; // 20100403


    //====================================================================
    //======== BODY ======================================================
    //====================================================================
    //==================================================
    //======== MONITOR  ================================
    //==================================================
    assign  CoreInstHit = CacheHit;

    //==================================================
    //====== Clear Controler ===========================
    //==================================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) ICE1d <= 2'h0;
        else      ICE1d <= ICE;
    end

    assign IceNegEdge = ({ICE1d, ICE} == 2'b10);

    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)           ClrAdr <= 9'h0;
        else if(IceNegEdge) ClrAdr <= 9'h0;
        else if(~ClrAdr[8]) ClrAdr <= ClrAdr + 1'b1;
    end
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)       Clear <= 1'h1;
        else if(~ICE)   Clear <= 1'h1;
        else            Clear <= ~ClrAdr[8];
    end

    //==================================================
    //======== Main State Machine  =====================
    //==================================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            MainState <= `IC_IDLE;
            ExCRQ     <= 1'b0;
            ExBRST    <= 1'b0;
            ExRDBuf   <= 32'h0;
            BAdr      <= 2'h0;
        end
        else if(MainState == `IC_IDLE) begin       // IDLE
            if     (~CacheEna & MstrRQ) begin
                MainState <= `IC_SNGL_RQ;
                ExCRQ     <= 1'b1;
                ExBRST    <= 1'b0;
            end
            else if( CacheEna & ~Hit & MstrRQ) begin
                MainState <= `IC_BRST_RQ;
                ExCRQ     <= 1'b1;
                ExBRST    <= 1'b1;
            end
        end
        else if(MainState == `IC_SNGL_RQ) begin     // Single
            if(ExCAK & ExRDAK) begin
                MainState <= `IC_MSTR_AK;
                ExCRQ     <= 1'b0;
                ExBRST    <= 1'b0;
                ExRDBuf   <= ExRD;
            end
            else if(ExCAK) begin
                MainState <= `IC_SNGL_WAIT;
                ExCRQ     <= 1'b0;
            end
        end
        else if(MainState == `IC_SNGL_WAIT) begin
            if(ExRDAK) begin
                MainState <= `IC_MSTR_AK;
                ExBRST    <= 1'b0;
                ExRDBuf   <= ExRD;
            end
        end
        else if(MainState == `IC_BRST_RQ) begin     // Burst
            if(ExCAK & ExRDAK)  begin
                MainState <= `IC_BRST_WAIT1;
                ExCRQ     <= 1'b0;
                if(MstrADR[3:2] == 2'h0) ExRDBuf <= ExRD;
                BAdr      <= 2'h1;
            end
            else if(ExCAK) begin
                MainState <= `IC_BRST_WAIT0;
                ExCRQ     <= 1'b0;
            end
        end
        else if(MainState == `IC_BRST_WAIT0) begin
            if(ExRDAK) begin
                MainState <= `IC_BRST_WAIT1;
                if(MstrADR[3:2] == 2'h0) ExRDBuf <= ExRD;
                BAdr      <= 2'h1;
            end
        end
        else if(MainState == `IC_BRST_WAIT1) begin
            if(ExRDAK) begin
                MainState <= `IC_BRST_WAIT2;
                if(MstrADR[3:2] == 2'h1) ExRDBuf <= ExRD;
                BAdr      <= 2'h2;
            end
        end
        else if(MainState == `IC_BRST_WAIT2) begin
            if(ExRDAK)  begin
                MainState <= `IC_BRST_WAIT3;
                if(MstrADR[3:2] == 2'h2) ExRDBuf <= ExRD;
                BAdr      <= 2'h3;
            end
        end
        else if(MainState == `IC_BRST_WAIT3) begin
            if(ExRDAK)  begin
                MainState <= `IC_MSTR_AK;
                ExBRST    <= 1'b0;
                if(MstrADR[3:2] == 2'h3) ExRDBuf <= ExRD;
                BAdr      <= 2'h0;
            end
        end
        else begin
            MainState <= `IC_IDLE;
        end
    end

    //====== CacheHit ===========================
    assign  CacheEna = ~Clear & ICE;
    assign  Hit       = ValidOut & (MstrADR[25:12] == TagAdr);
    assign  CacheHit   = MstrRQ & CacheEna & Hit;

    //====== Tag & Cache Memory Control ===========================
    assign  TagWe0   = ExRDAK & (MainState == `IC_BRST_WAIT3);
    assign  TagWe    = TagWe0 | ~ClrAdr[8];
    assign  CacheWe   = (ExRDAK & ((MainState == `IC_BRST_RQ   ) |
                                  (MainState == `IC_BRST_WAIT0) |
                                  (MainState == `IC_BRST_WAIT1) |
                                  (MainState == `IC_BRST_WAIT2) |
                                  (MainState == `IC_BRST_WAIT3))
                       | (MstrBrCountWe & (MainState == `IC_IDLE)));
    assign  ValidIn  = (Clear)? 1'b0 : 1'b1;
    assign  TagWrIndex = (Clear)? ClrAdr[7:0] : MstrADR[11:4];

    //====== TAG RAM ===========================
    P3ItagRam ITag(
        .addra  ( MstrICIndex[11:4]         ),
        .addrb  ( TagWrIndex                ),
        .clka   ( Clk                       ),
        .clkb   ( Clk                       ),
        .dinb   ( {ValidIn, MstrADR[25:12]} ),
        .douta  ( {ValidOut, TagAdr}        ),
        .ena    ( MstrICRe                  ),
        .web    ( TagWe                     )
    );

    assign  BrInst0 = (ExRD[31:28] == 4'b0001);
    always @(posedge Clk) if(MstrAK) MstrADR1d <= MstrADR[11:2];  // 20100403
    always @(posedge Clk) if(MstrAK) CacheRD1d  <= CacheRD;         // 20100403
    always @(posedge Clk) if(MstrAK) BrInst1_1d<= BrInst1;         // 20100403
    assign CacheRamWrAdr  = (MainState == `IC_IDLE)? MstrADR1d : {MstrADR[11:4], BAdr}; // 20100403
    assign CacheRamWrData = (MainState == `IC_IDLE)? {MstrBrCountIn, BrInst1_1d, CacheRD1d} : {{2{(BrInst0 & ExRD[23])}}, BrInst0, ExRD}; // 20100403

    //====== CASH RAM ===========================
    P3ICacheRam ICache(
        .addra  ( MstrICIndex[11:2]         ),
        //.addrb  ( {MstrADR[11:4], BAdr}     ),// 20100403
        .addrb  ( CacheRamWrAdr              ),
        .clka   ( Clk                       ),
        .clkb   ( Clk                       ),
        //.dinb   ( ExRD                      ),
        .dinb   ( CacheRamWrData             ), // 20100403
        .ena    ( MstrICRe                  ),
        //.douta  ( CacheRD                    ),
        .douta  ( {BrCount1, BrInst1,  CacheRD} ), // 20100403
        //.web    ( CacheWe                    )
        .web    ( CacheWe                    )
    );

    //==================================================
    //======== External Bus  ===========================
    //==================================================
    assign  ExADR[25:4]  = MstrADR[25:4];
    assign  ExADR[3:2]   = (MainState == `IC_SNGL_RQ)? MstrADR[3:2] : 2'b00;
    assign  ExADR[1:0]   = 2'b00;

    //==================================================
    //======== Master Bus     ==========================
    //==================================================
    assign  MstrAK  = (MainState[3] | CacheHit);
    assign  MstrRD  = (MainState[3])? ExRDBuf : CacheRD;
    assign  MstrBrInst = (MainState[3])? 1'b0  : BrInst1;  // 20100403
    assign  MstrBrCountOut = (MainState[3])? 2'b0  : BrCount1;  // 20100403
endmodule

/*=======================================================================*
 *  TARGET PROJECT: PSM3(Prosessor Study Model3)                         *
 *  TARGET SYSTEM : Psm3Sys (PSM3 TEST SYSTEM)                           *
 *  FUNCTION      : TagRam for Instruction Cache of PSM3                 *
 *=======================================================================*/
module P3ItagRam(
    addra   ,
    addrb   ,
    clka    ,
    clkb    ,
    dinb    ,
    douta   ,
    ena     ,
    web
);

    input   [7:0]   addra;
    input   [7:0]   addrb;
    input           clka;
    input           clkb;
    input   [14:0]  dinb;
    output  [14:0]  douta;
    input           ena;
    input           web;

    reg     [14:0]  mem[0:255];
    reg     [14:0]  douta;

    always @(posedge clka) if(ena) douta <= mem[addra];
    always @(posedge clkb) if(web) mem[addrb] <= dinb;

endmodule

/*=======================================================================*
 *  TARGET PROJECT: PSM3(Prosessor Study Model3)                         *
 *  TARGET SYSTEM : Psm3Sys (PSM3 TEST SYSTEM)                           *
 *  FUNCTION      : CacheRam for Instruction Cache of PSM3                 *
 *=======================================================================*/
module P3ICacheRam(
    addra   ,
    addrb   ,
    clka    ,
    clkb    ,
    dinb    ,
    douta   ,
    ena     ,
    web
);

    input   [9:0]   addra;
    input   [9:0]   addrb;
    input           clka;
    input           clkb;
    //input   [31:0]  dinb;
    input   [34:0]  dinb; // 20100403
    //output  [31:0]  douta;
    output  [34:0]  douta; // 20100403
    input           ena;
    input           web;

    //reg     [31:0]  mem[0:1023];
    reg     [34:0]  mem[0:1023]; // 20100403
    //reg     [31:0]  douta;
    reg     [34:0]  douta; // 20100403

    always @(posedge clka) if(ena) douta <= mem[addra];
    always @(posedge clkb) if(web) mem[addrb] <= dinb;

endmodule
