//=======================================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : *(PSM3 TARGET SYSTEM)
//  FUNCTION      : Psm3 Instruction Fetch Unit
//
//                      Copyright (C) 2008-2010 by Tomoyuki Inomoto
//=======================================================================
// 20100403:add Branch Prediction
// 20100529:Cash->Cache (spell miss)
//=======================================================================

 module P3Ifu(
     //======== SYSTEM =============
    Clk          ,
    RstN         ,
    Vector       ,

    //======== to/form DECU ========
    PC0          ,
    DecInstRQ    ,
    DecInstAK    ,
    Inst         ,
    Break0       ,
    JmpAdr       ,
    NextPcSel    ,
    PcWrited     ,
    BrPredict0   ,

    //======== INST BUS ============
    InstRQ       ,
    InstAK       ,
    InstADR      ,
    InstRD       ,
    InstBrInst   , // 20100403
    InstBrCountRd, // 20100403
    InstBrCountWr, // 20100403
    InstBrCountWe, // 20100403
    
    //======== INST Cache ===========
    ICacheRe     ,
    ICacheIndex  ,

    //======== DEBUG================
    DbgRegRq     ,
    DbgRegWe     ,
    DbgRegAdr    ,
    DbgWD
);
    //==============================
    //======== port  ===============
    //==============================
    //======== SYSTEM ==============
    input           Clk          ; // Clock
    input           RstN         ; // Reset (L-Active)
    input   [25:0]  Vector       ; // Vector of IRQ


    //======== to/form DECU ========
    output  [25:0]  PC0          ; // Program Counter
    input           DecInstRQ    ; // Instruction Reqest from DECU
    output          DecInstAK    ; // Ack of Instruction Reqest to DECU
    output  [31:0]  Inst         ; // Instruction Data
    output          Break0       ; // Break0
    input   [25:0]  JmpAdr       ; // Jump Address
    input   [1:0]   NextPcSel    ; // 00:IncPC, 01:JmpAdr, 1*:Vector
    output          PcWrited     ; // PC is OverWrited by Debuger
    output          BrPredict0   ; // Branch Prediction act // 20100403

    //======== INST BUS ================
    output          InstRQ       ; // INSTRUCTION BUS REQ
    input           InstAK       ; // INSTRUCTION BUS ACK
    output  [25:0]  InstADR      ; // INSTRUCTION BUS ADDRESS
    input   [31:0]  InstRD       ; // INSTRUCTION BUS READ DATA
    input           InstBrInst   ; // 20100403 indicate Inst is branch
    input   [1:0]   InstBrCountRd;// 20100403
    output  [1:0]   InstBrCountWr; // 20100403
    output          InstBrCountWe; // 20100403

    //======== INST Cache ================
    output          ICacheRe     ; // Instruction Cache Read Enable
    output  [11:2]  ICacheIndex  ; // Instruction Cache Index (4KByte)

    //======== DEBUG================
    input           DbgRegRq     ; // Debug Reg Access Request from Debugger
    input           DbgRegWe     ; // Register Write Enable from Debugger
    input   [4:0]   DbgRegAdr    ; // Register Address from Debugger
    input   [31:0]  DbgWD        ; // Register Write Data from Debugger

    //==============================
    //======== wire & reg ==========
    //==============================
    reg     [25:0]  PC0          ; // Program Counter
    reg     [25:0]  NextPC       ; // Next Program Counter
    reg     [31:0]  IB           ; // Instruction Buffer
    reg             BrPredictBuf ; // Branch Prediction Indicate buf
    reg             NextIBValid  ;
    reg             IBValid      ; // Instruction Buffer Valid
    reg             InstRQ       ;
    wire            PCUpDate     ;
    reg             DbgPcWe      ;
    reg             RstN1d       ;
    reg             DbgPcWe1d    ;
    reg             ICacheRe     ;
    reg     [25:0]  NextPCOffset ; // Next Program Counter Offset // 20100403 add
    wire    [31:0]  Inst0        ; // Instruction Data
    wire            InstBrPredict; // Branch Prediction Indicate from ICache 20100403
    reg     [1:0]   InstBrCountRd1d;// 20100403
    reg     [1:0]   InstBrCountWr; // 20100403
    reg             InstBrCountWe; // 20100403

    //==============================
    //======== BODY       ==========
    //==============================
    //======= PcWrited =================================
    assign PcWrited = DbgPcWe1d;

    //======== PC0 =====================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)   DbgPcWe <= 1'h0;
        else        DbgPcWe <= DbgRegRq & DbgRegWe & (DbgRegAdr == 5'h0F);
    end
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)           PC0 <= 26'h0;
        else if(DbgPcWe)    PC0 <= {DbgWD[25:2], 2'h0};
        else if(PCUpDate)   PC0 <= NextPC;
    end

    //======== NextPC ====================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)   RstN1d <= 1'h0;
        else        RstN1d <= 1'h1;
    end
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)   DbgPcWe1d <= 1'h0;
        else        DbgPcWe1d <= DbgPcWe;
    end

    always @(BrPredict0 or Inst0) begin // 20100403 add
        if(BrPredict0) begin
            NextPCOffset = {Inst0[23:0], 2'h0};
        end
        else begin
            NextPCOffset = 26'h4;
        end
    end
    //always @(DbgPcWe1d or RstN or RstN1d or NextPcSel or PC0 or JmpAdr or Vector) begin // 20100403 remove
    always @(DbgPcWe1d or RstN or RstN1d or NextPcSel or PC0 or JmpAdr or Vector or NextPCOffset) begin // 20100403 add
        if(DbgPcWe1d | ({RstN, RstN1d} == 2'b10)) NextPC = PC0;
        else if(NextPcSel == 2'b01)               NextPC = JmpAdr;
        //else if(NextPcSel == 2'b00)               NextPC = PC0 + 3'h4; // 20100403 remove
        else if(NextPcSel == 2'b00)               NextPC = PC0 + NextPCOffset;  // 20100403 add
        else                                      NextPC = Vector;
    end

    //======== IB ====================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)       IB <= 32'h0;
        else if(InstAK) IB <= InstRD;
    end
    
    //======== BrPredictBuf ==================================== //20100403 add
    assign InstBrPredict = InstBrCountRd[1];
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)       BrPredictBuf <= 1'h0;
        else if(InstAK) BrPredictBuf <= InstBrPredict;
    end
    //======== InstBrCountRd1d ==================================== //20100403 add
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)       InstBrCountRd1d <= 2'h0;
        else if(InstAK) InstBrCountRd1d <= InstBrCountRd;
    end
    //======== InstBrCountWe ==================================== //20100403 add
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)       InstBrCountWe <= 1'h0;
        else if(InstAK) InstBrCountWe <= InstBrInst;
    end
    //======== InstBrCountWr ==================================== //20100403 add
    always  @(BrPredictBuf or NextPcSel or InstBrCountRd1d) begin
        if(BrPredictBuf) begin // Predict taken
            if(NextPcSel == 2'b01) begin // taken
                if(InstBrCountRd1d == 2'h0) InstBrCountWr = InstBrCountRd1d;
                else                        InstBrCountWr = InstBrCountRd1d - 1'b1;
            end
            else if(NextPcSel == 2'b00) begin // not taken
                if(InstBrCountRd1d == 2'h3) InstBrCountWr = InstBrCountRd1d;
                else                        InstBrCountWr = InstBrCountRd1d + 1'b1;
            end
            else begin
                InstBrCountWr = InstBrCountRd1d;
            end
        end
        else begin // Predict not taken
            if(NextPcSel == 2'b01) begin // taken
                if(InstBrCountRd1d == 2'h3) InstBrCountWr = InstBrCountRd1d;
                else                        InstBrCountWr = InstBrCountRd1d + 1'b1;
            end
            else if(NextPcSel == 2'b00) begin // not taken
                if(InstBrCountRd1d == 2'h0) InstBrCountWr = InstBrCountRd1d;
                else                        InstBrCountWr = InstBrCountRd1d - 1'b1;
            end
            else begin
                InstBrCountWr = InstBrCountRd1d;
            end
        end
    end

    //======== IBValid ====================================
    always  @(DecInstRQ or IBValid or InstAK or DbgPcWe) begin
        if(DbgPcWe) NextIBValid = 1'b0;
        else begin
            case({DecInstRQ, IBValid, InstAK})
                3'b001 : NextIBValid = 1'b1;
                3'b010 : NextIBValid = 1'b1;
                3'b011 : NextIBValid = 1'b1;
                3'b111 : NextIBValid = 1'b1;
                default: NextIBValid = 1'b0;
            endcase
        end
    end
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)   IBValid <= 1'h0;
        else        IBValid <= NextIBValid;
    end

    //======== DecInstAK ====================================
    assign DecInstAK = DecInstRQ & (IBValid | InstAK);

    //======== Inst ====================================
    //assign Inst = (IBValid)? IB : InstRD; // 20100403 remove
    assign Inst0 = (IBValid)? IB : InstRD;  // 20100403 add
    assign BrPredict0 = (IBValid)? BrPredictBuf : InstBrPredict; // 20100403 add
    assign Inst  = (BrPredict0)? {Inst0[31:24], 24'h1} : Inst0;  // 20100403 add

    //======== Break0 ==================================
    assign Break0 = (Inst[31:26] == 6'b111110);

    //======== ICacheRe(PCUpDate) =========================
    always  @(DecInstRQ or IBValid or InstRQ or InstAK) begin
        case({DecInstRQ, IBValid, InstRQ, InstAK})
            4'b0000 : ICacheRe = 1'b1;
            4'b1000 : ICacheRe = 1'b1;
            4'b1011 : ICacheRe = 1'b1;
            4'b1100 : ICacheRe = 1'b1;
            default : ICacheRe = 1'b0;
        endcase
    end
    assign PCUpDate = ICacheRe;

    //======== ICacheIndex =========================
    assign ICacheIndex = NextPC[11:2];       // 4KByte

    //======== IBUS =====================================
    //======== InstRQ ====================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)           InstRQ <= 1'b0;
        else if(ICacheRe)   InstRQ <= 1'b1;
        else if(InstAK)     InstRQ <= 1'b0;
    end
    //======== InstADR ====================================
    assign  InstADR = PC0;

endmodule
