//=======================================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : *(PSM3 TARGET SYSTEM)
//  FUNCTION      : Psm3 Load & Stre Unit
//
//                      Copyright (C) 2008-2010 by Tomoyuki Inomoto
//=======================================================================
// 20100327:UpDateAfter, PtrUpDate0, PtrUpDateWait and relational signals are removed
// 20100529:Cash->Cache (spell miss)
//=======================================================================

module P3Lsu(
    //======== SYSTEM ====================
    Clk             ,
    RstN            ,

    //======== from/to DECU ==============
    GoAhead         ,
    DataRQ0         ,
    DataRW0         ,
    DataSIZE0       ,
    //DrainRQ0        , // 20100403 remove
    //UpDateAfter     , // 20100327
    //PtrUpDate0      , // 20100327
    //PtrUpDate       , // 20100327
    RfRdDataA       ,
    //RfRdDataB       ,

    LsuRD           ,
    LsuWait         ,
    LsuAct          ,

    //======== from ALU ==============
    DataADR0        ,

    //======== DATA BUS ==================
    DataRQ          ,
    DataAK          ,
    DataRW          ,
    DataBE          ,
    DataADR         ,
    DataWD          ,
    DataRD          ,

    //======== DATA Cache =================
    //DCacheDrainRQ    , // 20100403
    DCacheRe        ,
    DCacheArea      ,
    DCacheIndex
);
    //==========================================================
    //======== port  ===========================================
    //==========================================================
    //======== SYSTEM ====================
    input           Clk             ;
    input           RstN            ;

    //======== from/to DECU ==============
    input           GoAhead         ;
    input           DataRQ0         ; // DATA BUS REQEST (1:REQ)
    input           DataRW0         ; // DATA BUS READ/WRITE (1:WRITE)
    input   [2:0]   DataSIZE0       ; // DATA BUS DATA SIZE (000:WORD, 010:HALF, 011:Signed HALF, 100:BYTE, 101:Signed Half)
    //input           DrainRQ0        ; // Data Cache Drain RQ (for Write Back Cache)  // 20100403 remove
    //input           UpDateAfter     ; // UpDateAfter // 20100327
    //input           PtrUpDate0      ; // Pointer UpDate Flag // 20100327
    //output          PtrUpDate       ; // Pointer UpDate Flag // 20100327
    input   [31:0]  RfRdDataA       ; // PortA Read Data
    //input   [31:0]  RfRdDataB       ; // PortB Read Data // 20100327

    output  [31:0]  LsuRD           ; // LSU DataBus/DadaCache Read Data
    output          LsuWait         ; // LSU Wait
    output          LsuAct          ; // LSU Active

    //======== from ALU ==============
    input   [27:0]  DataADR0        ; // Address Of data bus

    //======== DATA BUS ==================
    output          DataRQ          ; // DATA BUS REQ
    input           DataAK          ; // DATA BUS ACK
    output          DataRW          ; // DATA BUS READ/WRITE (1:WRITE)
    output  [3:0]   DataBE          ; // DATA BUS BYTE ENABLE
    output  [27:0]  DataADR         ; // DATA BUS ADDRESS
    output  [31:0]  DataWD          ; // DATA BUS WRITE DATA
    input   [31:0]  DataRD          ; // DATA BUS READ DATA


    //======== DATA Cache =================
    //output          DCacheDrainRQ    ; // Data Cache Drain RQ (for Write Back Cache) // 20100403
    output          DCacheRe        ; // Data Cache Read Enable
    output  [27:23] DCacheArea      ; // Data Cache Accsess Area
    output  [11:2]  DCacheIndex     ; // Data Cache Index (4KByte)

    //==========================================================
    //======== wire & reg ======================================
    //==========================================================
    reg             LsuAct          ;
    wire    [27:0]  DataADR1        ;
    reg             DataRQ1         ; // DATA BUS REQ
    reg             DataRW          ; // DATA BUS READ/WRITE (1:WRITE)
    reg     [2:0]   DataSIZE        ; // DATA BUS DATA SIZE (000:WORD, 010:HALF, 011:Signed HALF, 100:BYTE, 101:Signed Half)
    reg     [3:0]   DataBE          ; // DATA BUS BYTE ENABLE
    reg     [27:0]  DataADR         ; // DATA BUS ADDRESS
    reg     [31:0]  DataWD          ; // DATA BUS WRITE DATA
    reg     [31:0]  DataWD0         ; // DATA BUS WRITE DATA FF
    //reg     [31:0]  DataRD1d        ; // DATA BUS READ DATA 1 Delay
    //reg     [31:0]  TrimedRD        ; // Trimed Read Data
    reg     [31:0]  LsuRD           ; // LSU DataBus/DadaCache Read Data
    //reg             PtrUpDate       ; // Pointer UpDate Flag
    //reg             PtrUpDateWait   ; // Wait if read & PtrUpDate
    //reg             TrimedRDSel     ; // TrimedRD Select
    //reg             RdTrimWait      ; // Read Data Trim wait
    //reg             DataAK1d        ; // DataAK 1cycle Delay
    //reg             DCacheDrainRQ   ; // Data Cache Drain RQ (for Write Back Cache) // 20100403

    //==========================================================
    //======== BODY       ======================================
    //==========================================================

    //===========================================
    //======== to DATA Cache  ====================
    //===========================================
    //======== DataADR1  ======================
    //assign DataADR1 = (UpDateAfter)? RfRdDataB[27:0] : DataADR0; // 20100327 remove
    assign DataADR1 = DataADR0; // 20100327 add

    //======== DCacheArea    ======================
    assign DCacheArea = DataADR1[27:23];

    //======== DCacheIndex    ======================
    assign DCacheIndex = DataADR1[11:2];     // 4KByte

    //======== DCacheRe    =========================
    assign DCacheRe = GoAhead & DataRQ0;

    //======== DCacheDrainRQ =================
    //always @(posedge Clk or negedge RstN) begin // 20100403
    //    if(~RstN)           DCacheDrainRQ <= 1'h0;
    //    else if(GoAhead)    DCacheDrainRQ <= DrainRQ0 & DataRQ0;
    //end
    //===========================================
    //======== to DATA BUS   ====================
    //===========================================
    //======== DataRQ ====================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)           DataRQ1 <= 1'b0;
        else if(GoAhead)    DataRQ1 <= DataRQ0;
        else if(DataAK)     DataRQ1 <= 1'b0;
    end

    //======== DataRW ====================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)                     DataRW <= 1'b0;
        else if(GoAhead & DataRQ0)    DataRW <= DataRW0;
    end

    //======== DataSIZE0 =================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)                     DataSIZE <= 3'h0;
        else if(GoAhead & DataRQ0)    DataSIZE <= DataSIZE0;
    end

    //======== DataADR =================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)                     DataADR <= 28'h0;
        else if(GoAhead & DataRQ0)    DataADR <= DataADR1;
    end

    //======== DataBE ====================
    always @(DataSIZE or DataADR) begin
        casex({DataSIZE[2:1], DataADR[1:0]})
            4'b010x : DataBE = 4'b1100;
            4'b011x : DataBE = 4'b0011;
            4'b1x00 : DataBE = 4'b1000;
            4'b1x01 : DataBE = 4'b0100;
            4'b1x10 : DataBE = 4'b0010;
            4'b1x11 : DataBE = 4'b0001;
            default : DataBE = 4'b1111;
        endcase
    end

    //======== DataWD ========================================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)                     DataWD0 <= 32'h0;
        else if(GoAhead & DataRQ0)    DataWD0 <= RfRdDataA;
    end

    always @(DataSIZE or DataADR or DataWD0) begin
        casex({DataSIZE[2:1], DataADR[1:0]})
            4'b010x : DataWD = {DataWD0[15:0], 16'h0};
            4'b011x : DataWD = {16'h0, DataWD0[15: 0]};
            4'b1x00 : DataWD = {DataWD0[7:0], 24'h0};
            4'b1x01 : DataWD = {8'h0,  DataWD0[7:0], 16'h0};
            4'b1x10 : DataWD = {16'h0, DataWD0[7:0], 8'h0};
            4'b1x11 : DataWD = {24'h0, DataWD0[7:0]};
            default : DataWD = DataWD0;
        endcase
    end

    //===========================================
    //======== to DECU  =========================
    //===========================================
    //======== LsuAct ====================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)           LsuAct <= 1'b0;
        else if(GoAhead)    LsuAct <= DataRQ0;
    end

    //======== PtrUpDate ==================== 
    //always @(posedge Clk or negedge RstN) begin // 20100327 remove
    //    if(~RstN)           PtrUpDate <= 1'b0;
    //    else if(GoAhead)    PtrUpDate <= (PtrUpDate0 & DataRQ0);
    //    else                PtrUpDate <= 1'b0;
    //end

    //======== PtrUpDateWait ====================
    //always @(posedge Clk or negedge RstN) begin // 20100327 remove
    //    if(~RstN)           PtrUpDateWait <= 1'b0;
    //    else if(GoAhead)    PtrUpDateWait <= (PtrUpDate0 & DataRQ0 & ~DataRW0);
    //    else                PtrUpDateWait <= 1'b0;
    //end

    //======== RdTrimWait ====================
    //always @(posedge Clk or negedge RstN) begin // 20100327 remove
    //    if(~RstN)           RdTrimWait <= 1'b0;
    //    else if(GoAhead)    RdTrimWait <= (DataSIZE0[2] | DataSIZE0[1]) & DataRQ0 & ~DataRW0;
    //    else if(DataAK1d)   RdTrimWait <= 1'b0;
    //end

    //======== TrimedRDSel ====================
    //always @(posedge Clk or negedge RstN) begin // 20100327 remove
    //    if(~RstN)        TrimedRDSel <= 1'b0;
    //    else if(GoAhead) TrimedRDSel <= 1'b0;
    //    else             TrimedRDSel <= DataSIZE[2] | DataSIZE[1] | (PtrUpDateWait & DataAK);
    //end

    //======== DataAK1d ====================
    //always @(posedge Clk or negedge RstN) begin // 20100327 remove
    //    if(~RstN)           DataAK1d <= 1'b0;
    //    else if(GoAhead)    DataAK1d <= 1'b0;
    //    else                DataAK1d <= DataAK;
    //end

    //======== DataRD1d ====================
    //always @(posedge Clk or negedge RstN) begin // 20100327 remove
    //    if(~RstN)           DataRD1d <= 32'h0;
    //    else if(DataAK)     DataRD1d <= DataRD;
    //end

    //======= TrimedRD =====================
    //always  @(DataRD1d or DataADR or DataSIZE) begin // 20100327 remove
    //    casex({DataSIZE, DataADR[1:0]})
    //        5'b01_0_0x : TrimedRD = { 16'h0,             DataRD1d[31:16]}; // Load Unsigned Half H
    //        5'b01_1_0x : TrimedRD = {{16{DataRD1d[31]}}, DataRD1d[31:16]}; // Load Signed   Half H
    //        5'b01_0_1x : TrimedRD = { 16'h0,             DataRD1d[15: 0]}; // Load Unsigned Half L
    //        5'b01_1_1x : TrimedRD = {{16{DataRD1d[15]}}, DataRD1d[15: 0]}; // Load Signed   Half L
    //        5'b1x_0_00 : TrimedRD = { 16'h0,             DataRD1d[31:24]}; // Load Unsigned Byte HH
    //        5'b1x_1_00 : TrimedRD = {{24{DataRD1d[31]}}, DataRD1d[31:24]}; // Load Signed   Byte HH
    //        5'b1x_0_01 : TrimedRD = { 24'h0,             DataRD1d[23:16]}; // Load Unsigned Byte HL
    //        5'b1x_1_01 : TrimedRD = {{24{DataRD1d[23]}}, DataRD1d[23:16]}; // Load Signed   Byte HL
    //        5'b1x_0_10 : TrimedRD = { 24'h0,             DataRD1d[15: 8]}; // Load Unsigned Byte LH
    //        5'b1x_1_10 : TrimedRD = {{24{DataRD1d[15]}}, DataRD1d[15: 8]}; // Load Signed   Byte LH
    //        5'b1x_0_11 : TrimedRD = { 24'h0,             DataRD1d[ 7: 0]}; // Load Unsigned Byte LL
    //        5'b1x_1_11 : TrimedRD = {{24{DataRD1d[ 7]}}, DataRD1d[ 7: 0]}; // Load Signed   Byte LL
    //        default    : TrimedRD = DataRD1d                             ; // Load          Word
    //    endcase
    //end

    always  @(DataRD or DataADR or DataSIZE) begin // 20100327 add
        casex({DataSIZE, DataADR[1:0]})
            5'b01_0_0x : LsuRD = { 16'h0,           DataRD[31:16]}; // Load Unsigned Half H
            5'b01_1_0x : LsuRD = {{16{DataRD[31]}}, DataRD[31:16]}; // Load Signed   Half H
            5'b01_0_1x : LsuRD = { 16'h0,           DataRD[15: 0]}; // Load Unsigned Half L
            5'b01_1_1x : LsuRD = {{16{DataRD[15]}}, DataRD[15: 0]}; // Load Signed   Half L
            5'b1x_0_00 : LsuRD = { 16'h0,           DataRD[31:24]}; // Load Unsigned Byte HH
            5'b1x_1_00 : LsuRD = {{24{DataRD[31]}}, DataRD[31:24]}; // Load Signed   Byte HH
            5'b1x_0_01 : LsuRD = { 24'h0,           DataRD[23:16]}; // Load Unsigned Byte HL
            5'b1x_1_01 : LsuRD = {{24{DataRD[23]}}, DataRD[23:16]}; // Load Signed   Byte HL
            5'b1x_0_10 : LsuRD = { 24'h0,           DataRD[15: 8]}; // Load Unsigned Byte LH
            5'b1x_1_10 : LsuRD = {{24{DataRD[15]}}, DataRD[15: 8]}; // Load Signed   Byte LH
            5'b1x_0_11 : LsuRD = { 24'h0,           DataRD[ 7: 0]}; // Load Unsigned Byte LL
            5'b1x_1_11 : LsuRD = {{24{DataRD[ 7]}}, DataRD[ 7: 0]}; // Load Signed   Byte LL
            default    : LsuRD = DataRD                             ; // Load          Word
        endcase
    end

    //======== LsuRD ====================
    //always @(DataRD or TrimedRD or TrimedRDSel) begin // 20100327 remove
    //    if(TrimedRDSel) LsuRD = TrimedRD;
    //    else            LsuRD = DataRD;
    //end

    //======== LsuWait ====================
    //assign LsuWait = PtrUpDateWait | RdTrimWait | (DataRQ1 & ~DataAK);// 20100327 remove
    assign LsuWait = (DataRQ1 & ~DataAK);
    //assign DataRQ = DataRQ1 & ~DCacheDrainRQ; // 20100403
    assign DataRQ = DataRQ1; // 20100403

endmodule
