//=======================================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : *(PSM3 TARGET SYSTEM)
//  FUNCTION      : Psm3 PipeLine Controler
//
//                      Copyright (C) 2008-2010 by Tomoyuki Inomoto
//=======================================================================
// 20100529:Cash->Cache (spell miss)
//=======================================================
 module P3PlCtrl(
    //======== SYSTEM ===================
    Clk             ,
    RstN            ,

    //======== InterLock Signal from DCache ====
    LockByDCache    ,

    //======== for Inst Read Control ====
    DecInstRQ       ,
    DecInstAK       ,

    //======== for Data Hazard Detect ===
    IR              ,
    RfWrAdrEx       ,
    RfWeEx          ,

    RfRdAdrA        ,
    RfRdAdrB        ,
    RfRdAdrC        ,

    //UpDateCV        , // 20100327 remove
    //UpDateSZ        , // 20100327 remove

    AluWait         ,
    LsuWait         ,

    //======== for Debug ================
    DbgRq           ,
    DbgEna          ,

    //======== for Pipline Cntrol =======
    GoAhead         ,

    //======== for Register Write Mask ==
    RegWeMask       ,
    
    FwdDataValid    , // Data for forwarding valid // 20100327
    RegAFwdEna      , // Reg A Forwarding Enable // 20100327 add
    RegBFwdEna      , // Reg B Forwarding Enable // 20100327 add
    RegCFwdEna        // Reg C Forwarding Enable // 20100327 add
);
    //==================================================
    //======== port  ===================================
    //==================================================
    //======== SYSTEM ==============
    input           Clk             ;
    input           RstN            ;

    //======== InterLock Signal from DCache ====
    input           LockByDCache    ; // InterLock Signal from DCache

    //======== for Instruction Read Control ========
    output          DecInstRQ       ;
    input           DecInstAK       ;

    //======== for Data Hazard Detect ========
    input   [31:28] IR              ; // Instruction Register
    input   [3:0]   RfWrAdrEx       ; // Register file Write Address
    input           RfWeEx          ; // Register file Write Enable

    input   [3:0]   RfRdAdrA        ; // Register file Read Address A
    input   [3:0]   RfRdAdrB        ; // Register file Read Address B
    input   [3:0]   RfRdAdrC        ; // Register file Read Address C

    //input           UpDateCV        ; // UpDate Carry & Over // 20100327 remove
    //input           UpDateSZ        ; // UpDate Sign & Zero  // 20100327 remove

    input           AluWait         ; // ALU Wait
    input           LsuWait         ; // LSU Wait

    //======== for Debug ========
    input           DbgRq           ;
    output          DbgEna          ;

    //======== for Pipline Cntrol ========
    output          GoAhead         ;
    //======== for Register Write Mask ========
    output          RegWeMask       ; // 1:Mask, 0:NotMask

    input           FwdDataValid    ; // Data for forwarding valid // 20100327
    output          RegAFwdEna      ; // Reg A Forwarding Enable // 20100327 add
    output          RegBFwdEna      ; // Reg B Forwarding Enable // 20100327 add
    output          RegCFwdEna      ; // Reg C Forwarding Enable // 20100327 add

    //==================================================
    //======== wire & reg ==============================
    //==================================================
    //reg           IRValid         ; // Instruction Register is Valid
    reg             InstCmpl        ; // Instruction Complete
    reg             RegWrCmpl       ; // Register Write Cmplete
    reg             DataHazerd      ; // Detect Data Hazerd
    //wire            FlagChange      ; // 20100327 remove
    wire            RegAHit         ;
    wire            RegBHit         ;
    wire            RegCHit         ;


    //==================================================
    //======== BODY       ==============================
    //==================================================
    //======== RegWrCmpl ========
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)                            RegWrCmpl <= 1'b0;
        else if(GoAhead)                     RegWrCmpl <= 1'b0;
        else if({AluWait, LsuWait} == 2'b00) RegWrCmpl <= 1'b1;
    end

    //======== for Data Hazard Detect ========
    //assign  FlagChange = (UpDateCV | UpDateSZ); // 20100327 remove
    assign  RegAHit    = (RfWrAdrEx == RfRdAdrA);
    assign  RegBHit    = (RfWrAdrEx == RfRdAdrB);
    assign  RegCHit    = (RfWrAdrEx == RfRdAdrC);
    assign  RegAFwdEna = RegAHit & FwdDataValid & ~RegWrCmpl; // 20100327 add
    assign  RegBFwdEna = RegBHit & FwdDataValid & ~RegWrCmpl; // 20100327 add
    assign  RegCFwdEna = RegCHit & FwdDataValid & ~RegWrCmpl; // 20100327 add
    //always  @(RegWrCmpl or IR or RfWeEx or FlagChange or RegAHit or RegBHit or RegCHit or FwdDataValid) begin // 20100327 remove
    always  @(RegWrCmpl or IR or RfWeEx or RegAHit or RegBHit or RegCHit or FwdDataValid) begin // 20100327 add
        if(RegWrCmpl) begin
            DataHazerd = 1'b0;
        end
        else begin
            DataHazerd = 1'b0;
            if(RfWeEx) begin
                //if((IR[31] == 1'b0) & (FlagChange | (~IR[28] & RegAHit))) begin           //== Branch // 20100327 remove
                if((IR[31] == 1'b0) & (~IR[28] & RegAHit)) begin           //== Branch  // 20100327 add
                    DataHazerd = 1'b1;
                end
                if((IR[31:29] == 3'b100) & (RegBHit | (~IR[28] & RegCHit))) begin         //== LD
                    //DataHazerd = 1'b1; // 20100327 remove
                    DataHazerd = ~FwdDataValid; // 20100327 add
                end
                if((IR[31:29] == 3'b101) & (RegAHit | RegBHit | (~IR[28] & RegCHit))) begin //== ST
                    //DataHazerd = 1'b1; // 20100327 remove
                    DataHazerd = ~FwdDataValid; // 20100327 add
                end
                if((IR[31:29] == 3'b110) & (RegBHit | (~IR[28] & RegCHit))) begin         //== ALU
                    //DataHazerd = 1'b1; // 20100327 remove
                    DataHazerd = ~FwdDataValid; // 20100327 add
                end
            end
        end
    end

    //======== InstCmpl ========================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN)           InstCmpl <= 1'b0;
        else if(DecInstAK)  InstCmpl <= 1'b0;
        else if(GoAhead)    InstCmpl <= 1'b1;
    end
    //======== GoAhead ========================
    assign GoAhead = ({InstCmpl, DataHazerd, AluWait, LsuWait} == 4'h0)  & ~LockByDCache;

    //======== DecInstRQ ========================
    assign DecInstRQ = ~DbgRq & (GoAhead | InstCmpl);

    //======== DbgEna ========================
    assign DbgEna = DbgRq & InstCmpl & RegWrCmpl;

    //======== RegWeMask ========================
    assign RegWeMask = RegWrCmpl;

endmodule
