//=======================================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : *(PSM3 TARGET SYSTEM)
//  FUNCTION      : Psm3 Register File
//
//                      Copyright (C) 2008-2010 by Tomoyuki Inomoto
//=======================================================================
// 20100327:RfRdDataB and RfRdDataC don't contain PC1.
//         :Some signals about forwarding are added. (RegAFwdEna, RegBFwdEna, RegCFwdEna, FwdData)
//=======================================================================
module P3Rf(
    //======== SYSTEM ==============
    Clk             ,
    RstN            ,
    PID             ,

    BusLockAK       ,

    //======== from P3PlCtrl ========
    GoAhead         ,
    DecInstAK       ,
    RegWeMask       ,

    //======== from/to P3Id =========
    PC0             ,
    PC1             ,

    RfWrAdrEx       ,
    RfWeEx          ,
    RfWrDEx         ,

    RfRdAdrA        ,
    RfRdAdrB        ,
    RfRdAdrC        ,

    EscapeSR        ,
    ReturnSR        ,

    SetIE           ,
    ClrIE           ,

    SetICE          ,
    ClrICE          ,

    SetDCE          ,
    ClrDCE          ,

    SetBL           ,
    ClrBL           ,

    SR              ,
    IEforID         ,

    //======== from/to EXU ==========
    UpDateCV        ,
    UpDateSZ        ,
    Over            ,
    Carry           ,
    Sign            ,
    Zero            ,

    RfRdDataA       ,
    RfRdDataB       ,
    RfRdDataC       ,
    RfRdDA          , // 20100327 add
    RfRdDC          ,

    RegAFwdEna      ,
    RegBFwdEna      ,
    RegCFwdEna      ,
    FwdData         ,

    //======== DEBUG================
    DbgRegRq        ,
    DbgRegWe        ,
    DbgRegAdr       ,
    DbgWD           ,
    DbgRD

);
    //==================================================
    //======== port  ===================================
    //==================================================
    //======== SYSTEM ==============
    input           Clk             ;
    input           RstN            ;
    input           PID             ;

    input           BusLockAK       ; // Bus Lock ACK

    //======== from P3PlCtrl ===========
    input           GoAhead         ;
    input           DecInstAK       ; // Instruction Reqest From DECU
    input           RegWeMask       ; // 1:Mask, 0:NotMask

    //======== from/to P3Id ===========
    input   [25:0]  PC0             ; // Next Program Counter Value
    input   [25:0]  PC1             ; // Current Program Counter Value

    input   [4:0]   RfWrAdrEx       ; // Register file Write Address
    input           RfWeEx          ; // Register file Write Enable
    input   [31:0]  RfWrDEx         ; // Register file Write Data

    input   [3:0]   RfRdAdrA        ; // Register file Read Address A
    input   [3:0]   RfRdAdrB        ; // Register file Read Address B
    input   [4:0]   RfRdAdrC        ; // Register file Read Address C

    input           EscapeSR        ; // EscapeSR
    input           ReturnSR        ; // ReturnSR

    input           SetIE           ; // SET   IE@SR
    input           ClrIE           ; // CLEAR IE@SR

    input           SetICE          ; // SET   ICE@SR
    input           ClrICE          ; // CLEAR ICE@SR

    input           SetDCE          ; // SET   DCE@SR
    input           ClrDCE          ; // CLEAR DCE@SR

    input           SetBL           ; // SET   BL@SR
    input           ClrBL           ; // CLEAR BL@SR

    output  [7:0]   SR              ; //0:Z, 1:S, 2:C, 3:V, 4:IE, 5:ICE, 6:DCE, 7:BL
    output          IEforID         ; // IrqEnable for Instruction Decorder

    //======== from/to EXU ===========
    input           UpDateCV        ; // UpDate Carry & Over
    input           UpDateSZ        ; // UpDate Sign & Zero
    input           Over            ; // Flag
    input           Carry           ; // Flag
    input           Sign            ; // Flag
    input           Zero            ; // Flag

    output  [31:0]  RfRdDataA       ; // PortA Read Data
    output  [31:0]  RfRdDataB       ; // PortB Read Data
    output  [31:0]  RfRdDataC       ; // PortC Read Data

    output  [31:0]  RfRdDA          ; // GPR PortA Read Data
    output  [31:0]  RfRdDC          ; // GPR PortC Read Data
    
    input           RegAFwdEna      ; // Reg A Forwarding Enable // 20100327 add
    input           RegBFwdEna      ; // Reg B Forwarding Enable // 20100327 add
    input           RegCFwdEna      ; // Reg C Forwarding Enable // 20100327 add
    input   [31:0]  FwdData         ; // Data for forwarding // 20100327

    //======== DEBUG================
    input           DbgRegRq        ; // Debug Reg Access Request from Debugger
    input           DbgRegWe        ; // Register Write Enable from Debugger
    input   [4:0]   DbgRegAdr       ; // Register Address from Debugger
    input   [31:0]  DbgWD           ; // Register Write Data from Debugger
    output  [31:0]  DbgRD           ; // Register Read  Data to   Debugger


    //==================================================
    //======== wire & reg ==============================
    //==================================================
    wire    [4:0]   RfRdAdrC1       ; // Register file Read Address C
    wire    [4:0]   RfWrA           ;
    wire            RfWe            ;
    wire    [31:0]  RfWrD           ;
    reg     [31:0]  RfRdDataA       ; // PortA Read Data
    reg     [31:0]  RfRdDataB       ; // PortB Read Data
    reg     [31:0]  RfRdDataC       ; // PortC Read Data
    wire            IsrWe           ;
    wire            DbgSrWe         ;
    reg     [7:0]   SR              ; // Status Register 0:Z, 1:S, 2:C, 3:V, 4:IE, 5:ICE, 6:DCE, 7:BL
    reg     [7:0]   ISR             ; // Escape Status Register //0:Z, 1:S, 2:C, 3:V, 4:IE, 5:ICE, 6:DCE, 7:BL
    reg     [31:0]  DbgRD           ;
    reg             IEforID         ; // IrqEnable for Instruction Decorder
    //wire    [31:0]  RfRdDA          ; // GPR Read Data // 20100327 remove
    wire    [31:0]  RfRdDB          ; // GPR Read Data
    reg             BusLockAK1d     ;
    wire    [31:0]  RfRdDC0         ; // GPR PortC Read Data // 20100327 add

    //==================================================
    //======== BODY       ==============================
    //==================================================
    //======== Register Write Access Signals ===========
    assign  RfWrA = (DbgRegRq)? DbgRegAdr : RfWrAdrEx;
    assign  RfWrD = (DbgRegRq)? DbgWD     : RfWrDEx;
    assign  RfWe  = (DbgRegRq)? DbgRegWe  : (~RegWeMask & RfWeEx);

    //======== GPR =====================================
    GPR GPR(
        .Clk            ( Clk               ),
        .RfWrA          ( RfWrA[3:0]        ),
        .RfWe           ( RfWe & ~RfWrA[4]  ),
        .RfWrD          ( RfWrD             ),
        .RfRdAdrA       ( RfRdAdrA          ),
        .RfRdAdrB       ( RfRdAdrB          ),
        .RfRdAdrC       ( RfRdAdrC1[3:0]    ),
        .RfRdDA         ( RfRdDA            ),
        .RfRdDB         ( RfRdDB            ),
        //.RfRdDC         ( RfRdDC            ) // 20100327 remove
        .RfRdDC         ( RfRdDC0            ) // 20100327 add
    );

    //======== RfRdDataA =====================================
    //always @(RfRdDA) RfRdDataA = RfRdDA; // 20100327 remove
    always @(RfRdDA or FwdData or RegAFwdEna) begin // 20100327 add
        if(RegAFwdEna) RfRdDataA = FwdData;
        else           RfRdDataA = RfRdDA;
    end
    //======== RfRdDataB =====================================
    //always @(RfRdAdrB or RfRdDB or PC1) begin // 20100327 remove
    //    if(RfRdAdrB[3:0] == 4'hF) RfRdDataB = {6'h0, PC1};
    //    else                      RfRdDataB = RfRdDB;
    //end
    always @(RfRdDB or FwdData or RegBFwdEna) begin // 20100327 add
        if(RegBFwdEna) RfRdDataB = FwdData;
        else           RfRdDataB = RfRdDB;
    end

    //======== BusLockAK1d =====================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)   BusLockAK1d <= 1'b0;
        else        BusLockAK1d <= BusLockAK;
    end

    //======== RfRdDC ===================================== 
    assign  RfRdDC = (RegCFwdEna)? FwdData : RfRdDC0;

    //======== RfRdDataC =====================================
    assign RfRdAdrC1 = (DbgRegRq)? DbgRegAdr : RfRdAdrC;
    //always @(RfRdAdrC1 or RfRdDC or PID or ISR or SR or PC1 or BusLockAK1d) begin  // 20100327 remove
    always @(RfRdAdrC1 or RfRdDC0 or PID or ISR or SR or BusLockAK1d or FwdData or RegCFwdEna) begin // 20100327 add
        if(RfRdAdrC1[4]) begin
            if(RfRdAdrC1[1:0] == 2'b01) RfRdDataC = {16'h0, PID, 7'h0, ISR};
            else                        RfRdDataC = {16'h0, PID, 7'h0, BusLockAK1d, SR[6:0]};
        end
        else begin
            //if(RfRdAdrC1[3:0] == 4'hF) RfRdDataC = {6'h0, PC1}; // 20100327 remove
            //else                       RfRdDataC = RfRdDC;
            if(RegCFwdEna) RfRdDataC = FwdData;
            else           RfRdDataC = RfRdDC0;
        end
    end



    //======== DbgRD =====================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)                        DbgRD <= 32'h0;
        else if(RfRdAdrC1 == 5'h10)      DbgRD <= {6'h0, PC0}; // 20100327
        else if(RfRdAdrC1 == 5'h0F)      DbgRD <= {6'h0, PC1}; // 20100327
        else                             DbgRD <= RfRdDataC;
    end

    //=====================================================
    //========= ISR =======================================
    //=====================================================
    assign  IsrWe = RfWe & (RfWrA[4:0] == 5'h11);

    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)                   ISR <= 8'h0;
        else if(IsrWe)              ISR <= RfWrD[7:0];
        else if(GoAhead & EscapeSR) ISR <= {SR[7:5], 1'b1, SR[3:0]};
        //else if(DecInstAK & EscapeSR) ISR <= {SR[6:5], 1'b1, SR[3:0]};
    end

    //=====================================================
    //========= SR ========================================
    //=====================================================
    assign  DbgSrWe = DbgRegRq & DbgRegWe & (DbgRegAdr == 5'h12);

    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)               SR[7] <= 1'b0;
        else if(DbgSrWe)        SR[7] <= DbgWD[7];
        else if(GoAhead)
        //else if(DecInstAK)
            if      (SetBL )    SR[7] <= 1'b1;
            else  if(ClrBL )    SR[7] <= 1'b0;
            else  if(ReturnSR ) SR[7] <= ISR[7];
    end

    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)               SR[6] <= 1'b0;
        else if(DbgSrWe)        SR[6] <= DbgWD[6];
        else if(GoAhead)
        //else if(DecInstAK)
            if      (SetDCE )   SR[6] <= 1'b1;
            else  if(ClrDCE )   SR[6] <= 1'b0;
            else  if(ReturnSR ) SR[6] <= ISR[6];
    end

    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)               SR[5] <= 1'b0;
        else if(DbgSrWe)        SR[5] <= DbgWD[5];
        else if(GoAhead)
        //else if(DecInstAK)
            if      (SetICE )   SR[5] <= 1'b1;
            else  if(ClrICE )   SR[5] <= 1'b0;
            else  if(ReturnSR ) SR[5] <= ISR[5];
    end

    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)               SR[4] <= 1'b0;
        else if(DbgSrWe)        SR[4] <= DbgWD[4];
        else if(GoAhead)
        //else if(DecInstAK)
            if      (SetIE )    SR[4] <= 1'b1;
            else  if(ClrIE )    SR[4] <= 1'b0;
            else  if(ReturnSR ) SR[4] <= ISR[4];
    end

    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)               IEforID <= 1'b0;
        else if(DbgSrWe)        IEforID <= DbgWD[4];
        //else if(GoAhead)
        else if(DecInstAK)
            if      (SetIE )    IEforID <= 1'b1;
            else  if(ClrIE )    IEforID <= 1'b0;
            else  if(ReturnSR ) IEforID <= ISR[4];
    end


    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)                SR[3] <= 1'b0;
        else if(DbgSrWe)         SR[3] <= DbgWD[3];
        else if(UpDateCV & RfWe) SR[3] <= Over;
        else if(ReturnSR)        SR[3] <= ISR[3];
    end

    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)                SR[2] <= 1'b0;
        else if(DbgSrWe)         SR[2] <= DbgWD[2];
        else if(UpDateCV & RfWe) SR[2] <= Carry;
        else if(ReturnSR)        SR[2] <= ISR[2];
    end

    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)                SR[1] <= 1'b0;
        else if(DbgSrWe)         SR[1] <= DbgWD[1];
        else if(UpDateSZ & RfWe) SR[1] <= Sign;
        else if(ReturnSR)        SR[1] <= ISR[1];
    end

    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)                SR[0] <= 1'b0;
        else if(DbgSrWe)         SR[0] <= DbgWD[0];
        else if(UpDateSZ & RfWe) SR[0] <= Zero;
        else if(ReturnSR)        SR[0] <= ISR[0];
    end


endmodule


/*=======================================================================*
 *  TARGET PROJECT: PSM3(Prosessor Study Model3)                         *
 *  TARGET SYSTEM : *(PSM3 TARGET SYSTEM)                                *
 *  FUNCTION      : Genaral Purpose Register                             *
 *=======================================================================*/
module GPR(
    //======== SYSTEM ==============
    Clk             ,

    //======== WRITE ==============
    RfWrA           ,
    RfWe            ,
    RfWrD           ,

    //======== READ ==============
    RfRdAdrA        ,
    RfRdAdrB        ,
    RfRdAdrC        ,

    RfRdDA          ,
    RfRdDB          ,
    RfRdDC
);
    //==================================================
    //======== port  ===================================
    //==================================================
    //======== SYSTEM ==============
    input           Clk             ;

    //======== WRITE ==============
    input   [3:0]   RfWrA           ;
    input           RfWe            ;
    input   [31:0]  RfWrD           ;

    //======== READ ==============
    input   [3:0]   RfRdAdrA        ; // Register file Read Address A
    input   [3:0]   RfRdAdrB        ; // Register file Read Address B
    input   [3:0]   RfRdAdrC        ; // Register file Read Address C

    output  [31:0]  RfRdDA          ;
    output  [31:0]  RfRdDB          ;
    output  [31:0]  RfRdDC          ;

    //==================================================
    //======== wire & reg ==============================
    //==================================================
    reg     [31:0]  GPR[0:15]       ;

    //==================================================
    //======== BODY       ==============================
    //==================================================
    always  @(posedge Clk) if(RfWe) GPR[RfWrA] <= RfWrD;
    assign  RfRdDA = GPR[RfRdAdrA];
    assign  RfRdDB = GPR[RfRdAdrB];
    assign  RfRdDC = GPR[RfRdAdrC];

endmodule
