//=======================================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : *(PSM3 TARGET SYSTEM)
//  FUNCTION      : Top Module of Psm3(Core + L1-Cache)
//
//                      Copyright (C) 2008-2010 by Tomoyuki Inomoto
//=======================================================================
// 20100403:add Branch Prediction
// 20100529:Cash->Cache (spell miss)
//=======================================================================
module P3Top(
    //======== SYSTEM ========================
    Clk             ,
    RstN            ,
    Vector          ,
    PID             ,
    IRQ             ,
    BusLockRQ       ,
    BusLockAK       ,

    //======= to/from Common Unit ======
    ComInB          ,
    ComInC          ,
    ComFunc         ,

    ComBSftRQ       ,
    ComBSftAK       ,
    ComBSftCmpl     ,
    ComBSftRes      ,

    ComDivRQ        ,
    ComDivAK        ,
    ComDivCmpl      ,
    ComDivRes       ,

    //======== INST BUS ================
    InstCRQ         ,
    InstCAK         ,
    InstBRST        ,
    InstADR         ,
    InstRD          ,
    InstRDAK        ,

    //======== DATA BUS ==================
    DataCRQ         ,
    DataCAK         ,
    DataBRST        ,
    DataRW          ,
    DataBE          ,
    DataADR         ,
    DataWD          ,
    DataWDAK        ,
    DataRD          ,
    DataRDAK        ,

    //======== DEBUG================
    CacheClr        ,
    Break           ,
    DbgRq           ,
    DbgRegRq        ,
    DbgEna          ,
    DbgRegWe        ,
    DbgRegAdr       ,
    DbgWD           ,
    DbgRD           ,
    DecInstAK       ,
    Bubble          ,

    //======== MONITOR ================
    CoreGoAhead     ,
    CoreInstRQ      ,
    CoreInstAK      ,
    CoreInstHit     ,
    CoreDataRQ      ,
    CoreDataAK      ,
    CoreDataHit
);
    //============================================================
    //======== port  =============================================
    //============================================================
    //======== SYSTEM ========================
    input           Clk             ; // Clock
    input           RstN            ; // Reset (L-Active)
    input   [25:0]  Vector          ; // Vector of IRQ
    input           PID             ; // Processor ID
    input           IRQ             ; // Interrupt Reqest
    output          BusLockRQ       ; // Bus Lock Reqest
    input           BusLockAK       ; // Bus Lock ACK

    //======= to/from Common Unit =================
    output  [31:0]  ComInB          ; // PortB Data
    output  [31:0]  ComInC          ; // PortC Data
    output  [3:0]   ComFunc         ; // FUNCTION

    output          ComBSftRQ       ; // ComBSft REQ
    input           ComBSftAK       ; // ComBSft ACK
    input           ComBSftCmpl     ; // ComBSft Complete
    input   [31:0]  ComBSftRes      ; // Result of ComBSft

    output          ComDivRQ        ; // Common Divisor REQ
    input           ComDivAK        ; // Common Divisor ACK
    input           ComDivCmpl      ; // Common Divisor Complete
    input   [31:0]  ComDivRes       ; // Result of Common Divisor

    //======== INST BUS ================
    output          InstCRQ         ; // INSTRUCTION BUS COMMAND REQ
    input           InstCAK         ; // INSTRUCTION BUS COMMAND ACK
    output          InstBRST        ; // INSTRUCTION BUS SINGLE/BURST
    output  [25:0]  InstADR         ; // INSTRUCTION BUS ADDRESS
    input   [31:0]  InstRD          ; // INSTRUCTION BUS READ DATA
    input           InstRDAK        ; // INSTRUCTION BUS READ DATA ACK

    //======== DATA BUS ==================
    output          DataCRQ         ; // DATA BUS COMMAND REQ
    input           DataCAK         ; // DATA BUS COMMAND ACK
    output          DataBRST        ; // DATA BUS SINGLE/BURST
    output          DataRW          ; // DATA BUS READ/WRITE (1:WRITE)
    output  [3:0]   DataBE          ; // DATA BUS BYTE ENABLE
    output  [27:0]  DataADR         ; // DATA BUS ADDRESS
    output  [31:0]  DataWD          ; // DATA BUS WRITE DATA
    input           DataWDAK        ; // DATA BUS WRITE DATA ACK
    input   [31:0]  DataRD          ; // DATA BUS READ DATA
    input           DataRDAK        ; // DATA BUS READ DATA ACK

    //======== DEBUG================
    input           CacheClr        ; // Cache Clear Signal frim Debugger
    output          Break           ; // BREAK Instruction Detect Signal to Debugger
    input           DbgRq           ; // Debug Reqest from Debugger
    input           DbgRegRq        ; // Debug Reg Access Request from Debugger
    output          DbgEna          ; // Debug Enable to Debugger
    input           DbgRegWe        ; // Register Write Enable from Debugger
    input   [4:0]   DbgRegAdr       ; // Register Address from Debugger
    input   [31:0]  DbgWD           ; // Register Write Data from Debugger
    output  [31:0]  DbgRD           ; // Register Read  Data to   Debugger
    output          DecInstAK       ; // Ack of Instruction Reqest to DECU
    output          Bubble          ; // Bubble

    //======== MONITOR ================
    output          CoreGoAhead     ;
    output          CoreInstRQ      ;
    output          CoreInstAK      ;
    output          CoreInstHit     ;
    output          CoreDataRQ      ;
    output          CoreDataAK      ;
    output          CoreDataHit     ;
    //============================================================
    //======== wire & reg ========================================
    //============================================================
    //======== SYSTEM ========================
    wire            ICE             ; // Inst Cache Enable
    wire            DCE             ; // Data Cache Enable

    //======== INTERNAL INST BUS ================
    //wire            CoreInstRQ      ; // CORE INSTRUCTION BUS REQ
    //wire            CoreInstAK      ; // CORE INSTRUCTION BUS ACK
    wire    [25:0]  CoreInstADR     ; // CORE INSTRUCTION BUS ADDRESS
    wire    [31:0]  CoreInstRD      ; // CORE INSTRUCTION BUS READ DATA
    wire            CoreBrInst      ; // 20100403 indicate Inst is branch
    wire    [1:0]   CoreBrCountIC2IF;// 20100403
    wire    [1:0]   CoreBrCountIF2IC; // 20100403
    wire            CoreBrCountWe   ; // 20100403

    
    //======== INST Cache ================
    wire            ICacheRe       ; // Instruction Cache Read Enable
    wire    [11:2]  ICacheIndex    ; // Instruction Cache Index (4KByte)

    //======== INTERNAL DATA BUS ==================
    //wire            CoreDataRQ      ; // CORE DATA BUS REQ
    //wire            CoreDataAK      ; // CORE DATA BUS ACK
    wire            CoreDataRW      ; // CORE DATA BUS READ/WRITE (1:WRITE)
    wire    [3:0]   CoreDataBE      ; // CORE DATA BUS BYTE ENABLE
    wire    [27:0]  CoreDataADR     ; // CORE DATA BUS ADDRESS
    wire    [31:0]  CoreDataWD      ; // CORE DATA BUS WRITE DATA
    wire    [31:0]  CoreDataRD      ; // CORE DATA BUS READ DATA

    //======== DATA Cache =================
    //wire            DCacheDrainRQ    ; // Data Cache Drain RQ (for Write Back Cache) // 20100403
    wire            DCacheRe        ; // Data Cache Read Enable
    wire    [27:23] DCacheArea      ; // Data Cache Accsess Area
    wire    [11:2]  DCacheIndex     ; // Data Cache Index (4KByte)
    wire            LockByDCache    ; // InterLock Signal from DCache

    //======== DEBUG =================
    reg             CacheClr1d      ; // Buffer of CacheClr

    //============================================================
    //======== BODY       ========================================
    //============================================================
    //=========================================
    //======== PSM3 Core ======================
    //=========================================
    P3Core P3Core(
        .Clk             ( Clk             ),
        .RstN            ( RstN            ),
        .Vector          ( Vector          ),
        .PID             ( PID             ),
        .ICE             ( ICE             ),
        .DCE             ( DCE             ),
        .IRQ             ( IRQ             ),
        .BusLockRQ       ( BusLockRQ       ),
        .BusLockAK       ( BusLockAK       ),
        .LockByDCache    ( LockByDCache    ),

        .ComInB          ( ComInB          ),
        .ComInC          ( ComInC          ),
        .ComFunc         ( ComFunc         ),

        .ComBSftRQ       ( ComBSftRQ       ),
        .ComBSftAK       ( ComBSftAK       ),
        .ComBSftCmpl     ( ComBSftCmpl     ),
        .ComBSftRes      ( ComBSftRes      ),

        .ComDivRQ        ( ComDivRQ        ),
        .ComDivAK        ( ComDivAK        ),
        .ComDivCmpl      ( ComDivCmpl      ),
        .ComDivRes       ( ComDivRes       ),

        .InstRQ          ( CoreInstRQ      ),
        .InstAK          ( CoreInstAK      ),
        .InstADR         ( CoreInstADR     ),
        .InstRD          ( CoreInstRD      ),
        .InstBrInst      ( CoreBrInst      ), // 20100403
        .InstBrCountRd   ( CoreBrCountIC2IF), // 20100403
        .InstBrCountWr   ( CoreBrCountIF2IC), // 20100403
        .InstBrCountWe   ( CoreBrCountWe   ), // 20100403
        .ICacheRe        ( ICacheRe        ),
        .ICacheIndex     ( ICacheIndex     ),
        .DataRQ          ( CoreDataRQ      ),
        .DataAK          ( CoreDataAK      ),
        .DataRW          ( CoreDataRW      ),
        .DataBE          ( CoreDataBE      ),
        .DataADR         ( CoreDataADR     ),
        .DataWD          ( CoreDataWD      ),
        .DataRD          ( CoreDataRD      ),
        //.DCacheDrainRQ    ( DCacheDrainRQ    ), // 20100403
        .DCacheRe        ( DCacheRe        ),
        .DCacheArea      ( DCacheArea      ),
        .DCacheIndex     ( DCacheIndex     ),
        .Break           ( Break           ),
        .DbgRq           ( DbgRq           ),
        .DbgRegRq        ( DbgRegRq        ),
        .DbgEna          ( DbgEna          ),
        .DbgRegWe        ( DbgRegWe        ),
        .DbgRegAdr       ( DbgRegAdr       ),
        .DbgWD           ( DbgWD           ),
        .DbgRD           ( DbgRD           ),
        .DecInstAK       ( DecInstAK       ),
        .Bubble          ( Bubble          ),
        .CoreGoAhead     ( CoreGoAhead     )
    );
    //=========================================
    //======== CacheClr1d  =====================
    //=========================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)   CacheClr1d <= 2'h0;
        else        CacheClr1d <= CacheClr & DbgRq;
    end

    //=========================================
    //======== L1-Instruction Cache ============
    //=========================================
    P3ICache4K32b4Brst P3ICache( // 4KByte
        .Clk             ( Clk             ),
        .RstN            ( RstN            ),
        .ICE             ( ICE & ~CacheClr1d),
        .MstrRQ          ( CoreInstRQ      ),
        .MstrAK          ( CoreInstAK      ),
        .MstrADR         ( CoreInstADR     ),
        .MstrRD          ( CoreInstRD      ),
        .MstrBrInst      ( CoreBrInst      ), // 20100403
        .MstrBrCountOut  ( CoreBrCountIC2IF), // 20100403
        .MstrBrCountIn   ( CoreBrCountIF2IC), // 20100403
        .MstrBrCountWe   ( CoreBrCountWe   ), // 20100403
        .MstrICRe        ( ICacheRe        ),
        .MstrICIndex     ( ICacheIndex     ),
        .ExCRQ           ( InstCRQ         ),
        .ExCAK           ( InstCAK         ),
        .ExBRST          ( InstBRST        ),
        .ExADR           ( InstADR         ),
        .ExRD            ( InstRD          ),
        .ExRDAK          ( InstRDAK        ),
        .CoreInstHit     ( CoreInstHit     )
    );

    //=========================================
    //======== L1-Data Cache ===================
    //=========================================
    P3DCache4K32bWB P3DCache( // 4KByte
        .Clk             ( Clk             ),
        .RstN            ( RstN            ),
        .DCE             ( DCE & ~CacheClr1d),
        .MstrRQ          ( CoreDataRQ      ),
        .MstrAK          ( CoreDataAK      ),
        .MstrRW          ( CoreDataRW      ),
        .MstrBE          ( CoreDataBE      ),
        .MstrADR         ( CoreDataADR     ),
        .MstrWD          ( CoreDataWD      ),
        .MstrRD          ( CoreDataRD      ),
        //.MstrDCDrainRQ   ( DCacheDrainRQ    ), // 20100403
        .MstrDCDrainRQ   ( 1'b0            ), // 20100403
        .MstrDCRe        ( DCacheRe        ),
        .MstrDCArea      ( DCacheArea      ),
        .MstrDCIndex     ( DCacheIndex     ),
        .MstrLock        ( LockByDCache    ),
        .ExCRQ           ( DataCRQ         ),
        .ExCAK           ( DataCAK         ),
        .ExBRST          ( DataBRST        ),
        .ExRW            ( DataRW          ),
        .ExBE            ( DataBE          ),
        .ExADR           ( DataADR         ),
        .ExWD            ( DataWD          ),
        .ExWDAK          ( DataWDAK        ),
        .ExRD            ( DataRD          ),
        .ExRDAK          ( DataRDAK        ),
        .CoreDataHit     ( CoreDataHit     )
    );

endmodule
