//=======================================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : *(PSM3 TARGET SYSTEM)
//  FUNCTION      : Top Module of Dual PSM3d(Core + L1-Cache)
//
//                      Copyright (C) 2008-2010 by Tomoyuki Inomoto
//=======================================================================
// 20100529:Cash->Cache (spell miss)
//=======================================================================

module P3dTop(
    //======== SYSTEM ===============
    Clk             ,
    RstN            ,

    CacheClr         ,


    //======== P0 SYSTEM ============
    P0IRQ           ,

    //======== P0 INST BUS ==========
    P0InstCRQ       ,
    P0InstCAK       ,
    P0InstBRST      ,
    P0InstADR       ,
    P0InstRD        ,
    P0InstRDAK      ,

    //======== P0 DATA BUS ==========
    P0DataCRQ       ,
    P0DataCAK       ,
    P0DataBRST      ,
    P0DataRW        ,
    P0DataBE        ,
    P0DataADR       ,
    P0DataWD        ,
    P0DataWDAK      ,
    P0DataRD        ,
    P0DataRDAK      ,

    //======== P0 DEBUG==============
    P0Break         ,
    P0DbgRegRq      ,
    P0DbgRq         ,
    P0DbgEna        ,
    P0DbgRegWe      ,
    P0DbgRegAdr     ,
    P0DbgWD         ,
    P0DbgRD         ,
    P0DecInstAK     ,
    P0Bubble        ,

    //======== P0 MONITOR ===========
    P0CoreGoAhead   ,
    P0CoreInstRQ    ,
    P0CoreInstAK    ,
    P0CoreInstHit   ,
    P0CoreDataRQ    ,
    P0CoreDataAK    ,
    P0CoreDataHit   ,


    //======== P1 SYSTEM ============
    P1IRQ           ,

    //======== P1 INST BUS ==========
    P1InstCRQ       ,
    P1InstCAK       ,
    P1InstBRST      ,
    P1InstADR       ,
    P1InstRD        ,
    P1InstRDAK      ,

    //======== P1 DATA BUS ==========
    P1DataCRQ       ,
    P1DataCAK       ,
    P1DataBRST      ,
    P1DataRW        ,
    P1DataBE        ,
    P1DataADR       ,
    P1DataWD        ,
    P1DataWDAK      ,
    P1DataRD        ,
    P1DataRDAK      ,

    //======== P1 DEBUG==============
    P1Break         ,
    P1DbgRq         ,
    P1DbgRegRq      ,
    P1DbgEna        ,
    P1DbgRegWe      ,
    P1DbgRegAdr     ,
    P1DbgWD         ,
    P1DbgRD         ,
    P1DecInstAK     ,
    P1Bubble        ,

    //======== P1 MONITOR ===========
    P1CoreGoAhead   ,
    P1CoreInstRQ    ,
    P1CoreInstAK    ,
    P1CoreInstHit   ,
    P1CoreDataRQ    ,
    P1CoreDataAK    ,
    P1CoreDataHit
);
    //============================================================
    //======== port  =============================================
    //============================================================
    //======== SYSTEM ========================
    input           Clk             ; // Clock
    input           RstN            ; // Reset (L-Active)

    input           CacheClr        ; // Cache Clear Signal frim Debugger

    //======== P0 SYSTEM ========================
    input           P0IRQ           ; // Interrupt Reqest

    //======== P0 INST BUS ================
    output          P0InstCRQ       ; // INSTRUCTION BUS COMMAND REQ
    input           P0InstCAK       ; // INSTRUCTION BUS COMMAND ACK
    output          P0InstBRST      ; // INSTRUCTION BUS SINGLE/BURST
    output  [25:0]  P0InstADR       ; // INSTRUCTION BUS ADDRESS
    input   [31:0]  P0InstRD        ; // INSTRUCTION BUS READ DATA
    input           P0InstRDAK      ; // INSTRUCTION BUS READ DATA ACK

    //======== P0 DATA BUS ==================
    output          P0DataCRQ       ; // DATA BUS COMMAND REQ
    input           P0DataCAK       ; // DATA BUS COMMAND ACK
    output          P0DataBRST      ; // DATA BUS SINGLE/BURST
    output          P0DataRW        ; // DATA BUS READ/WRITE (1:WRITE)
    output  [3:0]   P0DataBE        ; // DATA BUS BYTE ENABLE
    output  [27:0]  P0DataADR       ; // DATA BUS ADDRESS
    output  [31:0]  P0DataWD        ; // DATA BUS WRITE DATA
    input           P0DataWDAK      ; // DATA BUS WRITE DATA ACK
    input   [31:0]  P0DataRD        ; // DATA BUS READ DATA
    input           P0DataRDAK      ; // DATA BUS READ DATA ACK

    //======== P0 DEBUG================
    output          P0Break         ; // BREAK Instruction Detect Signal to Debugger
    input           P0DbgRq         ; // Debug Reqest from Debugger
    input           P0DbgRegRq      ; // Debug Reg Access Request from Debugger
    output          P0DbgEna        ; // Debug Enable to Debugger
    input           P0DbgRegWe      ; // Register Write Enable from Debugger
    input   [4:0]   P0DbgRegAdr     ; // Register Address from Debugger
    input   [31:0]  P0DbgWD         ; // Register Write Data from Debugger
    output  [31:0]  P0DbgRD         ; // Register Read  Data to   Debugger
    output          P0DecInstAK     ; // Ack of Instruction Reqest to DECU
    output          P0Bubble        ; // Bubble

    //======== P0 MONITOR ================
    output          P0CoreGoAhead   ;
    output          P0CoreInstRQ    ;
    output          P0CoreInstAK    ;
    output          P0CoreInstHit   ;
    output          P0CoreDataRQ    ;
    output          P0CoreDataAK    ;
    output          P0CoreDataHit   ;

    //======== P1 SYSTEM ========================
    input           P1IRQ           ; // Interrupt Reqest

    //======== P1 INST BUS ================
    output          P1InstCRQ       ; // INSTRUCTION BUS COMMAND REQ
    input           P1InstCAK       ; // INSTRUCTION BUS COMMAND ACK
    output          P1InstBRST      ; // INSTRUCTION BUS SINGLE/BURST
    output  [25:0]  P1InstADR       ; // INSTRUCTION BUS ADDRESS
    input   [31:0]  P1InstRD        ; // INSTRUCTION BUS READ DATA
    input           P1InstRDAK      ; // INSTRUCTION BUS READ DATA ACK

    //======== P1 DATA BUS ==================
    output          P1DataCRQ       ; // DATA BUS COMMAND REQ
    input           P1DataCAK       ; // DATA BUS COMMAND ACK
    output          P1DataBRST      ; // DATA BUS SINGLE/BURST
    output          P1DataRW        ; // DATA BUS READ/WRITE (1:WRITE)
    output  [3:0]   P1DataBE        ; // DATA BUS BYTE ENABLE
    output  [27:0]  P1DataADR       ; // DATA BUS ADDRESS
    output  [31:0]  P1DataWD        ; // DATA BUS WRITE DATA
    input           P1DataWDAK      ; // DATA BUS WRITE DATA ACK
    input   [31:0]  P1DataRD        ; // DATA BUS READ DATA
    input           P1DataRDAK      ; // DATA BUS READ DATA ACK

    //======== P1 DEBUG================
    output          P1Break         ; // BREAK Instruction Detect Signal to Debugger
    input           P1DbgRq         ; // Debug Reqest from Debugger
    input           P1DbgRegRq      ; // Debug Reg Access Request from Debugger
    output          P1DbgEna        ; // Debug Enable to Debugger
    input           P1DbgRegWe      ; // Register Write Enable from Debugger
    input   [4:0]   P1DbgRegAdr     ; // Register Address from Debugger
    input   [31:0]  P1DbgWD         ; // Register Write Data from Debugger
    output  [31:0]  P1DbgRD         ; // Register Read  Data to   Debugger
    output          P1DecInstAK     ; // Ack of Instruction Reqest to DECU
    output          P1Bubble        ; // Bubble

    //======== P1 MONITOR ================
    output          P1CoreGoAhead   ;
    output          P1CoreInstRQ    ;
    output          P1CoreInstAK    ;
    output          P1CoreInstHit   ;
    output          P1CoreDataRQ    ;
    output          P1CoreDataAK    ;
    output          P1CoreDataHit   ;


    //============================================================
    //======== wire & reg ========================================
    //============================================================
    wire            P0BusLockRQ     ;
    wire            P0BusLockAK     ;
    wire            P1BusLockRQ     ;
    wire            P1BusLockAK     ;
    wire            P0DataCRQ0      ;
    wire            P1DataCRQ0      ;

    //======= to/from Common Unit =================
    wire    [31:0]  P0ComInB        ; // PortB Data
    wire    [31:0]  P0ComInC        ; // PortC Data
    wire    [3:0]   P0ComFunc       ; // FUNCTION
    wire    [31:0]  P1ComInB        ; // PortB Data
    wire    [31:0]  P1ComInC        ; // PortC Data
    wire    [3:0]   P1ComFunc       ; // FUNCTION

    wire            P0ComBSftRQ     ; // Common Barrel Shifter REQ
    wire            P0ComBSftAK     ; // Common Barrel Shifter ACK
    wire            P0ComBSftCmpl   ; // Common Barrel Shifter Complete
    wire            P1ComBSftRQ     ; // Common Barrel Shifter REQ
    wire            P1ComBSftAK     ; // Common Barrel Shifter ACK
    wire            P1ComBSftCmpl   ; // Common Barrel Shifter Complete
    wire    [31:0]  ComBSftRes      ; // Result of Common Barrel Shifter

    wire            P0ComDivRQ     ; // Common Divisor REQ
    wire            P0ComDivAK     ; // Common Divisor ACK
    wire            P0ComDivCmpl   ; // Common Divisor Complete
    wire            P1ComDivRQ     ; // Common Divisor REQ
    wire            P1ComDivAK     ; // Common Divisor ACK
    wire            P1ComDivCmpl   ; // Common Divisor Complete
    wire    [31:0]  ComDivRes      ; // Result of Common Divisor

    //============================================================
    //======== BODY       ========================================
    //============================================================
    //=========================================
    //======== P0 =============================
    //=========================================
    P3Top P0(
        .Clk             ( Clk             ),
        .RstN            ( RstN            ),
        .Vector          ( 26'h10          ),
        .PID             ( 1'b0            ),
        .IRQ             ( P0IRQ           ),
        .BusLockRQ       ( P0BusLockRQ     ),
        .BusLockAK       ( P0BusLockAK     ),

        .ComInB          ( P0ComInB        ),
        .ComInC          ( P0ComInC        ),
        .ComFunc         ( P0ComFunc       ),

        .ComBSftRQ       ( P0ComBSftRQ     ),
        .ComBSftAK       ( P0ComBSftAK     ),
        .ComBSftCmpl     ( P0ComBSftCmpl   ),
        .ComBSftRes      ( ComBSftRes      ),

        .ComDivRQ        ( P0ComDivRQ      ),
        .ComDivAK        ( P0ComDivAK      ),
        .ComDivCmpl      ( P0ComDivCmpl    ),
        .ComDivRes       ( ComDivRes       ),

        .InstCRQ         ( P0InstCRQ       ),
        .InstCAK         ( P0InstCAK       ),
        .InstBRST        ( P0InstBRST      ),
        .InstADR         ( P0InstADR       ),
        .InstRD          ( P0InstRD        ),
        .InstRDAK        ( P0InstRDAK      ),

        .DataCRQ         ( P0DataCRQ0      ),
        .DataCAK         ( P0DataCAK       ),
        .DataBRST        ( P0DataBRST      ),
        .DataRW          ( P0DataRW        ),
        .DataBE          ( P0DataBE        ),
        .DataADR         ( P0DataADR       ),
        .DataWD          ( P0DataWD        ),
        .DataWDAK        ( P0DataWDAK      ),
        .DataRD          ( P0DataRD        ),
        .DataRDAK        ( P0DataRDAK      ),

        .CacheClr        ( CacheClr        ),
        .Break           ( P0Break         ),
        .DbgRq           ( P0DbgRq         ),
        .DbgRegRq        ( P0DbgRegRq      ),
        .DbgEna          ( P0DbgEna        ),
        .DbgRegWe        ( P0DbgRegWe      ),
        .DbgRegAdr       ( P0DbgRegAdr     ),
        .DbgWD           ( P0DbgWD         ),
        .DbgRD           ( P0DbgRD         ),
        .DecInstAK       ( P0DecInstAK     ),
        .Bubble          ( P0Bubble        ),

        .CoreGoAhead     ( P0CoreGoAhead   ),
        .CoreInstRQ      ( P0CoreInstRQ    ),
        .CoreInstAK      ( P0CoreInstAK    ),
        .CoreInstHit     ( P0CoreInstHit   ),
        .CoreDataRQ      ( P0CoreDataRQ    ),
        .CoreDataAK      ( P0CoreDataAK    ),
        .CoreDataHit     ( P0CoreDataHit   )

    );
    //=========================================
    //======== P1 =============================
    //=========================================
    P3Top P1(
        .Clk             ( Clk             ),
        .RstN            ( RstN            ),
        .Vector          ( 26'h20          ),
        .PID             ( 1'b1            ),
        .IRQ             ( P1IRQ           ),
        .BusLockRQ       ( P1BusLockRQ     ),
        .BusLockAK       ( P1BusLockAK     ),

        .ComInB          ( P1ComInB        ),
        .ComInC          ( P1ComInC        ),
        .ComFunc         ( P1ComFunc       ),

        .ComBSftRQ       ( P1ComBSftRQ     ),
        .ComBSftAK       ( P1ComBSftAK     ),
        .ComBSftCmpl     ( P1ComBSftCmpl   ),
        .ComBSftRes      ( ComBSftRes      ),

        .ComDivRQ        ( P1ComDivRQ      ),
        .ComDivAK        ( P1ComDivAK      ),
        .ComDivCmpl      ( P1ComDivCmpl    ),
        .ComDivRes       ( ComDivRes       ),

        .InstCRQ         ( P1InstCRQ       ),
        .InstCAK         ( P1InstCAK       ),
        .InstBRST        ( P1InstBRST      ),
        .InstADR         ( P1InstADR       ),
        .InstRD          ( P1InstRD        ),
        .InstRDAK        ( P1InstRDAK      ),

        .DataCRQ         ( P1DataCRQ0      ),
        .DataCAK         ( P1DataCAK       ),
        .DataBRST        ( P1DataBRST      ),
        .DataRW          ( P1DataRW        ),
        .DataBE          ( P1DataBE        ),
        .DataADR         ( P1DataADR       ),
        .DataWD          ( P1DataWD        ),
        .DataWDAK        ( P1DataWDAK      ),
        .DataRD          ( P1DataRD        ),
        .DataRDAK        ( P1DataRDAK      ),

        .CacheClr        ( CacheClr        ),
        .Break           ( P1Break         ),
        .DbgRq           ( P1DbgRq         ),
        .DbgRegRq        ( P1DbgRegRq      ),
        .DbgEna          ( P1DbgEna        ),
        .DbgRegWe        ( P1DbgRegWe      ),
        .DbgRegAdr       ( P1DbgRegAdr     ),
        .DbgWD           ( P1DbgWD         ),
        .DbgRD           ( P1DbgRD         ),
        .DecInstAK       ( P1DecInstAK     ),
        .Bubble          ( P1Bubble        ),

        .CoreGoAhead     ( P1CoreGoAhead   ),
        .CoreInstRQ      ( P1CoreInstRQ    ),
        .CoreInstAK      ( P1CoreInstAK    ),
        .CoreInstHit     ( P1CoreInstHit   ),
        .CoreDataRQ      ( P1CoreDataRQ    ),
        .CoreDataAK      ( P1CoreDataAK    ),
        .CoreDataHit     ( P1CoreDataHit   )
    );

    //=========================================
    //======== BusLock Controler ==============
    //=========================================
    assign  P0DataCRQ = P0DataCRQ0 & ~P1BusLockAK;
    assign  P1DataCRQ = P1DataCRQ0 & ~P0BusLockAK;
    BusLockCntr BLC(
        .Clk             ( Clk             ),
        .RstN            ( RstN            ),

        .P0BusLockRQ     ( P0BusLockRQ     ),
        .P0BusLockAK     ( P0BusLockAK     ),
        .P0DataCRQ       ( P0DataCRQ0      ),

        .P1BusLockRQ     ( P1BusLockRQ     ),
        .P1BusLockAK     ( P1BusLockAK     ),
        .P1DataCRQ       ( P1DataCRQ0      )
    );

    //=========================================
    //======== Common Barrel Shifter ==========
    //=========================================
    P3ComBSft CBSft(
        .Clk             ( Clk             ),
        .RstN            ( RstN            ),

        .P0ComBSftRQ     ( P0ComBSftRQ     ),
        .P0ComBSftAK     ( P0ComBSftAK     ),
        .P0ComBSftCmpl   ( P0ComBSftCmpl   ),
        .P0ComBSftInB    ( P0ComInB        ),
        .P0ComBSftInC    ( P0ComInC[4:0]   ),
        .P0ComFunc       ( P0ComFunc       ),

        .P1ComBSftRQ     ( P1ComBSftRQ     ),
        .P1ComBSftAK     ( P1ComBSftAK     ),
        .P1ComBSftCmpl   ( P1ComBSftCmpl   ),
        .P1ComBSftInB    ( P1ComInB        ),
        .P1ComBSftInC    ( P1ComInC[4:0]   ),
        .P1ComFunc       ( P1ComFunc       ),

        .ComBSftRes      ( ComBSftRes      )
    );


    //=========================================
    //======== Common Divisor        ==========
    //=========================================
    P3ComDiv CDiv(
        .Clk             ( Clk             ),
        .RstN            ( RstN            ),

        .P0ComDivRQ      ( P0ComDivRQ      ),
        .P0ComDivAK      ( P0ComDivAK      ),
        .P0ComDivCmpl    ( P0ComDivCmpl    ),
        .P0ComDivInB     ( P0ComInB        ),
        .P0ComDivInC     ( P0ComInC        ),
        .P0ComFunc       ( P0ComFunc       ),

        .P1ComDivRQ      ( P1ComDivRQ      ),
        .P1ComDivAK      ( P1ComDivAK      ),
        .P1ComDivCmpl    ( P1ComDivCmpl    ),
        .P1ComDivInB     ( P1ComInB        ),
        .P1ComDivInC     ( P1ComInC        ),
        .P1ComFunc       ( P1ComFunc       ),

        .ComDivRes       ( ComDivRes       )
    );

endmodule

module BusLockCntr(
    //======== SYSTEM ===============
    Clk             ,
    RstN            ,

    //======== P0 ===================
    P0BusLockRQ     ,
    P0BusLockAK     ,
    P0DataCRQ       ,

    //======== P1 ===================
    P1BusLockRQ     ,
    P1BusLockAK     ,
    P1DataCRQ
);
    //======== SYSTEM ========================
    input           Clk             ; // Clock
    input           RstN            ; // Reset (L-Active)

    //======== P0 ============================
    input           P0BusLockRQ     ; // Bus Lock Reqest
    output          P0BusLockAK     ; // Bus Lock ACK
    input           P0DataCRQ       ; // DATA BUS REQ

    //======== P1 ============================
    input           P1BusLockRQ     ; // Bus Lock Reqest
    output          P1BusLockAK     ; // Bus Lock ACK
    input           P1DataCRQ       ; // DATA BUS REQ
    //============================================================
    //======== wire & reg ========================================
    //============================================================
    reg             P0BusLockAK     ; // Bus Lock ACK
    reg             P1BusLockAK     ; // Bus Lock ACK

    //============================================================
    //======== BODY       ========================================
    //============================================================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            P0BusLockAK <= 1'b0;
            P1BusLockAK <= 1'b0;
        end
        else if({P1BusLockAK, P0BusLockAK} == 2'b00) begin
            if({P1DataCRQ, P0DataCRQ} == 2'b00) begin
                if     (P0BusLockRQ) P0BusLockAK <= 1'b1;
                else if(P1BusLockRQ) P1BusLockAK <= 1'b1;
            end
        end
        else if(P0BusLockAK) begin
            if(~P0BusLockRQ) begin
                P0BusLockAK <= 1'b0;
                P1BusLockAK <= 1'b0;
            end
        end
        else if(P1BusLockAK) begin
            if(~P1BusLockRQ) begin
                P0BusLockAK <= 1'b0;
                P1BusLockAK <= 1'b0;
            end
        end
    end
endmodule
