//============================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : Psm3Sys (PSM3 TEST SYSTEM)
//  FUNCTION      : Vga(video out interface) memory Read Controler
//  VERSION       : Ver0.60
//
//                      Copyright (C) 2008 by Tomoyuki Inomoto
//============================================================
//`define     VGA_FRAME_WORD  16'd38400
`define     VGA_FRAME_WORD_DIV8  13'd4800

`define     VGA_RC_IDLE         3'b000
`define     VGA_RC_BANK0RQ0     3'b001
`define     VGA_RC_BANK0WAIT1   3'b010
`define     VGA_RC_BANK0RQ1     3'b011
`define     VGA_RC_BANK1WAIT0   3'b100
`define     VGA_RC_BANK1RQ0     3'b101
`define     VGA_RC_BANK1WAIT1   3'b110
`define     VGA_RC_BANK1RQ1     3'b111

module VgaRC(
    Clk         ,
    RstN        ,

    VgaMode     ,
    VgaEna      ,
    VgaBase0    ,
    VgaBase1    ,

    VgaCRQ      ,
    VgaCAK      ,
    VgaADR      ,
    VgaRD       ,
    VgaRDAK     ,

    VgaFifoFull ,
    VgaFifoWe   ,
    VgaFifoWD
);
    //==================================================
    //======== port  ===================================
    //==================================================
    //======== SYSTEM ==============
    input           Clk         ;
    input           RstN        ;

    //======== VgaCR ==============
    input           VgaMode     ;
    input           VgaEna      ;
    input   [25:4]  VgaBase0    ;
    input   [25:4]  VgaBase1    ;

    //======== DataBus ==============
    output          VgaCRQ      ;
    input           VgaCAK      ;
    output  [25:0]  VgaADR      ;
    input   [31:0]  VgaRD       ;
    input           VgaRDAK     ;

    //======== FIFO ==============
    input           VgaFifoFull ;
    output          VgaFifoWe   ;
    output  [31:0]  VgaFifoWD   ;

    //==================================================
    //======== wire & reg ==============================
    //==================================================
    reg     [25:4]  VgaADR0     ;
    reg     [25:4]  VgaADR1     ;
    reg     [12:0]  WordCounter ;
    reg     [2:0]   StateCounter;
    reg     [3:0]   BufMemAdr   ;
    wire            BufMemWe    ;
    wire    [31:0]  BufMemRD    ;

    //====================================================================
    //======== BODY ======================================================
    //====================================================================
    assign  VgaCRQ = StateCounter[0];
    assign  VgaADR[25:4] = (StateCounter[2])? VgaADR1 : VgaADR0;
    assign  VgaADR[3:0]  = 4'h0;
    //======== Read Control State Machine ===========================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            VgaADR0     <= 22'h0;
            VgaADR1     <= 22'h0;
            WordCounter <= 13'h0;
            StateCounter<= 3'b000;
        end
        else if(StateCounter == 3'h0) begin
            if(~VgaEna) begin
                WordCounter <= 13'h0;
            end
            else if(~VgaFifoFull) begin
                StateCounter  <= StateCounter + 1'b1;
                if(WordCounter == 13'h0) begin
                    VgaADR0     <= VgaBase0;
                    VgaADR1     <= VgaBase1;
                end
            end
        end
        else if(StateCounter == 3'h1) begin
            if(VgaCAK) begin
                StateCounter  <= StateCounter + 1'b1;
                VgaADR0       <= VgaADR0 + 1'h1;
            end
        end
        else if(StateCounter == 3'h2) begin
                StateCounter  <= StateCounter + 1'b1;
        end
        else if(StateCounter == 3'h3) begin
            if(VgaCAK) begin
                StateCounter  <= StateCounter + 1'b1;
                VgaADR0       <= VgaADR0 + 1'h1;
            end
        end
        else if(StateCounter == 3'h4) begin
                StateCounter  <= StateCounter + 1'b1;
        end
        else if(StateCounter == 3'h5) begin
            if(VgaCAK) begin
                StateCounter  <= StateCounter + 1'b1;
                VgaADR1       <= VgaADR1 + 1'h1;
            end
        end
        else if(StateCounter == 3'h6) begin
                StateCounter  <= StateCounter + 1'b1;
                WordCounter   <= WordCounter + 1'b1;
        end
        else begin // (StateCounter == `VGA_RC_BANK1RQ1)
            if(VgaCAK) begin
                StateCounter  <= StateCounter + 1'b1;
                VgaADR1       <= VgaADR1 + 1'h1;
                if(WordCounter == `VGA_FRAME_WORD_DIV8) WordCounter <= 13'h0;
            end
        end
    end
    //======== BufMemAdr ===========================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)        BufMemAdr <= 4'h0;
        else if(~VgaEna) BufMemAdr <= 4'h0;
        else if(VgaRDAK) BufMemAdr <= BufMemAdr + 1'b1;
    end

    //======== BufMem ===========================
    assign  BufMemWe = ~BufMemAdr[3] & VgaRDAK;
    BufMem BufMem(
        .Clk         ( Clk              ),
        .We          ( BufMemWe         ),
        .Adr         ( BufMemAdr[2:0]   ),
        .Di          ( VgaRD            ),
        .Do          ( BufMemRD         )
    );
    //======== VgaFifoWe ===========================
    assign  VgaFifoWe = BufMemAdr[3] & VgaRDAK;
    //======== VgaFifoWD ===========================
    assign  VgaFifoWD[ 0] = (VgaRD[ 3])? ((VgaMode)? (VgaRD[ 0] ^ BufMemRD[ 0]) : VgaRD[ 0]) : BufMemRD[ 0];
    assign  VgaFifoWD[ 1] = (VgaRD[ 3])? ((VgaMode)? (VgaRD[ 1] ^ BufMemRD[ 1]) : VgaRD[ 1]) : BufMemRD[ 1];
    assign  VgaFifoWD[ 2] = (VgaRD[ 3])? ((VgaMode)? (VgaRD[ 2] ^ BufMemRD[ 2]) : VgaRD[ 2]) : BufMemRD[ 2];
    //assign    VgaFifoWD[ 3] = (VgaRD[ 3])? ((VgaMode)? (VgaRD[ 3] ^ BufMemRD[ 3]) : VgaRD[ 3]) : BufMemRD[ 3];
    assign  VgaFifoWD[ 3] = 1'b0;

    assign  VgaFifoWD[ 4] = (VgaRD[ 7])? ((VgaMode)? (VgaRD[ 4] ^ BufMemRD[ 4]) : VgaRD[ 4]) : BufMemRD[ 4];
    assign  VgaFifoWD[ 5] = (VgaRD[ 7])? ((VgaMode)? (VgaRD[ 5] ^ BufMemRD[ 5]) : VgaRD[ 5]) : BufMemRD[ 5];
    assign  VgaFifoWD[ 6] = (VgaRD[ 7])? ((VgaMode)? (VgaRD[ 6] ^ BufMemRD[ 6]) : VgaRD[ 6]) : BufMemRD[ 6];
    //assign    VgaFifoWD[ 7] = (VgaRD[ 7])? ((VgaMode)? (VgaRD[ 7] ^ BufMemRD[ 7]) : VgaRD[ 7]) : BufMemRD[ 7];
    assign  VgaFifoWD[ 7] = 1'b0;

    assign  VgaFifoWD[ 8] = (VgaRD[11])? ((VgaMode)? (VgaRD[ 8] ^ BufMemRD[ 8]) : VgaRD[ 8]) : BufMemRD[ 8];
    assign  VgaFifoWD[ 9] = (VgaRD[11])? ((VgaMode)? (VgaRD[ 9] ^ BufMemRD[ 9]) : VgaRD[ 9]) : BufMemRD[ 9];
    assign  VgaFifoWD[10] = (VgaRD[11])? ((VgaMode)? (VgaRD[10] ^ BufMemRD[10]) : VgaRD[10]) : BufMemRD[10];
    //assign    VgaFifoWD[11] = (VgaRD[11])? ((VgaMode)? (VgaRD[11] ^ BufMemRD[11]) : VgaRD[11]) : BufMemRD[11];
    assign  VgaFifoWD[11] = 1'b0;

    assign  VgaFifoWD[12] = (VgaRD[15])? ((VgaMode)? (VgaRD[12] ^ BufMemRD[12]) : VgaRD[12]) : BufMemRD[12];
    assign  VgaFifoWD[13] = (VgaRD[15])? ((VgaMode)? (VgaRD[13] ^ BufMemRD[13]) : VgaRD[13]) : BufMemRD[13];
    assign  VgaFifoWD[14] = (VgaRD[15])? ((VgaMode)? (VgaRD[14] ^ BufMemRD[14]) : VgaRD[14]) : BufMemRD[14];
    //assign    VgaFifoWD[15] = (VgaRD[15])? ((VgaMode)? (VgaRD[15] ^ BufMemRD[15]) : VgaRD[15]) : BufMemRD[15];
    assign  VgaFifoWD[15] = 1'b0;

    assign  VgaFifoWD[16] = (VgaRD[19])? ((VgaMode)? (VgaRD[16] ^ BufMemRD[16]) : VgaRD[16]) : BufMemRD[16];
    assign  VgaFifoWD[17] = (VgaRD[19])? ((VgaMode)? (VgaRD[17] ^ BufMemRD[17]) : VgaRD[17]) : BufMemRD[17];
    assign  VgaFifoWD[18] = (VgaRD[19])? ((VgaMode)? (VgaRD[18] ^ BufMemRD[18]) : VgaRD[18]) : BufMemRD[18];
    //assign    VgaFifoWD[19] = (VgaRD[19])? ((VgaMode)? (VgaRD[19] ^ BufMemRD[19]) : VgaRD[19]) : BufMemRD[19];
    assign  VgaFifoWD[19] = 1'b0;

    assign  VgaFifoWD[20] = (VgaRD[23])? ((VgaMode)? (VgaRD[20] ^ BufMemRD[20]) : VgaRD[20]) : BufMemRD[20];
    assign  VgaFifoWD[21] = (VgaRD[23])? ((VgaMode)? (VgaRD[21] ^ BufMemRD[21]) : VgaRD[21]) : BufMemRD[21];
    assign  VgaFifoWD[22] = (VgaRD[23])? ((VgaMode)? (VgaRD[22] ^ BufMemRD[22]) : VgaRD[22]) : BufMemRD[22];
    //assign    VgaFifoWD[23] = (VgaRD[23])? ((VgaMode)? (VgaRD[23] ^ BufMemRD[23]) : VgaRD[23]) : BufMemRD[23];
    assign  VgaFifoWD[23] = 1'b0;

    assign  VgaFifoWD[24] = (VgaRD[27])? ((VgaMode)? (VgaRD[24] ^ BufMemRD[24]) : VgaRD[24]) : BufMemRD[24];
    assign  VgaFifoWD[25] = (VgaRD[27])? ((VgaMode)? (VgaRD[25] ^ BufMemRD[25]) : VgaRD[25]) : BufMemRD[25];
    assign  VgaFifoWD[26] = (VgaRD[27])? ((VgaMode)? (VgaRD[26] ^ BufMemRD[26]) : VgaRD[26]) : BufMemRD[26];
    //assign    VgaFifoWD[27] = (VgaRD[27])? ((VgaMode)? (VgaRD[27] ^ BufMemRD[27]) : VgaRD[27]) : BufMemRD[27];
    assign  VgaFifoWD[27] = 1'b0;

    assign  VgaFifoWD[28] = (VgaRD[31])? ((VgaMode)? (VgaRD[28] ^ BufMemRD[28]) : VgaRD[28]) : BufMemRD[28];
    assign  VgaFifoWD[29] = (VgaRD[31])? ((VgaMode)? (VgaRD[29] ^ BufMemRD[29]) : VgaRD[29]) : BufMemRD[29];
    assign  VgaFifoWD[30] = (VgaRD[31])? ((VgaMode)? (VgaRD[30] ^ BufMemRD[30]) : VgaRD[30]) : BufMemRD[30];
    //assign    VgaFifoWD[31] = (VgaRD[31])? ((VgaMode)? (VgaRD[31] ^ BufMemRD[31]) : VgaRD[31]) : BufMemRD[31];
    assign  VgaFifoWD[31] = 1'b0;

endmodule
//============================================================
//  TARGET SYSTEM : Psm3Sys (PSM3 TEST SYSTEM)
//  FUNCTION      : Vga(video out interface) Read Data Buffer
//=============================================================
module BufMem(
    Clk         ,
    We          ,
    Adr         ,
    Di          ,
    Do
);
    //==================================================
    //======== port  ===================================
    //==================================================
    input           Clk         ;
    input           We          ;
    input   [2:0]   Adr         ;
    input   [31:0]  Di          ;
    output  [31:0]  Do          ;

    //==================================================
    //======== wire & reg ==============================
    //==================================================
    reg     [31:0]  mem[0:7]    ;
    //====================================================================
    //======== BODY ======================================================
    //====================================================================
    always  @(posedge Clk) if(We) mem[Adr] <= Di;
    assign  Do = mem[Adr];

endmodule
