﻿//=======================================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  FUNCTION      : Debugger
//
//                      Copyright (C) 2008 by Tomoyuki Inomoto
//=======================================================================
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Psm3Debugger
{

    public partial class MainForm : Form
    {
        //================================================================================================
        //=　定数宣言
        //================================================================================================
        public const uint P3DBG_OK = 0;
        public const uint P3DBG_NG = 1;
        public const uint P3DBG_ILG = 0xFFFFFFFF;
        public const uint P3DBG_BREAK_ON = 2;
        public const uint P3DBG_BREAK_OFF = 3;
        private const uint MaxNumOfBreak = 16;                // 有効なBreakの数 （とりあえず16個まで）
        private const uint MaxNumOfLabel = 1024;              // 有効なプログラムラベルの数 （とりあえず1024個まで）
        private const uint MaxNumOfGlobalVar = 1024;          // 有効なグローバル変数ラベルの数 （とりあえず1024個まで）
        private const uint MaxProgramTableSize = (1024 * 1024); // ロードモジュールのテーブルサイズ(とりあえず１M行）
        private const uint MaxMemDownLoadTableSize = (1024 * 1024); // メモリダウンロード用テーブルサイズ(とりあえず1Mワード（4Mバイト分））
        private const uint BreakHexCode = 0xF8000000;      // Breakの所に埋め込むコード
        private const uint P0PMonCntrAdr = 0x0C0000E0;
        private const uint P0PMonCouner0Adr = 0x0C0000E4;
        private const uint P0PMonCouner1Adr = 0x0C0000E8;
        private const uint P1PMonCntrAdr = 0x0C0000F0;
        private const uint P1PMonCouner0Adr = 0x0C0000F4;
        private const uint P1PMonCouner1Adr = 0x0C0000F8;
        private const uint PMonRunBit = 0x00000001;
        private const uint PMonClearBit = 0x00000002;
        //================================================================================================
        //=　GLOBAL変数宣言
        //================================================================================================
        private struct TargetPro
        {
            public uint ValidFlag;      // 有効フラグ
            //public uint BreakFlag;      // ソフトブレーク張ってれば1
            public uint Adr;            // アドレス
            public uint Hex;            // 命令のHexコード
            //public string LoadedCode;   // *.lstに含まれてるコード
            //public string IAsm; // 逆アセンブル結果
            public string Src;          // ソースコード
        };
        //====================== プログラム用テーブル関連 =============================================
        private TargetPro[] TgtPro0 = new TargetPro[MaxProgramTableSize]; // ロードモジュールのテーブル(とりあえず１M行）
        private uint SizeOfTgtPro0;                             // ↑のサイズ
        private uint[] Address2Index = new uint[MaxProgramTableSize];   // アドレスからTargetProを逆引きするための対応表
        private uint SizeOfAddress2Index;                       // ↑のサイズ
        private struct ProgramLabelTabel
        {
            public string Label;
            public string Function;
            public uint Addr;
            public uint ValidFlag;
        };
        private ProgramLabelTabel[] PrgLblTbl = new ProgramLabelTabel[MaxNumOfLabel]; // プログラムラベルのテーブル
        private uint SizeOfPrgLblTbl = 0;      // 登録されたラベル数
        private struct GlobalVarLabelTabel
        {
            public string Label;
            public uint Addr;
            public uint Size;
            public uint ValidFlag;
        };
        private GlobalVarLabelTabel[] GlobalVarLblTbl = new GlobalVarLabelTabel[MaxNumOfGlobalVar]; // グローバル変数ラベルのテーブル
        private uint SizeOfGlobalVarLblTbl = 0;      // 登録されたラベル数
        private uint LoadFileActFlag = 0;           // プログラムロード処理起動フラグ
        private uint LoadMapActFlag = 0;            // マップロード処理起動フラグ

        private struct CodeWindowValTable
        {
            public uint[] CodeWindowBreakFlagArray; //コードWindow表示用のBreakFlag
            public uint[] CodeWindowAdrArray;       //コードWindow表示用のAddress
            public uint[] CodeWindowHexArray;       //コードWindow表示用のHex
            public String[] CodeWindowSrcArray;     //コードWindow表示用のソース文字列
        };
        private CodeWindowValTable P0CodeWinTbl;
        private CodeWindowValTable P1CodeWinTbl;

        private struct BreakAddressTabel
        {
            public uint BreakAddr;
            public uint EscapeHex;
        };
        private BreakAddressTabel[] BrkAdrTbl = new BreakAddressTabel[MaxNumOfBreak]; // BREAKを張ってるアドレスのテーブル

        //====================== レジスタテーブル関連 =============================================
        private uint[] REG_P0 = new uint[19];                  // REGISTER
        private uint[] OLD_REG_P0 = new uint[19];              // REGISTER
        private uint[] REG_P1 = new uint[19];                  // REGISTER
        private uint[] OLD_REG_P1 = new uint[19];              // REGISTER
        public RegSetForm RegSet0 = new RegSetForm();
        public RegSetForm RegSet1 = new RegSetForm();

        //====================== メモリテーブル関連 =============================================
        private uint[] MEM0 = new uint[16];                  // MEM(メモリWindow用)
        private uint[] MEM1 = new uint[16];                  // MEM(メモリWindow用)
        private uint[] MEM2 = new uint[16];                  // MEM(メモリWindow用)
        private uint[] MEM3 = new uint[16];                  // MEM(メモリWindow用)
        private uint[] OLD_MEM0 = new uint[16];              // MEM(メモリWindow用)
        private uint[] OLD_MEM1 = new uint[16];              // MEM(メモリWindow用)
        private uint[] OLD_MEM2 = new uint[16];              // MEM(メモリWindow用)
        private uint[] OLD_MEM3 = new uint[16];              // MEM(メモリWindow用)
        public RegSetForm MemDatSet0 = new RegSetForm();
        public RegSetForm MemDatSet1 = new RegSetForm();
        public RegSetForm MemDatSet2 = new RegSetForm();
        public RegSetForm MemDatSet3 = new RegSetForm();
        public MemAdrSetForm MemAdrSet0 = new MemAdrSetForm();
        public MemAdrSetForm MemAdrSet1 = new MemAdrSetForm();
        public MemAdrSetForm MemAdrSet2 = new MemAdrSetForm();
        public MemAdrSetForm MemAdrSet3 = new MemAdrSetForm();
        public MsgForm MsgBox = new MsgForm();
        public MemDownLoadAdrSetForm MemDownLoadAdrSet = new MemDownLoadAdrSetForm();
        public MemUpLoadSetForm MemUpLoadSet = new MemUpLoadSetForm();
        public FillForm MemFill = new FillForm();

        //====================== 全体制御関連 =============================================
        private DbgCom DCom;
        private uint ProccesorRunFlag;
        private uint ProccesorStepFlag;
        private uint ProccesorStopFlag;
        private uint CurrentProsessorStatus;
        private uint OldProsessorStatus;
        private uint BreakStatus;
        private uint ConnectEnable;
        private uint PerformanceMonitorModeChangeFlag = 0;
        private uint PerformanceMonitorOnFlag = 0;
        private uint DEBUG_FLAG = 1;
        private uint SystemTimeCount = 0;
        private uint UpDownLoadBusyFlag = 0;
        private uint ProgramLoadCmplFlag = 0;
        private uint MemDownLoadFlag = 0;
        private uint MemDownLoadStartAdr = 0;
        private uint MemUpLoadFlag = 0;
        private uint MemUpLoadStartAdr = 0;
        private uint MemUpLoadSize = 0;
        public ErrMsgForm ErrMsgBox = new ErrMsgForm();

        //================================================================================================
        //=　MainFormコンストラクタ
        //================================================================================================
        public MainForm()
        {
            uint i;

            InitializeComponent();
            this.DCom = new DbgCom();
            DCom.ComPortNo = ComTextBox.Text;
            RegSet0.RegMode = 1;
            RegSet1.RegMode = 1;
            MemDatSet0.Text = "データ設定";
            MemDatSet1.Text = "データ設定";
            MemDatSet2.Text = "データ設定";
            MemDatSet3.Text = "データ設定";
            MemDatSet0.RegMode = 0;
            MemDatSet1.RegMode = 0;
            MemDatSet2.RegMode = 0;
            MemDatSet3.RegMode = 0;

            MemAdrSet0.RadioButton0.Checked = true;
            MemAdrSet0.RadioButton1.Checked = false;
            MemAdrSet0.RadioButton1.Enabled = false;
            MemAdrSet0.TextBox.Enabled = true;
            MemAdrSet0.ComboBox.Enabled = false;

            MemAdrSet1.RadioButton0.Checked = true;
            MemAdrSet1.RadioButton1.Checked = false;
            MemAdrSet1.RadioButton1.Enabled = false;
            MemAdrSet1.TextBox.Enabled = true;
            MemAdrSet1.ComboBox.Enabled = false;

            MemAdrSet2.RadioButton0.Checked = true;
            MemAdrSet2.RadioButton1.Checked = false;
            MemAdrSet2.RadioButton1.Enabled = false;
            MemAdrSet2.TextBox.Enabled = true;
            MemAdrSet2.ComboBox.Enabled = false;

            MemAdrSet3.RadioButton0.Checked = true;
            MemAdrSet3.RadioButton1.Checked = false;
            MemAdrSet3.RadioButton1.Enabled = false;
            MemAdrSet3.TextBox.Enabled = true;
            MemAdrSet3.ComboBox.Enabled = false;

            SizeOfTgtPro0 = 0;
            SizeOfAddress2Index = 0;

            CodeBox0.label1.Text = "P0 CODE";
            CodeBox1.label1.Text = "P1 CODE";

            RegBox0.label1.Text = "P0 REG";
            RegBox1.label1.Text = "P1 REG";

            for (i = 0; i < MaxProgramTableSize; i++) TgtPro0[i].ValidFlag = 0;

            P0CodeWinTbl.CodeWindowBreakFlagArray = new uint[20];
            P0CodeWinTbl.CodeWindowAdrArray = new uint[20];
            P0CodeWinTbl.CodeWindowHexArray = new uint[20];
            P0CodeWinTbl.CodeWindowSrcArray = new String[20];

            P1CodeWinTbl.CodeWindowBreakFlagArray = new uint[20];
            P1CodeWinTbl.CodeWindowAdrArray = new uint[20];
            P1CodeWinTbl.CodeWindowHexArray = new uint[20];
            P1CodeWinTbl.CodeWindowSrcArray = new String[20];

            CodeBox0.ProcessorNo = 0;
            CodeBox1.ProcessorNo = 1;
            
            ProccesorRunFlag = 0;
            ProccesorStepFlag = 0;
            ProccesorStopFlag = 0;

            CurrentProsessorStatus = DbgCom.PROCESSOR_STOP;
            OldProsessorStatus = DbgCom.PROCESSOR_STOP;

            ConnectEnable = 0;

            PMonComboBox.SelectedIndex = 0;

            for (i = 0; i < MaxNumOfBreak; i++)
            {
                BrkAdrTbl[i].BreakAddr = P3DBG_ILG;
            }
            ReadConfigFile();
        }
        //=　設定LOAD処理
        private void ReadConfigFile()
        {
            DialogResult Ret;
            System.IO.StreamReader FD;
            string ReadBuf;
            string[] ReadBufArray = new string[] { };

            //=========== ファイル名取得 ====================
            Ret = OpenCfgDialog.ShowDialog();
            if (Ret == System.Windows.Forms.DialogResult.OK)
            {
                //=========== ファイル存在チェック ====================
                if (System.IO.File.Exists(OpenCfgDialog.FileName) == true)
                {
                    FD = new System.IO.StreamReader(OpenCfgDialog.FileName, System.Text.Encoding.Default);
                    //=========== ファイルの終端まで１行ずつに読み込んでテーブルへ格納 ====================
                    while (FD.Peek() != -1)
                    {
                        ReadBuf = FD.ReadLine();
                        ReadBufArray = ReadBuf.Split(':');
                        //if (ReadBufArray[0].Contains("PORT") == true) ComTextBox.Text = ReadBufArray[1];
                        if (ReadBufArray[0].CompareTo("PORT") == 0) ComTextBox.Text = ReadBufArray[1];
                        if (ReadBufArray[0].CompareTo("MEM_BOX0_TOP_ADDRESS") == 0) MemBox0.TopAddress = Str2Num(ReadBufArray[1]);
                        if (ReadBufArray[0].CompareTo("MEM_BOX1_TOP_ADDRESS") == 0) MemBox1.TopAddress = Str2Num(ReadBufArray[1]);
                        if (ReadBufArray[0].CompareTo("MEM_BOX2_TOP_ADDRESS") == 0) MemBox2.TopAddress = Str2Num(ReadBufArray[1]);
                        if (ReadBufArray[0].CompareTo("MEM_BOX3_TOP_ADDRESS") == 0) MemBox3.TopAddress = Str2Num(ReadBufArray[1]);
                    }
                }
            }
        }

        //================================================================================================
        //=　終了処理
        //================================================================================================
        private void EndFileMenu_Click(object sender, EventArgs e)
        {
            this.Close();
        }
        //================================================================================================
        //=　MAP LOAD処理 (ユーザー操作受付) =============================================================
        //================================================================================================
        private void ReadMapFile_Click(object sender, EventArgs e)
        {
            LoadMapActFlag = 1;
        }
        private void LoadMapFileBody()
        {
            if (LoadMapActFlag != 0)
            {
                LoadMapActFlag = 0;
                ReadMapFromFile();
                //PrintPrgLblTbl();
                SetLblList();
            }
        }
        private uint ReadMapFromFile()
        {
            DialogResult Ret;
            System.IO.StreamReader FD;
            string ReadBuf;
            string[] ReadBufArray = new string[] { };
            uint i;
            uint Flag;

            //=========== ファイル名取得 ====================
            Ret = OpenMapDialog.ShowDialog();
            if (Ret == System.Windows.Forms.DialogResult.OK)
            {
                //=========== ファイル存在チェック ====================
                if (System.IO.File.Exists(OpenMapDialog.FileName) == true)
                {
                    FD = new System.IO.StreamReader(OpenMapDialog.FileName, System.Text.Encoding.Default);
                    //=========== テーブルのValidフラグを全クリア ====================
                    for (i = 0; i < MaxNumOfLabel; i++) PrgLblTbl[i].ValidFlag = 0;
                    //=========== ファイルの終端まで１行ずつに読み込んでテーブルへ格納 ====================
                    i = 0;
                    Flag = 0;
                    SizeOfPrgLblTbl = 0;
                    while (FD.Peek() != -1)
                    {
                        ReadBuf = FD.ReadLine();
                        if (ReadBuf.Contains("___BEGIN_PROGRAM_LABELS___") == true)
                        {
                            Flag = 1;
                            continue;
                        }
                        if (ReadBuf.Contains("___END_PROGRAM_LABELS___") == true)
                        {
                            Flag = 0;
                            break;
                        }
                        if (Flag == 1)
                        {
                            ReadBufArray = ReadBuf.Split(':');
                            if (ReadBufArray[3].Contains("Function Entry Point") == true)
                            {
                                PrgLblTbl[i].Label = ReadBufArray[0].Trim();
                                PrgLblTbl[i].Addr = Str2Num(ReadBufArray[1].Trim());
                                PrgLblTbl[i].Function = ReadBufArray[2].Trim();
                                PrgLblTbl[i].ValidFlag = 1;
                                ++i;
                                ++SizeOfPrgLblTbl;
                                if (i >= (MaxNumOfLabel))
                                {
                                    MessageBox.Show("読み込み可能な行数を超えています", "エラー");
                                    break;
                                }
                            }
                        }
                    }
                    FD = new System.IO.StreamReader(OpenMapDialog.FileName, System.Text.Encoding.Default);
                    //=========== テーブルのValidフラグを全クリア ====================
                    for (i = 0; i < MaxNumOfGlobalVar; i++) GlobalVarLblTbl[i].ValidFlag = 0;
                    //=========== ファイルの終端まで１行ずつに読み込んでテーブルへ格納 ====================
                    i = 0;
                    Flag = 0;
                    SizeOfGlobalVarLblTbl = 0;
                    while (FD.Peek() != -1)
                    {
                        ReadBuf = FD.ReadLine();
                        if (ReadBuf.Contains("___BEGIN_GLOBAL_VARIABLES___") == true)
                        {
                            Flag = 1;
                            continue;
                        }
                        if (ReadBuf.Contains("___END_GLOBAL_VARIABLES___") == true)
                        {
                            Flag = 0;
                            break;
                        }
                        if (Flag == 1)
                        {
                            ReadBufArray = ReadBuf.Split(':');
                            GlobalVarLblTbl[i].Label = ReadBufArray[3].Trim();
                            GlobalVarLblTbl[i].Addr = Str2Num(ReadBufArray[0].Trim());
                            GlobalVarLblTbl[i].Size = Str2Num(ReadBufArray[6].Trim());
                            GlobalVarLblTbl[i].ValidFlag = 1;
                            ++i;
                            ++SizeOfGlobalVarLblTbl;
                            if (i >= (MaxNumOfGlobalVar))
                            {
                                MessageBox.Show("読み込み可能な行数を超えています", "エラー");
                                break;
                            }
                        }
                    }
                }
            }
            return P3DBG_OK;
        }
        private void SetLblList()
        {
            uint i;
            if (SizeOfPrgLblTbl > 0)
            {
                CodeBox0.LblListComboBox.Items.Clear();
                CodeBox1.LblListComboBox.Items.Clear();
                for (i = 0; i < SizeOfPrgLblTbl; i++)
                {
                    CodeBox0.LblListComboBox.Items.Add(String.Format("0x{0:X8}", PrgLblTbl[i].Addr) + " : " + PrgLblTbl[i].Label);
                    CodeBox1.LblListComboBox.Items.Add(String.Format("0x{0:X8}", PrgLblTbl[i].Addr) + " : " + PrgLblTbl[i].Label);
                }
                CodeBox0.LblListComboBox.SelectedIndex = 0;
                CodeBox1.LblListComboBox.SelectedIndex = 0;
                CodeBox0.JumpBtn.Enabled = true;
                CodeBox1.JumpBtn.Enabled = true;
            }

            if (SizeOfGlobalVarLblTbl > 0)
            {
                MemAdrSet0.ComboBox.Items.Clear();
                MemAdrSet1.ComboBox.Items.Clear();
                MemAdrSet2.ComboBox.Items.Clear();
                MemAdrSet3.ComboBox.Items.Clear();
                for (i = 0; i < SizeOfGlobalVarLblTbl; i++)
                {
                    MemAdrSet0.ComboBox.Items.Add(String.Format("0x{0:X8}", GlobalVarLblTbl[i].Addr) + " : " + GlobalVarLblTbl[i].Label);
                    MemAdrSet1.ComboBox.Items.Add(String.Format("0x{0:X8}", GlobalVarLblTbl[i].Addr) + " : " + GlobalVarLblTbl[i].Label);
                    MemAdrSet2.ComboBox.Items.Add(String.Format("0x{0:X8}", GlobalVarLblTbl[i].Addr) + " : " + GlobalVarLblTbl[i].Label);
                    MemAdrSet3.ComboBox.Items.Add(String.Format("0x{0:X8}", GlobalVarLblTbl[i].Addr) + " : " + GlobalVarLblTbl[i].Label);
                }
                MemAdrSet0.ComboBox.SelectedIndex = 0;
                MemAdrSet1.ComboBox.SelectedIndex = 0;
                MemAdrSet2.ComboBox.SelectedIndex = 0;
                MemAdrSet3.ComboBox.SelectedIndex = 0;
                MemAdrSet0.ComboBoxEnableFlag = 1;
                MemAdrSet1.ComboBoxEnableFlag = 1;
                MemAdrSet2.ComboBoxEnableFlag = 1;
                MemAdrSet3.ComboBoxEnableFlag = 1;
                MemAdrSet0.RadioButton1.Enabled = true;
                MemAdrSet1.RadioButton1.Enabled = true;
                MemAdrSet2.RadioButton1.Enabled = true;
                MemAdrSet3.RadioButton1.Enabled = true;
            }
        }
        private void PrintPrgLblTbl()
        {
            uint i;
            string Str;
            for (i = 0; i < SizeOfPrgLblTbl; i++)
            {
                if (DEBUG_FLAG != 0) PrintMsgTextBox((PrgLblTbl[i].Label + " " + PrgLblTbl[i].Function + " " + String.Format("0x{0:X8}", PrgLblTbl[i].Addr)) + Environment.NewLine, 1);
            }
            for (i = 0; i < SizeOfGlobalVarLblTbl; i++)
            {
                Str = GlobalVarLblTbl[i].Size.ToString();
                if (DEBUG_FLAG != 0) PrintMsgTextBox((GlobalVarLblTbl[i].Label + " " + String.Format("0x{0:X8}", GlobalVarLblTbl[i].Addr) + " " + Str) + Environment.NewLine, 1);
            }
        }
        //========================================================================================================================
        //=　プログラムLOAD処理 (ユーザー操作受付) ===============================================================================
        //========================================================================================================================
        private void LoadFileMenu_Click(object sender, EventArgs e)
        {
            LoadFileActFlag = 1;
        }
        //=　プログラムLOAD処理（本体）
        private uint LoadFileBody()
        {
            uint Ret;

            if (LoadFileActFlag != 0)
            {
                LoadFileActFlag = 0;
                // ファイルの内容をテーブルに格納
                Ret = ReadListFile();
                if (Ret == P3DBG_NG) return P3DBG_NG;
                // プロセッサ停止
                DCom.DbgWrite(DbgCom.CntrReg, 0x3);
                // ターゲットにHexをロード
                PrintMsgBoxLabel("ロード中  0 バイト");
                MsgBox.Show();
                UpDownLoadBusyFlag = 1;
                //Ret = LoadHex2Target();
                backgroundWorker2.RunWorkerAsync();
                System.Threading.Thread.Sleep(100);
            }
            if(ProgramLoadCmplFlag == 1)
            {
                ProgramLoadCmplFlag = 0;
                UpDownLoadBusyFlag = 0;
                MsgBox.Hide();
                // コードWindowのスクロールバーのサイズを設定
                CodeBox0.vScrollBar1.Maximum = (int)SizeOfTgtPro0;
                CodeBox1.vScrollBar1.Maximum = (int)SizeOfTgtPro0;
                // 登録済みのブレークポイントを（とりあえず再設定）
                BrkAllSet();

                // プロセッサリセット＆レジスタクリア
                DCom.Reset();
                DCom.CoreRegWrite(DbgCom.PID0, 0, 0); //--- R0@P0 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID0, 1, 0); //--- R1@P0 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID0, 2, 0); //--- R2@P0 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID0, 3, 0); //--- R3@P0 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID0, 4, 0); //--- R4@P0 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID0, 5, 0); //--- R5@P0 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID0, 6, 0); //--- R6@P0 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID0, 7, 0); //--- R7@P0 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID0, 8, 0); //--- R8@P0 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID0, 9, 0); //--- R9@P0 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID0, 10, 0); //--- R10@P0 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID0, 11, 0); //--- R11@P0 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID0, 12, 0); //--- R12@P0 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID0, 13, 0); //--- R13@P0 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID0, 14, 0); //--- SR@P0 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID0, 15, 0); //--- PC@P0 <= 0x00000000

                DCom.CoreRegWrite(DbgCom.PID1, 0, 0); //--- R0@P1 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID1, 1, 0); //--- R1@P1 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID1, 2, 0); //--- R2@P1 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID1, 3, 0); //--- R3@P1 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID1, 4, 0); //--- R4@P1 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID1, 5, 0); //--- R5@P1 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID1, 6, 0); //--- R6@P1 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID1, 7, 0); //--- R7@P1 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID1, 8, 0); //--- R8@P1 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID1, 9, 0); //--- R9@P1 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID1, 10, 0); //--- R10@P1 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID1, 11, 0); //--- R11@P1 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID1, 12, 0); //--- R12@P1 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID1, 13, 0); //--- R13@P1 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID1, 14, 0); //--- SR@P1 <= 0x00000000
                DCom.CoreRegWrite(DbgCom.PID1, 15, 0); //--- PC@P1 <= 0x00000000

                // GUI更新
                BreakStatus = DbgCom.NOT_BREAK_STAT;
                DCom.CoreRegReadAll(DbgCom.PID0, ref REG_P0); //--- レジスタ値再読み込み
                DCom.CoreRegReadAll(DbgCom.PID1, ref REG_P1); //--- レジスタ値再読み込み
                RegBox0.RegValueSet2GUI(ref REG_P0, ref OLD_REG_P0, 1);
                RegBox1.RegValueSet2GUI(ref REG_P1, ref OLD_REG_P1, 1);

                if (MemBox0.CheckBox0.Checked == true)
                {
                    DCom.MemBlockRead3(MemBox0.TopAddress, 16, ref MEM0);//--- メモリ値再読み込み
                    MemBox0.MemValueSet2GUI(ref MEM0, ref OLD_MEM0);
                }
                if (MemBox1.CheckBox0.Checked == true)
                {
                    DCom.MemBlockRead3(MemBox1.TopAddress, 16, ref MEM1);//--- メモリ値再読み込み
                    MemBox1.MemValueSet2GUI(ref MEM1, ref OLD_MEM1);
                }
                if (MemBox2.CheckBox0.Checked == true)
                {
                    DCom.MemBlockRead3(MemBox2.TopAddress, 16, ref MEM2);//--- メモリ値再読み込み
                    MemBox2.MemValueSet2GUI(ref MEM2, ref OLD_MEM2);
                }
                if (MemBox3.CheckBox0.Checked == true)
                {
                    DCom.MemBlockRead3(MemBox3.TopAddress, 16, ref MEM3);//--- メモリ値再読み込み
                    MemBox3.MemValueSet2GUI(ref MEM3, ref OLD_MEM3);
                }
                RenwalCodeBoxByAdr(ref CodeBox0, 0, 0, 0, ref P0CodeWinTbl);// CodeBox0更新
                RenwalCodeBoxByAdr(ref CodeBox1, 0, 0, 0, ref P0CodeWinTbl);// CodeBox1更新
                CodeBox0.vScrollBar1.Value = (int)CodeBox0.TopIndex;
                CodeBox1.vScrollBar1.Value = (int)CodeBox1.TopIndex;
            }
            return P3DBG_OK;
        }
        //=　プログラムLOAD処理（テーブルの内容をターゲットへ書込み）※基本的に256ワード単位で書き込む
        private uint LoadHex2Target()
        {
            uint i,j,k;
            uint LoadedBytes;
            uint NumOf256WordTransmission; // 256ワード単位転送回数
            uint NumOfLastTransmission; // 最終（端数）転送ワード数
            uint[] Buf = new uint[256];

            NumOf256WordTransmission = SizeOfAddress2Index / 256;
            NumOfLastTransmission = SizeOfAddress2Index % 256;
            k = 0;
            //======== 256ワード単位の転送処理 ================
            for (i = 0; i < NumOf256WordTransmission; i++)
            {
                for (j = 0; j < 256; j++)
                {
                    Buf[j] = TgtPro0[Address2Index[k]].Hex;
                    ++k;
                }
                while (DCom.MemBlockWrite2((256 * i * 4), 256, ref Buf) != P3DBG_OK) ;
                LoadedBytes = ((i + 1) * 256 * 4)/1024;
                PrintMsgBoxLabel("ロード中  " + LoadedBytes.ToString() + "Kバイト");
            }
            //======== 最終（端数）の転送処理 ================
            for (j = 0; j < NumOfLastTransmission; j++)
            {
                Buf[j] = TgtPro0[Address2Index[k]].Hex;
                ++k;
            }
            while (DCom.MemBlockWrite2((256 * NumOf256WordTransmission * 4), NumOfLastTransmission, ref Buf) != P3DBG_OK) ;
            LoadedBytes = (((NumOf256WordTransmission * 256) + NumOfLastTransmission) * 4)/1024;
            PrintMsgBoxLabel("ロード中  " + LoadedBytes.ToString() + "Kバイト");
            return P3DBG_OK;
        }
        private void backgroundWorker2_DoWork(object sender, DoWorkEventArgs e)
        {
            LoadHex2Target();
        }
        private void backgroundWorker2_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ProgramLoadCmplFlag = 1;
        }
        //=　プログラムLOAD処理（ファイルの内容をテーブルに格納）
        private uint ReadListFile()
        {
            DialogResult Ret;
            System.IO.StreamReader FD;
            string ReadBuf;
            string[] ReadBufArray = new string[] { };
            uint i;

            //=========== ファイル名取得 ====================
            Ret = OpenFileDialog.ShowDialog();
            if (Ret == System.Windows.Forms.DialogResult.OK)
            {
                //=========== ファイル存在チェック ====================
                if (System.IO.File.Exists(OpenFileDialog.FileName) == true)
                {
                    PrintMsgTextBox(OpenFileDialog.FileName + "をロード" + Environment.NewLine, 0);
                    FD = new System.IO.StreamReader(OpenFileDialog.FileName, System.Text.Encoding.Default);
                    //=========== テーブルのValidフラグを全クリア ====================
                    for (i = 0; i < MaxProgramTableSize; i++)
                    {
                        TgtPro0[i].ValidFlag = 0;
                    }
                    //=========== ファイルの終端まで１行ずつに読み込んでテーブルへ格納 ====================
                    i = 0;
                    SizeOfTgtPro0 = 0;
                    SizeOfAddress2Index = 0;
                    while (FD.Peek() != -1)
                    {
                        ReadBuf = FD.ReadLine();
                        ReadBufArray = ReadBuf.Split(':');
                        //TgtPro0[i].BreakFlag = 0;
                        TgtPro0[i].Adr = Str2Num(ReadBufArray[0]);
                        TgtPro0[i].Hex = Str2Num(ReadBufArray[1]);
                        TgtPro0[i].Src = ReadBufArray[3];
                        if (TgtPro0[i].Adr != P3DBG_ILG)
                        {
                            Address2Index[TgtPro0[i].Adr >> 2] = i;
                            ++SizeOfAddress2Index;
                        }
                        TgtPro0[i].ValidFlag = 1;
                        ++i;
                        ++SizeOfTgtPro0;
                        if (i >= (MaxProgramTableSize))
                        {
                            MessageBox.Show("読み込み可能な行数を超えています", "エラー");
                            break;
                        }
                    }
                    FD.Close();
                    return P3DBG_OK;
                }
                else
                {
                    return P3DBG_NG;
                }
            }
            else
            {
                return P3DBG_NG;
            }

        }
        //========================================================================================================================
        //=　CodeBox関連処理 =====================================================================================================
        //========================================================================================================================
        //=　Breakを張ってるアドレスか否かを検査
        private uint IsBreakAddress(uint Addr)
        {
            uint i;
            if (Addr == P3DBG_ILG) return 0;
            for (i = 0; i < MaxNumOfBreak; i++)
            {
                if (BrkAdrTbl[i].BreakAddr == Addr) return 1;
            }
            return 0;
        }
        //=　CodeBox表示用テーブルを作成(通常モード)
        private void MakeCodeBoxValueTableNomal(uint Index, ref CodeWindowValTable CTB)
        {
            uint i;
            for (i = 0; i < 20; i++)
            {
                if (TgtPro0[Index + i].ValidFlag != 0)
                {
                    CTB.CodeWindowBreakFlagArray[i] = IsBreakAddress(TgtPro0[Index + i].Adr);
                    CTB.CodeWindowAdrArray[i] = TgtPro0[Index + i].Adr;
                    CTB.CodeWindowHexArray[i] = TgtPro0[Index + i].Hex;
                    CTB.CodeWindowSrcArray[i] = TgtPro0[Index + i].Src;
                }
                else
                {
                    CTB.CodeWindowBreakFlagArray[i] = 0;
                    CTB.CodeWindowAdrArray[i] = P3DBG_ILG;
                    CTB.CodeWindowHexArray[i] = P3DBG_ILG;
                    CTB.CodeWindowSrcArray[i] = "";
                }
            }
        }
        //=　CodeBox表示用テーブルを作成(逆アセモード)
        private void MakeCodeBoxValueTableIAsm(uint Adr, ref CodeWindowValTable CTB)
        {
            uint i;
            uint[] MemForIAsm = new uint[20];
            P3IAsm IAsm = new P3IAsm();

            DCom.MemBlockRead(Adr, 20, ref MemForIAsm);
            for (i = 0; i < 20; i++)
            {
                CTB.CodeWindowBreakFlagArray[i] = IsBreakAddress(Adr + i * 4);
                CTB.CodeWindowAdrArray[i] = (Adr + i * 4);
                CTB.CodeWindowHexArray[i] = MemForIAsm[i];
                IAsm.IAsm((Adr + i * 4), MemForIAsm[i], ref CTB.CodeWindowSrcArray[i]);
            }
        }
        //==== アドレスからIndexを逆引き
        private uint Adr2CodeTabeleIndex(uint Adr)
        {
            if ((Adr >> 2) >= SizeOfAddress2Index)
            {
                return P3DBG_ILG;
            }
            else
            {
                return Address2Index[(Adr >> 2)];
            }
        }
        //=　CodeBox更新 (アドレスベース)
        private void RenwalCodeBoxByAdr(ref CodeBox CBox, uint Adr, uint PC, uint Bubble, ref CodeWindowValTable CTB)
        {
            uint Ret;
            uint BaseIndex;
            uint BaseAdr;

            //==== アドレスからIndexを逆引き ===========
            Ret = Adr2CodeTabeleIndex(Adr);
            if (Ret != P3DBG_ILG)//==== 逆引き成功 ===========
            {
                if (Ret <= 10)
                {
                    BaseIndex = 0;
                    CBox.TopIndex = Ret;
                    CBox.TopIndexOffset = Ret;
                }
                else
                {
                    BaseIndex = Ret - 10;
                    CBox.TopIndex = Ret;
                    CBox.TopIndexOffset = 10;
                }
                CBox.Mode = CodeBox.SRC_MODE;
                MakeCodeBoxValueTableNomal(BaseIndex, ref CTB);
                CBox.UpDateCodeWindow(PC, Bubble, ref CTB.CodeWindowBreakFlagArray, ref CTB.CodeWindowAdrArray, ref CTB.CodeWindowHexArray, ref CTB.CodeWindowSrcArray);
            }
            else//==== 逆引き失敗 ===========
            {
                if ((Adr >> 2) <= 10)
                {
                    BaseAdr = 0;
                    CBox.TopAddress = Adr;
                    CBox.TopAddressOffset = Adr;
                }
                else
                {
                    BaseAdr = Adr - 4 * 10;
                    CBox.TopAddress = Adr;
                    CBox.TopAddressOffset = 4 * 10;
                }
                CBox.Mode = CodeBox.IASM_MODE;
                MakeCodeBoxValueTableIAsm(BaseAdr, ref CTB);
                CBox.UpDateCodeWindow(PC, Bubble, ref CTB.CodeWindowBreakFlagArray, ref CTB.CodeWindowAdrArray, ref CTB.CodeWindowHexArray, ref CTB.CodeWindowSrcArray);
            }

        }
        //=　CodeBox更新 (Indexベース)
        private void RenwalCodeBoxByIndex(ref CodeBox CBox, uint Index, uint PC, uint Bubble, ref CodeWindowValTable CTB)
        {
            uint BaseIndex;

            if (Index <= 10)
            {
                BaseIndex = 0;
                CBox.TopIndex = Index;
                CBox.TopIndexOffset = Index;
            }
            else
            {
                BaseIndex = Index - 10;
                CBox.TopIndex = Index;
                CBox.TopIndexOffset = 10;
            }
            CBox.Mode = CodeBox.SRC_MODE;
            MakeCodeBoxValueTableNomal(BaseIndex, ref CTB);
            CBox.UpDateCodeWindow(PC, Bubble, ref CTB.CodeWindowBreakFlagArray, ref CTB.CodeWindowAdrArray, ref CTB.CodeWindowHexArray, ref CTB.CodeWindowSrcArray);
        }
        //=　CodeBox処理 (ユーザー操作)   ================================================
        private void CodeBox_ActionCheckAndExe(ref CodeBox CBox)
        {
            //=============== スクロール処理 ====================
            if (CBox.ScrollFlag != 0)
            {
                CodeBoxScrollAction(ref CBox);
                CBox.ScrollFlag = 0;
            }
            //=============== Break処理 ====================
            if (CBox.BreakChangeFlag != 0)
            {
                CodeBoxBreakAction(ref CBox);
                CBox.BreakChangeFlag = 0;
            }
            //=============== Break処理 ====================
            if (CBox.JumpFlag != 0)
            {
                CodeBoxJumpAction(ref CBox);
                CBox.JumpFlag = 0;
            }
        }
        //=　CodeBox処理 (スクロール)   
        private void CodeBoxScrollAction(ref CodeBox CBox)
        {
            uint Index;
            uint Adr;
            uint PC;
            uint Bubble;

            if (CBox.ProcessorNo == 0)
            {
                Bubble = DCom.IsBubble(0);
                if ((BreakStatus & DbgCom.P0BreakBit) != 0) PC = REG_P0[15];
                else PC = REG_P0[16];
            }
            else
            {
                Bubble = DCom.IsBubble(1);
                if ((BreakStatus & DbgCom.P1BreakBit) != 0) PC = REG_P1[15];
                else PC = REG_P1[16];
            }

            if (CBox.Mode == CodeBox.SRC_MODE) //==== SRC_MODEのとき
            {
                if (CBox.ScrollUpDwn == CodeBox.SCROLL_UP)
                {
                    Index = CBox.TopIndex + CBox.ScrollValue;
                    if (CBox.ProcessorNo == 0) RenwalCodeBoxByIndex(ref CBox, Index, PC, Bubble, ref P0CodeWinTbl);
                    else RenwalCodeBoxByIndex(ref CBox, Index, PC, Bubble, ref P1CodeWinTbl);
                }
                else
                {
                    if (CBox.TopIndex < CBox.ScrollValue) Index = 0;
                    else Index = CBox.TopIndex - CBox.ScrollValue;
                    if (CBox.ProcessorNo == 0) RenwalCodeBoxByIndex(ref CBox, Index, PC, Bubble, ref P0CodeWinTbl);
                    else RenwalCodeBoxByIndex(ref CBox, Index, PC, Bubble, ref P1CodeWinTbl);
                }
            }
            else//==== IASM_MODEのとき
            {
                if (CBox.ScrollUpDwn == CodeBox.SCROLL_UP)
                {
                    Adr = CBox.TopAddress + CBox.ScrollValue * 4;
                    if (CBox.ProcessorNo == 0) RenwalCodeBoxByAdr(ref CBox, Adr, PC, Bubble, ref P0CodeWinTbl);
                    else RenwalCodeBoxByAdr(ref CBox, Adr, PC, Bubble, ref P1CodeWinTbl);
                }
                else
                {
                    if (CBox.TopAddress < CBox.ScrollValue * 4) Adr = 0;
                    else Adr = CBox.TopAddress - (CBox.ScrollValue * 4);
                    if (CBox.ProcessorNo == 0) RenwalCodeBoxByAdr(ref CBox, Adr, PC, Bubble, ref P0CodeWinTbl);
                    else RenwalCodeBoxByAdr(ref CBox, Adr, PC, Bubble, ref P1CodeWinTbl);
                }
            }
        }
        //=　CodeBox処理[ JUMP ] =======================================
        private void CodeBoxJumpAction(ref CodeBox CBox)
        {
            uint Index;
            uint Adr;
            uint PC;
            uint Bubble;

            if (CBox.ProcessorNo == 0)
            {
                Bubble = DCom.IsBubble(0);
                if ((BreakStatus & DbgCom.P0BreakBit) != 0) PC = REG_P0[15];
                else PC = REG_P0[16];
            }
            else
            {
                Bubble = DCom.IsBubble(1);
                if ((BreakStatus & DbgCom.P1BreakBit) != 0) PC = REG_P1[15];
                else PC = REG_P1[16];
            }

            Adr = PrgLblTbl[CBox.LblListComboBox.SelectedIndex].Addr;
            if (CBox.Mode == CodeBox.SRC_MODE) //==== SRC_MODEのとき
            {
                Index = Adr2CodeTabeleIndex(Adr);
                CBox.vScrollBar1.Value = (int)Index;
                //if (CBox.ProcessorNo == 0) RenwalCodeBoxByAdr(ref CBox, Adr, PC, Bubble, ref P0CodeWinTbl);
                //else RenwalCodeBoxByAdr(ref CBox, Adr, PC, Bubble, ref P1CodeWinTbl);
            }
            else
            {
                CBox.vScrollBar1.Value = (int)(Adr >> 2);
            }
        }
        //=　CodeBox処理 (ブレーク)   
        private void CodeBoxBreakAction(ref CodeBox CBox)
        {
            uint Index;
            uint Addr;
            uint PC;
            uint Bubble;
            uint Ret;

            if (CBox.Mode == CodeBox.SRC_MODE) //==== SRC_MODEのとき
            {
                Index = CBox.TopIndex + CBox.BreakChangeIndex - CBox.TopIndexOffset;
                Addr = GetAddrFromIndex(Index);
            }
            else//==== IASM_MODEのとき
            {
                Addr = CBox.TopAddress + (CBox.BreakChangeIndex * 4) - CBox.TopAddressOffset;
            }
            if (Addr != P3DBG_ILG)
            {
                Ret = BrkAdrTblAddOrSub(Addr);
            }
            else
            {
                Ret = P3DBG_ILG;
            }

            if (CBox.ProcessorNo == 0)
            {
                Bubble = DCom.IsBubble(0);
                if ((BreakStatus & DbgCom.P0BreakBit) != 0)
                {
                    PC = REG_P0[15];
                }
                else
                {
                    PC = REG_P0[16];
                }
            }
            else
            {
                Bubble = DCom.IsBubble(1);
                if ((BreakStatus & DbgCom.P1BreakBit) != 0)
                {
                    PC = REG_P1[15];
                }
                else
                {
                    PC = REG_P1[16];
                }
            }
            CodeBox0.FoceScroll();
            CodeBox1.FoceScroll();
        }
        //=　CodeBox処理 (IndexからBreak対象アドレスを取得)   
        private uint GetAddrFromIndex(uint Index)
        {
            if (TgtPro0[Index].ValidFlag == 0) return P3DBG_ILG;
            return TgtPro0[Index].Adr;
        }
        //================================================================================================
        //=　BREAK関連処理 ===============================================================================
        //================================================================================================
        //=　CodeBox処理 (Breakアドレスのテーブルに登録済みなら登録抹消、未登録で登録化なら登録)   
        private uint BrkAdrTblAddOrSub(uint Addr)
        {
            uint i;
            uint Hex;

            Hex = 0;
            //==== Breakアドレスのテーブルに登録済みなら登録抹消 ====
            for (i = 0; i < MaxNumOfBreak; i++)
            {
                if (BrkAdrTbl[i].BreakAddr == Addr)
                {
                    BrkAdrTbl[i].BreakAddr = P3DBG_ILG;
                    DCom.MemWrite(Addr, BrkAdrTbl[i].EscapeHex);
                    return P3DBG_BREAK_OFF;
                }
            }
            //=== 未登録で登録可なら登録 ========
            for (i = 0; i < MaxNumOfBreak; i++)
            {
                if (BrkAdrTbl[i].BreakAddr == P3DBG_ILG)
                {
                    BrkAdrTbl[i].BreakAddr = Addr;
                    DCom.MemRead(Addr, ref Hex);
                    BrkAdrTbl[i].EscapeHex = Hex;
                    DCom.MemWrite(Addr, BreakHexCode);
                    return P3DBG_BREAK_ON;
                }
            }
            MessageBox.Show("設定可能なBreakの数を超えました", "エラー");
            return P3DBG_NG;
        }
        //=　すべての登録済みBreakを設定
        private uint BrkAllSet()
        {
            uint i;
            uint Addr;
            uint Hex;

            Hex = 0;
            //=== 登録可なら登録 ========
            for (i = 0; i < MaxNumOfBreak; i++)
            {
                if (BrkAdrTbl[i].BreakAddr != P3DBG_ILG)
                {
                    Addr = BrkAdrTbl[i].BreakAddr;
                    DCom.MemRead(Addr, ref Hex);
                    BrkAdrTbl[i].EscapeHex = Hex;
                    DCom.MemWrite(Addr, BreakHexCode);
                }
            }
            return P3DBG_OK;
        }
        //=　すべての登録済みBreakを削除
        private uint BrkAllClear()
        {
            uint i;
            uint Addr;

            //==== Breakアドレスのテーブルに登録済みなら登録抹消 ====
            for (i = 0; i < MaxNumOfBreak; i++)
            {
                if (BrkAdrTbl[i].BreakAddr != P3DBG_ILG)
                {
                    Addr = BrkAdrTbl[i].BreakAddr;
                    BrkAdrTbl[i].BreakAddr = P3DBG_ILG;
                    DCom.MemWrite(Addr, BrkAdrTbl[i].EscapeHex);
                    return P3DBG_BREAK_OFF;
                }
            }
            return P3DBG_OK;
        }
        //================================================================================================
        //=　REGBOX 関連  ================================================================================
        //================================================================================================
        //=　REGBOX0 
        private void RegBox0_ActionCheckAndExe()
        {
            if (RegBox0.RefreshFlag != 0) //--- 「更新」ボタン押下時の処理
            {
                DCom.CoreRegReadAll(DbgCom.PID0, ref REG_P0); //--- レジスタ値再読み込み
                if ((BreakStatus & DbgCom.P0BreakBit) != 0) RegBox0.RegValueSet2GUI(ref REG_P0, ref OLD_REG_P0, 0);
                else RegBox0.RegValueSet2GUI(ref REG_P0, ref OLD_REG_P0, 1);
                RegBox0.RefreshFlag = 0;
            }
            if (RegBox0.RegSetFlag != 0) //--- レジスタ値（DoubleClick）時の処理
            {
                if (RegBox0.RegNo == 0) RegSet0.RegLabel.Text = RegBox0.RegLabel0.Text;
                else if (RegBox0.RegNo == 1) RegSet0.RegLabel.Text = RegBox0.RegLabel1.Text;
                else if (RegBox0.RegNo == 2) RegSet0.RegLabel.Text = RegBox0.RegLabel2.Text;
                else if (RegBox0.RegNo == 3) RegSet0.RegLabel.Text = RegBox0.RegLabel3.Text;
                else if (RegBox0.RegNo == 4) RegSet0.RegLabel.Text = RegBox0.RegLabel4.Text;
                else if (RegBox0.RegNo == 5) RegSet0.RegLabel.Text = RegBox0.RegLabel5.Text;
                else if (RegBox0.RegNo == 6) RegSet0.RegLabel.Text = RegBox0.RegLabel6.Text;
                else if (RegBox0.RegNo == 7) RegSet0.RegLabel.Text = RegBox0.RegLabel7.Text;
                else if (RegBox0.RegNo == 8) RegSet0.RegLabel.Text = RegBox0.RegLabel8.Text;
                else if (RegBox0.RegNo == 9) RegSet0.RegLabel.Text = RegBox0.RegLabel9.Text;
                else if (RegBox0.RegNo == 10) RegSet0.RegLabel.Text = RegBox0.RegLabel10.Text;
                else if (RegBox0.RegNo == 11) RegSet0.RegLabel.Text = RegBox0.RegLabel11.Text;
                else if (RegBox0.RegNo == 12) RegSet0.RegLabel.Text = RegBox0.RegLabel12.Text;
                else if (RegBox0.RegNo == 13) RegSet0.RegLabel.Text = RegBox0.RegLabel13.Text;
                else if (RegBox0.RegNo == 14) RegSet0.RegLabel.Text = RegBox0.RegLabel14.Text;
                else RegSet0.RegLabel.Text = RegBox0.RegLabel15.Text;
                RegSet0.TextBox.Text = String.Format("0x{0:X8}", REG_P0[RegBox0.RegNo]);
                RegSet0.RegNo = RegBox0.RegNo;
                RegSet0.Location = MousePosition;
                RegSet0.Show();
                RegBox0.RegSetFlag = 0;
            }
            //=============== RegSet 関連 ============================================
            if (RegSet0.ChangeFlag != 0)
            {
                DCom.CoreRegWrite(DbgCom.PID0, RegSet0.RegNo, RegSet0.RegVal); //--- レジスタ値設定
                DCom.CoreRegReadAll(DbgCom.PID0, ref REG_P0); //--- レジスタ値再読み込み
                if (RegSet0.RegNo == 15) RegBox0.RegValueSet2GUI(ref REG_P0, ref OLD_REG_P0, 0);
                else if ((BreakStatus & DbgCom.P0BreakBit) != 0) RegBox0.RegValueSet2GUI(ref REG_P0, ref OLD_REG_P0, 0);
                else RegBox0.RegValueSet2GUI(ref REG_P0, ref OLD_REG_P0, 1);
                RegSet0.ChangeFlag = 0;
            }
        }
        //=　REGBOX1
        private void RegBox1_ActionCheckAndExe()
        {
            if (RegBox1.RefreshFlag != 0) //--- 「更新」ボタン押下時の処理
            {
                DCom.CoreRegReadAll(DbgCom.PID1, ref REG_P1); //--- レジスタ値再読み込み
                if ((BreakStatus & DbgCom.P1BreakBit) != 0) RegBox1.RegValueSet2GUI(ref REG_P1, ref OLD_REG_P1, 0);
                else RegBox1.RegValueSet2GUI(ref REG_P1, ref OLD_REG_P1, 1);
                RegBox1.RefreshFlag = 0;
            }
            if (RegBox1.RegSetFlag != 0) //--- レジスタ値（DoubleClick）時の処理
            {
                if (RegBox1.RegNo == 0) RegSet1.RegLabel.Text = RegBox1.RegLabel0.Text;
                else if (RegBox1.RegNo == 1) RegSet1.RegLabel.Text = RegBox1.RegLabel1.Text;
                else if (RegBox1.RegNo == 2) RegSet1.RegLabel.Text = RegBox1.RegLabel2.Text;
                else if (RegBox1.RegNo == 3) RegSet1.RegLabel.Text = RegBox1.RegLabel3.Text;
                else if (RegBox1.RegNo == 4) RegSet1.RegLabel.Text = RegBox1.RegLabel4.Text;
                else if (RegBox1.RegNo == 5) RegSet1.RegLabel.Text = RegBox1.RegLabel5.Text;
                else if (RegBox1.RegNo == 6) RegSet1.RegLabel.Text = RegBox1.RegLabel6.Text;
                else if (RegBox1.RegNo == 7) RegSet1.RegLabel.Text = RegBox1.RegLabel7.Text;
                else if (RegBox1.RegNo == 8) RegSet1.RegLabel.Text = RegBox1.RegLabel8.Text;
                else if (RegBox1.RegNo == 9) RegSet1.RegLabel.Text = RegBox1.RegLabel9.Text;
                else if (RegBox1.RegNo == 10) RegSet1.RegLabel.Text = RegBox1.RegLabel10.Text;
                else if (RegBox1.RegNo == 11) RegSet1.RegLabel.Text = RegBox1.RegLabel11.Text;
                else if (RegBox1.RegNo == 12) RegSet1.RegLabel.Text = RegBox1.RegLabel12.Text;
                else if (RegBox1.RegNo == 13) RegSet1.RegLabel.Text = RegBox1.RegLabel13.Text;
                else if (RegBox1.RegNo == 14) RegSet1.RegLabel.Text = RegBox1.RegLabel14.Text;
                else RegSet1.RegLabel.Text = RegBox1.RegLabel15.Text;
                RegSet1.TextBox.Text = String.Format("0x{0:X8}", REG_P1[RegBox1.RegNo]);
                RegSet1.RegNo = RegBox1.RegNo;
                RegSet1.Location = MousePosition;
                RegSet1.Show();
                RegBox1.RegSetFlag = 0;
            }
            //=============== RegSet 関連 ============================================
            if (RegSet1.ChangeFlag != 0)
            {
                DCom.CoreRegWrite(DbgCom.PID1, RegSet1.RegNo, RegSet1.RegVal); //--- レジスタ値設定
                DCom.CoreRegReadAll(DbgCom.PID1, ref REG_P1); //--- レジスタ値再読み込み
                if (RegSet0.RegNo == 15) RegBox1.RegValueSet2GUI(ref REG_P1, ref OLD_REG_P1, 0);
                else if ((BreakStatus & DbgCom.P1BreakBit) != 0) RegBox1.RegValueSet2GUI(ref REG_P1, ref OLD_REG_P1, 0);
                else RegBox1.RegValueSet2GUI(ref REG_P1, ref OLD_REG_P1, 1);
                RegSet1.ChangeFlag = 0;
            }
        }
        //================================================================================================
        //=　MEMBOX0 関連 ================================================================================
        //================================================================================================
        private void MemBox_ActionCheckAndExe(uint No)
        {
            MemBox MBox;
            MemAdrSetForm AdrSet;
            RegSetForm DatSet;
            uint[] MEM;
            uint[] OLD_MEM;

            MBox = MemBox0;
            AdrSet = MemAdrSet0;
            DatSet = MemDatSet0;
            MEM = MEM0;
            OLD_MEM = OLD_MEM0;
            if (No == 1)
            {
                MBox = MemBox1;
                AdrSet = MemAdrSet1;
                DatSet = MemDatSet1;
                MEM = MEM1;
                OLD_MEM = OLD_MEM1;
            }
            else if (No == 2)
            {
                MBox = MemBox2;
                AdrSet = MemAdrSet2;
                DatSet = MemDatSet2;
                MEM = MEM2;
                OLD_MEM = OLD_MEM2;
            }
            else if(No == 3)
            {
                MBox = MemBox3;
                AdrSet = MemAdrSet3;
                DatSet = MemDatSet3;
                MEM = MEM3;
                OLD_MEM = OLD_MEM3;
            }
            //=============== 「更新」ボタン押下時の処理 ================================
            if (MBox.RefreshFlag != 0)
            {
                //DCom.MemRead16(MBox.TopAddress, ref MEM); //--- メモリ値再読み込み
                DCom.MemBlockRead3(MBox.TopAddress, 16, ref MEM);//--- メモリ値再読み込み
                MBox.MemValueSet2GUI(ref MEM, ref OLD_MEM);
                MBox.RefreshFlag = 0;
            }
            //=============== AdrSet 関連 ============================================
            if (MBox.AddressUpDataFlag != 0)
            {
                AdrSet.TextBox.Text = String.Format("0x{0:X8}", MBox.TopAddress);
                AdrSet.Location = MousePosition;
                AdrSet.Show();
                MBox.AddressUpDataFlag = 0;
            }
            if (AdrSet.ChangeFlag != 0)
            {
                if (AdrSet.RadioButton0.Checked == true)
                {
                    MBox.TopAddress = Str2Num(AdrSet.TextBox.Text) & 0xFFFFFFFC;
                }
                else
                {
                    MBox.TopAddress = GlobalVarLblTbl[AdrSet.ComboBox.SelectedIndex].Addr & 0xFFFFFFFC;
                }
                //DCom.MemRead16(MBox.TopAddress, ref MEM); //--- メモリ値再読み込み
                DCom.MemBlockRead3(MBox.TopAddress, 16, ref MEM);//--- メモリ値再読み込み
                MBox.MemValueSet2GUI(ref MEM, ref OLD_MEM);
                AdrSet.ChangeFlag = 0;
            }
            //=============== DatSet 関連 ============================================
            if (MBox.DataUpDataFlag != 0)
            {
                DatSet.RegLabel.Text = String.Format("0x{0:X8}", MBox.TopAddress + (MBox.DataNo * 4));
                DatSet.TextBox.Text = String.Format("0x{0:X8}", MEM[MBox.DataNo]);
                DatSet.RegNo = MBox.DataNo;
                DatSet.Location = MousePosition;
                DatSet.Show();
                MBox.DataUpDataFlag = 0;
            }
            if (DatSet.ChangeFlag != 0)
            {
                DCom.MemWrite(MBox.TopAddress + (DatSet.RegNo * 4), DatSet.RegVal);
                //DCom.MemRead16(MBox.TopAddress, ref MEM); //--- メモリ値再読み込み
                DCom.MemBlockRead3(MBox.TopAddress, 16, ref MEM);//--- メモリ値再読み込み
                MBox.MemValueSet2GUI(ref MEM, ref OLD_MEM);
                DatSet.ChangeFlag = 0;
            }
            //=============== スクロールバー 関連 ============================================
            if (MBox.ScrollFlag != 0)
            {
                //DCom.MemRead16(MBox.TopAddress, ref MEM); //--- メモリ値再読み込み
                DCom.MemBlockRead3(MBox.TopAddress, 16, ref MEM);//--- メモリ値再読み込み
                MBox.MemValueSet2GUI(ref MEM, ref OLD_MEM);
                MBox.ScrollFlag = 0;
            }

        }
        //================================================================================================
        //=　実行制御ボタン関連     ======================================================================
        //================================================================================================
        //=　実行制御ボタン[RUN] 
        private void RunBtn0_Click(object sender, EventArgs e)
        {
            ProccesorRunFlag = 1;
        }
        private void RunBtn0_ActionCheckAndExe()
        {
            uint CpuStatus0;
            uint CpuStatus1;
            uint CpuStatus;
            uint P0_PC;
            uint P1_PC;
            uint Flag0;
            uint Flag1;

            CpuStatus0 = 0;
            CpuStatus1 = 0;
            CpuStatus = 0;
            P0_PC = 0;
            P1_PC = 0;
            Flag0 = 0;
            Flag1 = 0;

            if (ProccesorRunFlag != 0)
            {
                ProccesorRunFlag = 0;
                //-- プロセッサ状態を取得 ---
                CpuStatus0 = DCom.GetProsessorStatus(DbgCom.P0);
                CpuStatus1 = DCom.GetProsessorStatus(DbgCom.P1);
                CpuStatus = CpuStatus0 & CpuStatus1;
                //-- STOPならRUNさせる
                if (CpuStatus == DbgCom.PROCESSOR_STOP)
                {
                    DCom.CashOffForce();
                    if ((BreakStatus & DbgCom.P0BreakBit) != 0)
                    {
                        DCom.CoreRegRead(DbgCom.PID0, 15, ref P0_PC);
                        if (IsBreakAddress(P0_PC) != 0)
                        {
                            BrkAdrTblAddOrSub(P0_PC);//Break登録抹消
                            Flag0 = 1;
                        }
                        else
                        {
                            if (DEBUG_FLAG != 0) PrintMsgTextBox("CPU0_STATUS == BREAK but PC != BREAK ADDRESS" + Environment.NewLine, 1);
                        }
                        DCom.CoreRegWrite(DbgCom.PID0, 15, P0_PC);
                    }
                    if ((BreakStatus & DbgCom.P1BreakBit) != 0)
                    {
                        DCom.CoreRegRead(DbgCom.PID1, 15, ref P1_PC);
                        if (IsBreakAddress(P1_PC) != 0)
                        {
                            BrkAdrTblAddOrSub(P1_PC);//Break登録抹消
                            Flag1 = 1;
                        }
                        else
                        {
                            if (DEBUG_FLAG != 0) PrintMsgTextBox("CPU1_STATUS == BREAK but PC != BREAK ADDRESS" + Environment.NewLine, 1);
                        }
                        DCom.CoreRegWrite(DbgCom.PID1, 15, P1_PC);
                    }
                    DCom.Step(1);
                    DCom.StepModeOff();
                    if (Flag0 != 0) BrkAdrTblAddOrSub(P0_PC);//Break再登録
                    if (Flag1 != 0) BrkAdrTblAddOrSub(P1_PC);//Break再登録
                    DCom.CashOnForce();
                    DCom.Run(1, 1);
                    OldProsessorStatus = DbgCom.PROCESSOR_RUN;
                    BreakStatus = DbgCom.NOT_BREAK_STAT;
                }
            }
        }
        //=　実行制御ボタン[STEP]
        private void StepBtn0_Click(object sender, EventArgs e)
        {
            ProccesorStepFlag = 1;
        }
        private void StepBtn0_ActionCheckAndExe()
        {
            uint CpuStatus0;
            uint CpuStatus1;
            uint CpuStatus;
            uint P0_PC;
            uint P1_PC;
            uint Flag0;
            uint Flag1;

            CpuStatus0 = 0;
            CpuStatus1 = 0;
            CpuStatus = 0;
            P0_PC = 0;
            P1_PC = 0;
            Flag0 = 0;
            Flag1 = 0;

            if (ProccesorStepFlag != 0)
            {
                ProccesorStepFlag = 0;
                //-- プロセッサ状態を取得 ---
                CpuStatus0 = DCom.GetProsessorStatus(DbgCom.P0);
                CpuStatus1 = DCom.GetProsessorStatus(DbgCom.P1);
                CpuStatus = CpuStatus0 & CpuStatus1;

                //-- STOPならSTEPさせる
                if (CpuStatus == DbgCom.PROCESSOR_STOP)
                {
                    DCom.CashOffForce();
                    if ((BreakStatus & DbgCom.P0BreakBit) != 0)
                    {
                        P0_PC = REG_P0[15];
                        if (IsBreakAddress(P0_PC) != 0)
                        {
                            BrkAdrTblAddOrSub(P0_PC);//Break登録抹消
                            Flag0 = 1;
                        }
                        else
                        {
                            if (DEBUG_FLAG != 0) PrintMsgTextBox("CPU0_STATUS == BREAK but PC != BREAK ADDRESS" + Environment.NewLine, 1);
                        }
                        DCom.CoreRegWrite(DbgCom.PID0, 15, P0_PC);
                    }
                    if ((BreakStatus & DbgCom.P1BreakBit) != 0)
                    {
                        P1_PC = REG_P1[15];
                        if (IsBreakAddress(P1_PC) != 0)
                        {
                            BrkAdrTblAddOrSub(P1_PC);//Break登録抹消
                            Flag1 = 1;
                        }
                        else
                        {
                            if (DEBUG_FLAG != 0) PrintMsgTextBox("CPU1_STATUS == BREAK but PC != BREAK ADDRESS" + Environment.NewLine, 1);
                        }
                        DCom.CoreRegWrite(DbgCom.PID1, 15, P1_PC);
                    }
                    DCom.Step(1);
                    DCom.StepModeOff();
                    if (Flag0 != 0) BrkAdrTblAddOrSub(P0_PC);//Break再登録
                    if (Flag1 != 0) BrkAdrTblAddOrSub(P1_PC);//Break再登録
                    OldProsessorStatus = DbgCom.PROCESSOR_RUN;
                    BreakStatus = DbgCom.NOT_BREAK_STAT;
                }
            }
        }
        //=　実行制御ボタン[STOP]
        private void StopBtn0_Click(object sender, EventArgs e)
        {
            ProccesorStopFlag = 1;
        }
        private void StopBtn0_ActionCheckAndExe()
        {
            if (ProccesorStopFlag != 0)
            {
                ProccesorStopFlag = 0;
                DCom.Stop(1, 1);
            }
        }
        //================================================================================================
        //=　メモリフィル 関連       =====================================================================
        //================================================================================================
        private void FillBtn_Click(object sender, EventArgs e)
        {
            MemFill.Show();
        }
        private void MemFill_ActionCheckAndExe()
        {
            uint StartAdr;
            uint Size;
            uint Data;
            uint i;
            uint NumOf256WordTransmission; // 256ワード単位転送回数
            uint NumOfLastTransmission; // 最終（端数）転送ワード数
            uint[] Buf = new uint[256];

            StartAdr = 0;
            Size = 0;
            Data = 0;
            if (MemFill.ChangeFlag != 0)
            {
                MemFill.ChangeFlag = 0;
                StartAdr = Str2Num(MemFill.StartAdrTextBox.Text);
                Size = Str2Num(MemFill.SizeTextBox.Text);
                Data = Str2Num(MemFill.DataTextBox.Text);

                for (i = 0; i < 256; i++) Buf[i] = Data;
                NumOf256WordTransmission = Size / 256;
                NumOfLastTransmission = Size % 256;
                //======== 256ワード単位の転送処理 ================
                for (i = 0; i < NumOf256WordTransmission; i++)
                {
                    while (DCom.MemBlockWrite2(StartAdr + (256 * i * 4), 256, ref Buf) != P3DBG_OK) ;
                }
                //======== 最終（端数）の転送処理 ================
                while (DCom.MemBlockWrite2(StartAdr + (256 * NumOf256WordTransmission * 4), NumOfLastTransmission, ref Buf) != P3DBG_OK) ;
                MemFill.RunBtn.Enabled = true;
                MemFill.Hide();
            }

        }
        //================================================================================================
        //=　メモリダウンロード 関連       =====================================================================
        //================================================================================================
        private void DownLoadBtn_Click(object sender, EventArgs e)
        {
            MemDownLoadFlag = 1;
        }
        private void MemDownLoad_ActionCheckAndExe()
        {
            if (MemDownLoadFlag != 0)
            {
                MemDownLoadFlag = 0;
                MemDownLoadAdrSet.Show();
            }
            if (MemDownLoadAdrSet.AdrSetOkFlag != 0)
            {
                MemDownLoadAdrSet.AdrSetOkFlag = 0;
                MemDownLoadStartAdr = Str2Num(MemDownLoadAdrSet.AdrTextBox.Text);
                MemDownLoad_Body();
            }

        }
        private void MemDownLoad_Body()
        {
            DialogResult Ret;
            System.IO.StreamReader FD;
            string ReadBuf;
            uint[] HexTbl = new uint[MaxMemDownLoadTableSize];
            uint DownLoadSize;
            uint i, j, k;
            //uint LoadedBytes;
            uint NumOf256WordTransmission; // 256ワード単位転送回数
            uint NumOfLastTransmission;    // 最終（端数）転送ワード数
            uint[] Buf = new uint[256];

            //ファイル読込み
            //=========== ファイル名取得 ====================
            Ret = OpenHexDialog.ShowDialog();
            DownLoadSize = 0;
            if (Ret == System.Windows.Forms.DialogResult.OK)
            {
                //=========== ファイル存在チェック ====================
                if (System.IO.File.Exists(OpenHexDialog.FileName) == true)
                {
                    PrintMsgTextBox(OpenHexDialog.FileName + "をロード" + Environment.NewLine, 0);
                    FD = new System.IO.StreamReader(OpenHexDialog.FileName, System.Text.Encoding.Default);
                    //=========== ファイルの終端まで１行ずつに読み込んでテーブルへ格納 ====================
                    i = 0;
                    while (FD.Peek() != -1)
                    {
                        ReadBuf = FD.ReadLine();
                        HexTbl[i] = Str2Num(ReadBuf);
                        ++i;
                        if (i >= MaxMemDownLoadTableSize)
                        {
                            MessageBox.Show("読み込み可能な行数を超えています", "エラー");
                            break;
                        }
                    }
                    FD.Close();
                    DownLoadSize = i;
                }
            }
            //データ書き込み
            if (DownLoadSize > 0)
            {
                NumOf256WordTransmission = DownLoadSize / 256;
                NumOfLastTransmission = DownLoadSize % 256;
                k = 0;
                //======== 256ワード単位の転送処理 ================
                for (i = 0; i < NumOf256WordTransmission; i++)
                {
                    for (j = 0; j < 256; j++) Buf[j] = HexTbl[k++];
                    while (DCom.MemBlockWrite2(MemDownLoadStartAdr + (256 * i * 4), 256, ref Buf) != P3DBG_OK) ;
                    //LoadedBytes = ((i + 1) * 256 * 4) / 1024;
                    //PrintMsgBoxLabel("ロード中  " + LoadedBytes.ToString() + "Kバイト");
                }
                //======== 最終（端数）の転送処理 ================
                for (j = 0; j < NumOfLastTransmission; j++) Buf[j] = HexTbl[k++];
                while (DCom.MemBlockWrite2(MemDownLoadStartAdr + (256 * NumOf256WordTransmission * 4), NumOfLastTransmission, ref Buf) != P3DBG_OK) ;
                //LoadedBytes = (((NumOf256WordTransmission * 256) + NumOfLastTransmission) * 4) / 1024;
                //PrintMsgBoxLabel("ロード中  " + LoadedBytes.ToString() + "Kバイト");
            }
        }
        //================================================================================================
        //=　メモリアップロード 関連       =====================================================================
        //================================================================================================
        private void UpLoadBtn_Click(object sender, EventArgs e)
        {
            MemUpLoadFlag = 1;
        }
        private void MemUpLoad_ActionCheckAndExe()
        {
            if (MemUpLoadFlag != 0)
            {
                MemUpLoadFlag = 0;
                MemUpLoadSet.Show();
            }
            if (MemUpLoadSet.AdrAndSizeSetOkFlag != 0)
            {
                MemUpLoadSet.AdrAndSizeSetOkFlag = 0;
                MemUpLoadStartAdr = Str2Num(MemUpLoadSet.AdrTextBox.Text);
                MemUpLoadSize = Str2Num(MemUpLoadSet.SizeTextBox.Text);
                MemUpLoad_Body();
            }
        }
        private void MemUpLoad_Body()
        {
            DialogResult Ret;
            System.IO.StreamWriter FD;
            uint i, j;
            uint NumOf256WordTransmission; // 256ワード単位転送回数
            uint NumOfLastTransmission;    // 最終（端数）転送ワード数
            uint[] Buf = new uint[256];
            string Str;

            //ファイル読込み
            //=========== ファイル名取得 ====================
            Ret = SaveHexDialog.ShowDialog();
            if (Ret == System.Windows.Forms.DialogResult.OK)
            {
                PrintMsgTextBox(MemUpLoadStartAdr.GetTypeCode() + "から" + MemUpLoadSize.ToString() + "ワードを" +SaveHexDialog.FileName + "へアップロード" + Environment.NewLine, 0);
                FD =new System.IO.StreamWriter(new System.IO.FileStream(SaveHexDialog.FileName, System.IO.FileMode.Create));
                //=========== ファイルの終端まで１行ずつに読み込んでテーブルへ格納 ====================
                //データ読み込み
                NumOf256WordTransmission = MemUpLoadSize / 256;
                NumOfLastTransmission = MemUpLoadSize % 256;
                //======== 256ワード単位の転送処理 ================
                for (i = 0; i < NumOf256WordTransmission; i++)
                {
                    while (DCom.MemBlockRead3(MemUpLoadStartAdr + (256 * i * 4), 256, ref Buf) != P3DBG_OK) ;
                    for (j = 0; j < 256; j++)
                    {
                        Str = String.Format("0x{0:X8}", Buf[j]);
                        FD.WriteLine(Str);
                    }
                    //UpLoadedBytes = ((i + 1) * 256 * 4) / 1024;
                    //PrintMsgBoxLabel("アップロード中  " + UpLoadedBytes.ToString() + "Kバイト");
                }
                //======== 最終（端数）の転送処理 ================
                while (DCom.MemBlockRead3(MemUpLoadStartAdr + (256 * NumOf256WordTransmission * 4), NumOfLastTransmission, ref Buf) != P3DBG_OK) ;
                for (j = 0; j < NumOfLastTransmission; j++)
                {
                    Str = String.Format("0x{0:X8}", Buf[j]);
                    FD.WriteLine(Str);
                    //PrintMsgTextBox(Str + Environment.NewLine, 1);
                }
                //UpLoadedBytes = (((NumOf256WordTransmission * 256) + NumOfLastTransmission) * 4) / 1024;
                //PrintMsgBoxLabel("アップロード中  " + UpLoadedBytes.ToString() + "Kバイト");
                FD.Close();
            }
        }
        //================================================================================================
        //=　プロセッサ状態監視処理 ======================================================================
        //================================================================================================
        private void ProcessorStatusCheckAndExe()
        {
            uint Flag0;
            uint Flag1;
            //uint Bubble0;
            //uint Bubble1;
            uint Index;

            Flag0 = 1;
            Flag1 = 1;
            BreakStatus = DCom.GetBreakStatus();
            if (BreakStatus != DbgCom.NOT_BREAK_STAT) DCom.Stop(1, 1);
            if ((BreakStatus & DbgCom.P0BreakBit) != 0) Flag0 = 0;
            if ((BreakStatus & DbgCom.P1BreakBit) != 0) Flag1 = 0;

            CurrentProsessorStatus = DCom.GetProsessorStatus(DbgCom.P0);
            CurrentProsessorStatus &= DCom.GetProsessorStatus(DbgCom.P1);
            if (CurrentProsessorStatus == DbgCom.PROCESSOR_STOP)
            {
                DCom.StepModeOff();
            }

            if ((OldProsessorStatus == DbgCom.PROCESSOR_RUN) && (CurrentProsessorStatus == DbgCom.PROCESSOR_STOP))
            {
                DCom.CoreRegReadAll(DbgCom.PID0, ref REG_P0); //--- レジスタ値再読み込み
                RegBox0.RegValueSet2GUI(ref REG_P0, ref OLD_REG_P0, Flag0);
                DCom.CoreRegReadAll(DbgCom.PID1, ref REG_P1); //--- レジスタ値再読み込み
                RegBox1.RegValueSet2GUI(ref REG_P1, ref OLD_REG_P1, Flag1);

                if (MemBox0.CheckBox0.Checked == true)
                {
                    DCom.MemBlockRead3(MemBox0.TopAddress, 16, ref MEM0);//--- メモリ値再読み込み
                    MemBox0.MemValueSet2GUI(ref MEM0, ref OLD_MEM0);
                }
                if (MemBox1.CheckBox0.Checked == true)
                {
                    DCom.MemBlockRead3(MemBox1.TopAddress, 16, ref MEM1);//--- メモリ値再読み込み
                    MemBox1.MemValueSet2GUI(ref MEM1, ref OLD_MEM1);
                }
                if (MemBox2.CheckBox0.Checked == true)
                {
                    DCom.MemBlockRead3(MemBox2.TopAddress, 16, ref MEM2);//--- メモリ値再読み込み
                    MemBox2.MemValueSet2GUI(ref MEM2, ref OLD_MEM2);
                }
                if (MemBox3.CheckBox0.Checked == true)
                {
                    DCom.MemBlockRead3(MemBox3.TopAddress, 16, ref MEM3);//--- メモリ値再読み込み
                    MemBox3.MemValueSet2GUI(ref MEM3, ref OLD_MEM3);
                }
                if (Flag0 == 0)
                {
                    if (CodeBox0.Mode == CodeBox.SRC_MODE) Index = Adr2CodeTabeleIndex(REG_P0[15]);
                    else Index = REG_P0[15] >> 2;
                }
                else
                {
                    if (CodeBox0.Mode == CodeBox.SRC_MODE) Index = Adr2CodeTabeleIndex(REG_P0[16]);
                    else Index = REG_P0[16] >> 2;
                }
                if (CodeBox0.vScrollBar1.Value == (int)Index) CodeBox0.FoceScroll();
                else                                          CodeBox0.vScrollBar1.Value = (int)Index;
                //Bubble1 = DCom.IsBubble(1);
                //if (Flag0 == 0) RenwalCodeBoxByAdr(ref CodeBox0, REG_P0[15], REG_P0[15], Bubble0, ref P0CodeWinTbl);// CodeBox0更新
                //else RenwalCodeBoxByAdr(ref CodeBox0, REG_P0[16], REG_P0[16], Bubble0, ref P0CodeWinTbl);// CodeBox0更新

                if (Flag1 == 0)
                {
                    if (CodeBox1.Mode == CodeBox.SRC_MODE) Index = Adr2CodeTabeleIndex(REG_P1[15]);
                    else Index = REG_P1[15] >> 2;
                }
                else
                {
                    if (CodeBox1.Mode == CodeBox.SRC_MODE) Index = Adr2CodeTabeleIndex(REG_P1[16]);
                    else Index = REG_P1[16] >> 2;
                }
                if (CodeBox1.vScrollBar1.Value == (int)Index) CodeBox1.FoceScroll();
                else                                          CodeBox1.vScrollBar1.Value = (int)Index;
                //Bubble0 = DCom.IsBubble(0);
                //if (Flag1 == 0) RenwalCodeBoxByAdr(ref CodeBox1, REG_P1[15], REG_P1[15], Bubble1, ref P1CodeWinTbl);// CodeBox1更新
                //else RenwalCodeBoxByAdr(ref CodeBox1, REG_P1[16], REG_P1[16], Bubble1, ref P1CodeWinTbl);// CodeBox1更新
            }
            OldProsessorStatus = CurrentProsessorStatus;
        }
        //================================================================================================
        //=　性能モニタ 関連==============================================================================
        //================================================================================================
        private void PMonComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            PerformanceMonitorModeChangeFlag = 1;
            if (PMonComboBox.SelectedIndex == 0)
            {
                C0TitleTextBox.Text = "総サイクル数";
                C1TitleTextBox.Text = "総命令実行数";
                ResTitleTextBox.Text = "CPI";
            }
            else if (PMonComboBox.SelectedIndex == 1)
            {
                C0TitleTextBox.Text = "総リクエスト数";
                C1TitleTextBox.Text = "総ヒット数";
                ResTitleTextBox.Text = "命令＄ Hit率";
            }
            else if (PMonComboBox.SelectedIndex == 2)
            {
                C0TitleTextBox.Text = "総リクエスト数";
                C1TitleTextBox.Text = "総ヒット数";
                ResTitleTextBox.Text = "データ＄ Hit率";
            }
            else
            {
                C0TitleTextBox.Text = "総リクエスト数";
                C1TitleTextBox.Text = "WAITサイクル数";
                ResTitleTextBox.Text = "平均レイテンシ";
            }
        }
        private void PMonChkBox_CheckedChanged(object sender, EventArgs e)
        {
            if (PMonChkBox.Checked == true) PMonTimer.Enabled = true;
            else PMonTimer.Enabled = false;
        }
        private void PMonTimer_Tick(object sender, EventArgs e)
        {
            PerformanceMonitorOnFlag = 1;
            PMonTimer.Enabled = false;
        }
        private string GetPMonRes(uint C0, uint C1, int Type)
        {
            string Ret;
            float Tmp;
            if (Type == 0) // CPI
            {
                if (C1 == 0)
                {
                    Ret = "XXXX";
                }
                else
                {
                    Tmp = (float)C0 / (float)C1;
                    Tmp = (float)Decimal.Round((Decimal)Tmp, 2);
                    Ret = Tmp.ToString();
                }
            }
            else // CPI以外
            {
                if (C0 == 0)
                {
                    Ret = "XXXX";
                }
                else
                {
                    Tmp = (float)C1 / (float)C0;
                    Tmp = (float)Decimal.Round((Decimal)Tmp, 2);
                    Ret = Tmp.ToString();
                }
            }
            return Ret;
        }
        private void PerformanceMonitor()
        {
            uint PMonCntrData;
            uint PMonCounter0;
            uint PMonCounter1;

            PMonCntrData = 0;
            PMonCounter0 = 0;
            PMonCounter1 = 0;
            //====== 設定変更 ========================================================
            if (PerformanceMonitorModeChangeFlag == 1)
            {
                PerformanceMonitorModeChangeFlag = 0;
                DCom.MemWrite(P0PMonCntrAdr, PMonClearBit); //カウンタクリア
                DCom.MemWrite(P1PMonCntrAdr, PMonClearBit); //カウンタクリア
                PMonCntrData = (uint)PMonComboBox.SelectedIndex << 4;
                PMonCntrData |= PMonRunBit;
                DCom.MemWrite(P0PMonCntrAdr, PMonCntrData); //設定反映＆カウンタRUN
                DCom.MemWrite(P1PMonCntrAdr, PMonCntrData); //設定反映＆カウンタRUN
            }
            //====== モニタ値読出し ========================================================
            if (PerformanceMonitorOnFlag == 1)
            {
                PerformanceMonitorOnFlag = 0;
                PMonCntrData = (uint)PMonComboBox.SelectedIndex << 4;
                PMonCntrData &= ~PMonRunBit;
                DCom.MemWrite(P0PMonCntrAdr, PMonCntrData);//カウンタSTOP
                DCom.MemWrite(P1PMonCntrAdr, PMonCntrData);//カウンタSTOP
                DCom.MemRead(P0PMonCouner0Adr, ref PMonCounter0); //P0カウンタ0読出し
                DCom.MemRead(P0PMonCouner1Adr, ref PMonCounter1); //P0カウンタ1読出し
                P0C0ValTextBox.Text = PMonCounter0.ToString();
                P0C1ValTextBox.Text = PMonCounter1.ToString();
                P0ResValTextBox.Text = GetPMonRes(PMonCounter0, PMonCounter1, PMonComboBox.SelectedIndex);
                DCom.MemRead(P1PMonCouner0Adr, ref PMonCounter0); //P1カウンタ0読出し
                DCom.MemRead(P1PMonCouner1Adr, ref PMonCounter1); //P1カウンタ1読出し
                P1C0ValTextBox.Text = PMonCounter0.ToString();
                P1C1ValTextBox.Text = PMonCounter1.ToString();
                P1ResValTextBox.Text = GetPMonRes(PMonCounter0, PMonCounter1, PMonComboBox.SelectedIndex);
                DCom.MemWrite(P0PMonCntrAdr, PMonClearBit); //カウンタクリア
                DCom.MemWrite(P1PMonCntrAdr, PMonClearBit); //カウンタクリア
                PMonCntrData |= PMonRunBit;
                DCom.MemWrite(P0PMonCntrAdr, PMonCntrData); //カウンタRUN
                DCom.MemWrite(P1PMonCntrAdr, PMonCntrData); //カウンタRUN
                PMonTimer.Enabled = true;
            }
        }
        //================================================================================================
        //=　システム監視タイマ ==========================================================================
        //=  基本的に、ターゲットに対するすべてのアクションは、このなかで実行　===========================
        //=  (ややこしい排他制御をしたくないから)　                            ===========================
        //================================================================================================
        private void SystemMonitorTimer_Tick(object sender, EventArgs e)
        {
            SystemMonitorTimer.Enabled = false;
            if (ConnectEnable != 0)
            {
                if (UpDownLoadBusyFlag == 0)
                {
                    //=　プロセッサ状態監視処理 ==================================
                    ProcessorStatusCheckAndExe();

                    //=== REGBOX0 関連イベント処理 =================================
                    RegBox0_ActionCheckAndExe();
                    //=== REGBOX1 関連イベント処理 =================================
                    RegBox1_ActionCheckAndExe();
                    //=== MEMBOX0 関連イベント処理 =================================
                    MemBox_ActionCheckAndExe(0);
                    MemBox_ActionCheckAndExe(1);
                    MemBox_ActionCheckAndExe(2);
                    MemBox_ActionCheckAndExe(3);
                    //=== MAPファイルのロード =====================================
                    LoadMapFileBody();
                    //=== CodeBox0 関連イベント処理 ================================
                    CodeBox_ActionCheckAndExe(ref CodeBox0);
                    //=== CodeBox1 関連イベント処理 ================================
                    CodeBox_ActionCheckAndExe(ref CodeBox1);

                    //=== RunBtn0 関連イベント処理 ================================
                    RunBtn0_ActionCheckAndExe();

                    //=== StopBtn0 関連イベント処理 ================================
                    StopBtn0_ActionCheckAndExe();

                    //=== StepBtn0 関連イベント処理 ================================
                    StepBtn0_ActionCheckAndExe();

                    //=== MemFill/MemDoenLoad/MemUpLoad 関連イベント処理 ================================
                    MemFill_ActionCheckAndExe();
                    MemDownLoad_ActionCheckAndExe();
                    MemUpLoad_ActionCheckAndExe();

                    //=== PerformanceMonitor 関連イベント処理 ================================
                    PerformanceMonitor();
                    //=== 状態表示 ================
                    SystemTimeCount = SystemTimeCount + 1;
                    if (SystemTimeCount >= 20) SystemTimeCount = 0;
                    if (CurrentProsessorStatus == DbgCom.PROCESSOR_RUN)
                    {
                        DbgComStatusLabel0.Text = "RUN";
                        if (SystemTimeCount == 10) DbgComStatusLabel0.ForeColor = System.Drawing.Color.White;
                        if (SystemTimeCount == 0) DbgComStatusLabel0.ForeColor = System.Drawing.Color.Green;
                        if (SystemTimeCount == 0) DbgComStatusLabel0.BackColor = System.Drawing.Color.Green;
                    }
                    else
                    {
                        DbgComStatusLabel0.Text = "BREAK";
                        if (SystemTimeCount == 10) DbgComStatusLabel0.ForeColor = System.Drawing.Color.White;
                        if (SystemTimeCount == 0) DbgComStatusLabel0.ForeColor = System.Drawing.Color.Red;
                        if (SystemTimeCount == 0) DbgComStatusLabel0.BackColor = System.Drawing.Color.Red;
                    }
                }
                //=== ターゲットへのロード =====================================
                LoadFileBody();
            }
            SystemMonitorTimer.Enabled = true;
        }
        //================================================================================================
        //=　メニュー操作（接続）
        //================================================================================================
        private void ConnectMenu_Click(object sender, EventArgs e)
        {
            uint Tmp = 0;
            if (DCom.MemRead(0x0c000000, ref Tmp) == P3DBG_OK)
            {
                RunBtn0.Enabled = true;
                StopBtn0.Enabled = true;
                StepBtn0.Enabled = true;
                FillBtn.Enabled = true;
                DownLoadBtn.Enabled = true;
                UpLoadBtn.Enabled = true;
                RegBox0.RegRefreshBtn.Enabled = true;
                RegBox1.RegRefreshBtn.Enabled = true;
                MemBox0.MemRefreshBtn.Enabled = true;
                MemBox1.MemRefreshBtn.Enabled = true;
                MemBox2.MemRefreshBtn.Enabled = true;
                MemBox3.MemRefreshBtn.Enabled = true;
                LoadFileMenu.Enabled = true;
                ReadMapFile.Enabled = true;
                DbgComStatusLabel0.Text = "接続中";
                DbgComStatusLabel0.BackColor = System.Drawing.Color.Black;
                RegBox0.RefreshFlag = 1;
                RegBox1.RefreshFlag = 1;
                MemBox0.RefreshFlag = 1;
                MemBox1.RefreshFlag = 1;
                MemBox2.RefreshFlag = 1;
                MemBox3.RefreshFlag = 1;
                CodeBox0.ScrollFlag = 1;
                CodeBox1.ScrollFlag = 1;
                ConnectEnable = 1;
            }
            else
            {
                ErrMsgBox.Show();
            }
        }
        //================================================================================================
        //=　COMポートの設定
        //================================================================================================
        private void ComTextBox_TextChanged(object sender, EventArgs e)
        {
            DCom.ComPortNo = ComTextBox.Text;
        }
        //================================================================================================
        //=　共通関数
        //================================================================================================
        //=　文字列を数値に変換
        private uint Char2Num(byte a)
        {
            if ((a & 0xF0) == 0x30) return (uint)(a & 0xF);
            return (uint)((a + 9) & 0xF);
        }
        private uint Str2Num(string TargerStr)
        {
            uint Tmp;
            int Len;
            int i;
            byte[] Str = Encoding.ASCII.GetBytes(TargerStr);
            Len = Str.Length;
            if (Len > 2)
            {
                if ((Str[0] == '0') && (Str[1] == 'x'))
                {
                    Tmp = 0;
                    for (i = 2; i < Len; i++)
                    {
                        Tmp = Tmp << 4;
                        Tmp = Tmp | Char2Num(Str[i]);
                    }
                }
                else if ((Str[0] == ' '))
                {
                    Tmp = P3DBG_ILG;
                }
                else
                {
                    if (Str[0] == '-') Tmp = (uint)Convert.ToInt32(TargerStr);
                    else Tmp = Convert.ToUInt32(TargerStr);
                }
            }
            else
            {
                if (Str[0] == '-') Tmp = (uint)Convert.ToInt32(TargerStr);
                else Tmp = Convert.ToUInt32(TargerStr);
            }
            return Tmp;
        }
        //=　MsgTextBox書き込み
        delegate void PrintMsgTextBoxCallBack(string text, uint flag);
        private void PrintMsgTextBox(string text, uint flag)
        {
            if (MsgTextBox.InvokeRequired == true)
            {
                PrintMsgTextBoxCallBack d = new PrintMsgTextBoxCallBack(PrintMsgTextBox);
                object[] obj = { text, flag };
                Invoke(d, obj);
            }
            else
            {
                if (flag == 0) MsgTextBox.Text = text;
                else MsgTextBox.AppendText(text);
            }
        }
        //=　MsgBoxのラベルへの書き込み
        delegate void PrintMsgBoxLabelCallBack(string text);
        private void PrintMsgBoxLabel(string text)
        {
            if (MsgBox.Msglabel.InvokeRequired == true)
            {
                PrintMsgBoxLabelCallBack d = new PrintMsgBoxLabelCallBack(PrintMsgBoxLabel);
                object[] obj = { text };
                Invoke(d, obj);
            }
            else
            {
                MsgBox.Msglabel.Text = text;
            }
        }
        //================================================================================================
        //=　いろいろテスト用のボタン
        //================================================================================================
        //========================================
        //=　DDRテスト
        //========================================
        private void DdrTestBtn_Click(object sender, EventArgs e)
        {
            DCom.DbgOpen();
            backgroundWorker1.RunWorkerAsync();
        }
        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            ConnectEnable = 0;
            System.Threading.Thread.Sleep(100);
            DdrTest();
            ConnectEnable = 1;
        }
        private void DdrTest()
        {
            uint i;
            uint j;
            uint Data;
            uint Offset;
            uint Num;
            uint[] WriteData = new uint[2048];
            uint[] ReadData = new uint[2048];
            uint NumOfError;
            string str; 

            //Data = 0xAAAABBBB;
            Data = 0x11112222;
            Num = 256;
            str = "";
            NumOfError = 0;
            for (i = 0; i < 100; i++)
            {
                Offset = (i * 2048 * 4);
                str = "========= " + String.Format("{0:X8}", Offset) + " =========  " + i.ToString();
                PrintMsgTextBox(str + Environment.NewLine, 1);
                str = "WRITE";
                PrintMsgTextBox(str + Environment.NewLine, 1);
                //System.Threading.Thread.Sleep(1000);
                for (j = 0; j < Num; j++)
                {
                    WriteData[j] = Data;
                    Data = Data + 0x01010101;
                }
                DCom.MemBlockWrite(Offset, Num, ref WriteData);
                WriteData[j] = Data;

                str = "READ";
                PrintMsgTextBox(str + Environment.NewLine, 1);
                //System.Threading.Thread.Sleep(100);
                //for (j = 0; j < Num; j++)
                //{
                //    DCom.MemRead(j * 4 + Offset, ref RData);
                //    ReadData[j] = RData;
                //}
                //DCom.MemBlockRead(Offset, Num, ref ReadData);
                DCom.MemBlockRead3(Offset, Num, ref ReadData);

                for (j = 0; j < Num; j++)
                {
                    if (WriteData[j] != ReadData[j])
                    {
                        ++NumOfError;
                        str = "ERROR: A:";
                        str += String.Format("{0:X8}", Offset + (j * 4));
                        str += " EXP:" + String.Format("{0:X8}", WriteData[j]);
                        str += " RES:" + String.Format("{0:X8}", ReadData[j]);
                        str += Environment.NewLine;
                        PrintMsgTextBox(str + Environment.NewLine, 1);
                    }
                }
            }
            str = "TOTAL ERROR =" + NumOfError.ToString() + Environment.NewLine;
            PrintMsgTextBox(str + Environment.NewLine, 1);
        }

    }
}
