﻿//=======================================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  FUNCTION      : Debugger
//
//                      Copyright (C) 2008 by Tomoyuki Inomoto
//=======================================================================
using System;
using System.Collections.Generic;
using System.Text;

namespace Psm3Debugger
{
    class P3IAsm
    {
        private const uint JP_INST_BIT= 0x80000000;
        private const uint JP_INST_MASK = 0xE0000000;
        private const uint JP_INST = 0x00000000;
        private const uint CALL_INST = 0x20000000;
        private const uint DJP_INST = 0x60000000;
        private const uint RTI_INST = 0x40000000;

        private const uint COND_CODE_MASK = 0x0F000000;
        private const uint COND_AL = 0x00000000;
        private const uint COND_EQ = 0x01000000;
        private const uint COND_NE = 0x02000000;
        private const uint COND_HS = 0x03000000;
        private const uint COND_LO = 0x04000000;
        private const uint COND_MI = 0x05000000;
        private const uint COND_PL = 0x06000000;
        private const uint COND_VS = 0x07000000;
        private const uint COND_VC = 0x08000000;
        private const uint COND_HI = 0x09000000;
        private const uint COND_LS = 0x0A000000;
        private const uint COND_GE = 0x0B000000;
        private const uint COND_LT = 0x0C000000;
        private const uint COND_GT = 0x0D000000;
        private const uint COND_LE = 0x0E000000;

        private const uint LDST_INST_BIT = 0xC0000000;
        private const uint ST_BIT = 0x20000000;
        private const uint IMM_BIT = 0x10000000;
        private const uint LDST_TYPE_MASK = 0x0F000000;

        private const uint LDW_BIT   = 0x00000000;
        private const uint LDWUB_BIT = 0x01000000;
        private const uint LDWUA_BIT = 0x02000000;
        private const uint LDH_BIT   = 0x03000000;
        private const uint LDHUB_BIT = 0x04000000;
        private const uint LDHUA_BIT = 0x05000000;
        private const uint LDB_BIT   = 0x06000000;
        private const uint LDBUB_BIT = 0x07000000;
        private const uint LDBUA_BIT = 0x08000000;
        private const uint LDSH_BIT   = 0x09000000;
        private const uint LDSHUB_BIT = 0x0A000000;
        private const uint LDSHUA_BIT = 0x0B000000;
        private const uint LDSB_BIT   = 0x0C000000;
        private const uint LDSBUB_BIT = 0x0D000000;
        private const uint LDSBUA_BIT = 0x0E000000;

        private const uint STW_BIT = 0x00000000;
        private const uint STWUB_BIT = 0x01000000;
        private const uint STWUA_BIT = 0x02000000;
        private const uint STH_BIT = 0x03000000;
        private const uint STHUB_BIT = 0x04000000;
        private const uint STHUA_BIT = 0x05000000;
        private const uint STB_BIT = 0x06000000;
        private const uint STBUB_BIT = 0x07000000;
        private const uint STBUA_BIT = 0x08000000;

        private const uint CALC_INST_BIT = 0xE0000000;
        private const uint FUNC_TYPE_MASK = 0x0F000000;

        private const uint FUNC_MOV  = 0x00000000;
        private const uint FUNC_MOVL = 0x01000000;
        private const uint FUNC_MOVH = 0x02000000;
        private const uint FUNC_ADD  = 0x03000000;
        private const uint FUNC_SUB  = 0x04000000;
        private const uint FUNC_AND  = 0x05000000;
        private const uint FUNC_OR   = 0x06000000;
        private const uint FUNC_XOR  = 0x07000000;
        private const uint FUNC_LSR  = 0x08000000;
        private const uint FUNC_LSL  = 0x09000000;
        private const uint FUNC_ASR  = 0x0A000000;
        private const uint FUNC_MUL  = 0x0B000000;
        private const uint FUNC_DIVU = 0x0C000000;
        private const uint FUNC_DIV  = 0x0D000000;
        private const uint FUNC_MODU = 0x0E000000;
        private const uint FUNC_MOD  = 0x0F000000;

        private const uint SYS_INST_BIT = 0xF0000000;
        private const uint SYS_INST_TYPE_MASK = 0xFF000000;

        private const uint SYS_INST_DI = 0xE0000000;
        private const uint SYS_INST_EI = 0xE8000000;
        private const uint SYS_INST_DIC = 0xE1000000;
        private const uint SYS_INST_EIC = 0xE9000000;
        private const uint SYS_INST_DDC = 0xE2000000;
        private const uint SYS_INST_EDC = 0xEA000000;
        private const uint SYS_INST_BULK = 0xE3000000;
        private const uint SYS_INST_BLK = 0xEB000000;

        private const uint CP_INST_BIT  = 0xF8000000;
        private const uint CP_INST_MASK = 0xFF000000;
        private const uint CPRD_INST   = 0xF0000000;
        private const uint CPWR_INST   = 0xF1000000;

        private const uint BREAK_INST_BIT = 0xFC000000;
        private const uint BREAK_INST      = 0xF8000000;

        private const uint NOP_INST_MASK = 0xFC000000;
        private const uint NOP_INST      = 0xFC000000;



        //================================================================================================
        //=　コンストラクタ
        //================================================================================================
        public P3IAsm()
        {
            ;
        }
        //================================================================================================
        //=　メソッド
        //================================================================================================
        //== 条件コード取得 ===============================
        private String GetConditionCode(uint Hex)
        {
            uint Tmp;
            Tmp = Hex & COND_CODE_MASK;
            if (Tmp == COND_EQ) return "EQ";
            if (Tmp == COND_NE) return "NE";
            if (Tmp == COND_HS) return "HS";
            if (Tmp == COND_LO) return "LO";
            if (Tmp == COND_MI) return "MI";
            if (Tmp == COND_PL) return "PL";
            if (Tmp == COND_VS) return "VS";
            if (Tmp == COND_VC) return "VC";
            if (Tmp == COND_HI) return "HI";
            if (Tmp == COND_LS) return "LS";
            if (Tmp == COND_GE) return "GE";
            if (Tmp == COND_LT) return "LT";
            if (Tmp == COND_GT) return "GT";
            if (Tmp == COND_LE) return "LE";
            return "AL";
        }
        //== Imm24取得 ===============================
        private String GetImm24(uint Adr, uint Hex)
        {
            uint Tmp,Tmp2;
            Tmp2 = 0;
            Tmp = (Hex & 0x00FFFFFF);
            if ((Hex & 0x00800000) != 0) Tmp2 = Tmp | 0xFF000000;
            Tmp2 = Tmp2 << 2;
            if ((Hex & 0x00800000) == 0)//正の数
            {
                Tmp2 = Tmp2 + Adr;
            }
            else
            {
                Tmp2 = ~Tmp2;
                Tmp2 += 1;
                Tmp2 = Adr - Tmp2;
            }
            return ", " + String.Format("0x{0:X8}", Tmp) + " (" + String.Format("0x{0:X8}", Tmp2) + ")";
        }
        //== GetJumpIAsm ===============================
        private String GetJumpIAsm(uint Adr, uint Hex)
        {
            String TmpRes;

            if ((Hex & JP_INST_MASK) == JP_INST)
            {
                TmpRes = "JP, ";
                TmpRes += GetConditionCode(Hex) + ", ";
                if((Hex & IMM_BIT) != 0) TmpRes += GetImm24(Adr, Hex);
                else TmpRes += GetReg((Hex & 0x00F00000) >> 20);
                return TmpRes;
            }
            else if ((Hex & JP_INST_MASK) == CALL_INST)
            {
                TmpRes = "CALL, ";
                TmpRes += GetConditionCode(Hex) + ", ";
                if ((Hex & IMM_BIT) != 0) TmpRes += GetImm24(Adr, Hex);
                else TmpRes += GetReg((Hex & 0x00F00000) >> 20);
                return TmpRes;
            }
            else if ((Hex & JP_INST_MASK) == DJP_INST)
            {
                if ((Hex & 0x00F00000) == 0x00B00000)
                {
                    TmpRes = "RTS, ";
                    TmpRes += GetConditionCode(Hex);
                }
                else
                {
                    TmpRes = "DJP, ";
                    TmpRes += GetConditionCode(Hex) + ", ";
                    TmpRes += GetReg((Hex & 0x00F00000) >> 20);
                }
                return TmpRes;
            }
            else
            {
                TmpRes = "RTI, ";
                TmpRes += GetConditionCode(Hex);
                return TmpRes;
            }

        }
        //== GetLdOpecode ===============================
        private String GetLdOpecode(uint Hex)
        {
            uint Tmp;
            Tmp = Hex & LDST_TYPE_MASK;
            if (Tmp == LDW_BIT) return "LDW, ";
            else if (Tmp == LDW_BIT)    return "LDW, ";
            else if (Tmp == LDWUB_BIT)  return "LDWUB, ";
            else if (Tmp == LDWUA_BIT)  return "LDWUA, ";
            else if (Tmp == LDH_BIT)    return "LDH, ";
            else if (Tmp == LDHUB_BIT)  return "LDHUB, ";
            else if (Tmp == LDHUA_BIT)  return "LDHUA, ";
            else if (Tmp == LDB_BIT)    return "LDB, ";
            else if (Tmp == LDBUB_BIT)  return "LDBUB, ";
            else if (Tmp == LDBUA_BIT)  return "LDBUA, ";
            else if (Tmp == LDSH_BIT)   return "LDSH, ";
            else if (Tmp == LDSHUB_BIT) return "LDSHUB, ";
            else if (Tmp == LDSHUA_BIT) return "LDSHUA, ";
            else if (Tmp == LDSB_BIT)   return "LDSB, ";
            else if (Tmp == LDSBUB_BIT) return "LDSBUB, ";
            else if (Tmp == LDSBUA_BIT) return "LDSBUA, ";
            else return "???, ";
        }
        //== GetStOpecode ===============================
        private String GetStOpecode(uint Hex)
        {
            uint Tmp;
            Tmp = Hex & LDST_TYPE_MASK;
            if (Tmp == STW_BIT) return "STW, ";
            else if (Tmp == STW_BIT) return "STW, ";
            else if (Tmp == STWUB_BIT) return "STWUB, ";
            else if (Tmp == STWUA_BIT) return "STWUA, ";
            else if (Tmp == STH_BIT) return "STH, ";
            else if (Tmp == STHUB_BIT) return "STHUB, ";
            else if (Tmp == STHUA_BIT) return "STHUA, ";
            else if (Tmp == STB_BIT) return "STB, ";
            else if (Tmp == STBUB_BIT) return "STBUB, ";
            else if (Tmp == STBUA_BIT) return "STBUA, ";
            else return "???, ";
        }
        //== GetReg ===============================
        private String GetReg(uint Reg)
        {
            if (Reg == 0) return "R0";
            else if (Reg == 1) return "R1";
            else if (Reg == 2) return "R2";
            else if (Reg == 3) return "R3";
            else if (Reg == 4) return "R4";
            else if (Reg == 5) return "R5";
            else if (Reg == 6) return "R6";
            else if (Reg == 7) return "R7";
            else if (Reg == 8) return "R8(GBR)";
            else if (Reg == 9) return "R9(FP)";
            else if (Reg == 10) return "R10(SP)";
            else if (Reg == 11) return "R11(LR)";
            else if (Reg == 12) return "R12(ILR)";
            else if (Reg == 13) return "ISR";
            else if (Reg == 14) return "SR";
            else  return "PC";
        }
        //== GetLdStIAsm ===============================
        private String GetLdStIAsm(uint Hex)
        {
            String TmpRes;
            if ((Hex & ST_BIT) == 0) // LD命令
            {
                TmpRes = GetLdOpecode(Hex);
            }
            else // ST命令
            {
                TmpRes = GetStOpecode(Hex);
            }
            TmpRes += GetReg((Hex & 0x00F00000) >> 20);
            TmpRes += ", ";
            TmpRes += GetReg((Hex & 0x000F0000) >> 16);
            TmpRes += ", ";
            if ((Hex & IMM_BIT) == 0)
            {
                TmpRes += GetReg((Hex & 0xF));
            }
            else
            {
                TmpRes += String.Format("0x{0:X4}", Hex & 0xFFFF);
            }
            return TmpRes;

        }
        //== GetCalcOpCode ===============================
        private String GetCalcOpCode(uint Hex)
        {
            uint Tmp;
            Tmp = Hex & FUNC_TYPE_MASK;
            if (Tmp == FUNC_MOV) return "MOV, ";
            else if (Tmp == FUNC_MOVL) return "MOVL,";
            else if (Tmp == FUNC_MOVH) return "MOVH,";
            else if (Tmp == FUNC_ADD) return "ADD, ";
            else if (Tmp == FUNC_SUB) return "SUB, ";
            else if (Tmp == FUNC_AND) return "AND, ";
            else if (Tmp == FUNC_OR) return "OR,  ";
            else if (Tmp == FUNC_XOR) return "XOR, ";
            else if (Tmp == FUNC_LSR) return "LSR, ";
            else if (Tmp == FUNC_LSL) return "LSL, ";
            else if (Tmp == FUNC_ASR) return "ASR, ";
            else if (Tmp == FUNC_MUL) return "MUL, ";
            else if (Tmp == FUNC_DIVU) return "DIVU, ";
            else if (Tmp == FUNC_DIV) return "DIV, ";
            else if (Tmp == FUNC_MODU) return "MODU, ";
            else if (Tmp == FUNC_MOD) return "MOD, ";
            else return "???, ";
        }
        //== GetCalcIAsm ===============================
        private String GetCalcIAsm(uint Hex)
        {
            String TmpRes;

            TmpRes = GetCalcOpCode(Hex);
            TmpRes += GetReg((Hex & 0x00F00000) >> 20);
            TmpRes += ", ";
            if(((Hex & FUNC_TYPE_MASK) != FUNC_MOV) && ((Hex & FUNC_TYPE_MASK) != FUNC_MOVH) && ((Hex & FUNC_TYPE_MASK) != FUNC_MOVL)){
                TmpRes += GetReg((Hex & 0x000F0000) >> 16);
                TmpRes += ", ";
            }
            if ((Hex & IMM_BIT) == 0)
            {
                TmpRes += GetReg((Hex & 0xF));
            }
            else
            {
                TmpRes += String.Format("0x{0:X4}", Hex & 0xFFFF);
            }
            return TmpRes;

        }
        //== GetSysIAsm ===============================
        private String GetSysIAsm(uint Hex)
        {
            uint Tmp;
            Tmp = Hex & SYS_INST_TYPE_MASK;
            if (Tmp == SYS_INST_DI) return "DI";
            else if (Tmp == SYS_INST_EI) return "EI";
            else if (Tmp == SYS_INST_DIC) return "DIC";
            else if (Tmp == SYS_INST_EIC) return "EIC";
            else if (Tmp == SYS_INST_DDC) return "DDC";
            else if (Tmp == SYS_INST_EDC) return "EDC";
            else if (Tmp == SYS_INST_BULK) return "BULK";
            else if (Tmp == SYS_INST_BLK) return "BLK";
            else return "???";
        }
        //== GetCpIAsm ===============================
        private String GetCpIAsm(uint Hex)
        {
            uint Tmp;
            String TmpRes;

            Tmp = Hex & CP_INST_MASK;
            if (Tmp == CPRD_INST) TmpRes = "CPRD, ";
            else                  TmpRes = "CPWR, ";
            TmpRes += GetReg((Hex & 0x00F00000) >> 20);
            TmpRes += ", ";
            TmpRes += GetReg((Hex & 0x000F0000) >> 16);
            TmpRes += ", ";
            TmpRes += String.Format("0x{0:X4}", ((Hex >> 12) & 0xF));
            TmpRes += String.Format("0x{0:X4}", ((Hex >> 8) & 0xF));
            TmpRes += String.Format("0x{0:X4}", ((Hex >> 4) & 0xF));
            TmpRes += GetReg(Hex & 0xF);
            return TmpRes;
        }
        //================================================================================================
        //=　逆アセ本体
        //================================================================================================
        public uint IAsm(uint Adr, uint Hex, ref String Res)
        {

            if ((Hex & JP_INST_BIT) == 0) //======== 分岐命令 =============
            {
                Res = GetJumpIAsm(Adr, Hex);
            }
            else if ((Hex & LDST_INST_BIT) == 0x80000000) //======== LD/ST命令 =============
            {
                Res = GetLdStIAsm(Hex);
            }
            else if ((Hex & CALC_INST_BIT) == 0xC0000000) //======== 演算命令 =============
            {
                Res = GetCalcIAsm(Hex);
            }
            else if ((Hex & SYS_INST_BIT) == 0xE0000000) //======== SYS制御 =============
            {
                Res = GetSysIAsm(Hex);
            }
            else if ((Hex & CP_INST_BIT) == 0xF0000000) //======== SYS制御 =============
            {
                Res = GetCpIAsm(Hex);
            }
            else if ((Hex & BREAK_INST_BIT) == BREAK_INST) //======== SYS制御 =============
            {
                Res = "BREAK";
            }
            else
            {
                Res = "NOP";
            }
            return 0;
        }

    }
}
